/*
 * Copyright (c) 2001, Silver Egg Technology Co., Ltd.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * $Id: NodeAdditionListener.java,v 1.1 2001/04/18 17:39:48 djvanmaren Exp $
 */

package net.sourceforge.e4graph;

import java.util.EventListener;

/**
 * The listener interface for receiving node addition events. The class that is 
 * interested in processing a node addition event implements this interface, 
 * and the object created with that class is registered with a storage, using 
 * the storage's <tt>addNodeAdditionListener</tt> method. When the node addition 
 * event occurs, that object's <tt>nodeAdded</tt> method is invoked.
 *
 * @see Storage#addNodeAdditionListener
 *
 * @author <a href="mailto:ryan@silveregg.co.jp">Ryan Shaw</a>
 */
public interface NodeAdditionListener extends EventListener {

    /**
     * Invoked when a node is added to the storage with which this listener 
     * is registered.
     *
     * @param m the node that was added
     */
    void nodeAdded(Node n);

}// NodeAdditionListener
