/*
 * Copyright (c) 2001, Silver Egg Technology Co., Ltd.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * $Id: VertexFilter.java,v 1.1 2001/04/18 17:39:48 djvanmaren Exp $
 */

package net.sourceforge.e4graph;

/**
 * Vertex filters are objects that know how to "filter out" vertices.
 * A vertex filter can be passed to the <tt>vertices</tt> method of 
 * the <tt>Node</tt> class an iteration only over a filtered subset
 * of vertices is desired.
 *
 * @see Node#vertices(VertexFilter)
 * @see Node#vertices(Vertex, VertexFilter)
 *
 * @author <a href="mailto:ryan@silveregg.co.jp">Ryan Shaw</a>
 */
public interface VertexFilter {
    
    /**
     * Returns true if the filter accepts the given vertex; false otherwise.
     *
     * @param v the vertex to test
     * @return true if the filter accepts the given vertex; false otherwise
     */
    boolean accept(Vertex v);
    
}// VertexFilter
