/*
 * e4drivers.h --
 *
 *	This file defines the class e4_Drivers that manages the
 *	registration of storage creation functions and drivers.
 *
 * Copyright (c) 2000-2001, JYL Software Inc.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE, EVEN IF
 * JYL SOFTWARE INC. IS MADE AWARE OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__E4_DRIVERS_H__
#define	__E4_DRIVERS_H__

/*
 * This defines the type of driver construction functions:
 */

typedef e4_StorageImpl *(*e4_StorageConstructor)(const char *);

/*
 * The Initialize method initializes and pre-registers all the
 * built-in drivers. You can call this many times, it protects
 * against re-initialization.
 */

e4_DLL void e4_InitializeStorageRegistry();

/*
 * Given a driver name get its registered storage constructor.
 */

e4_DLL e4_StorageConstructor e4_GetStorageConstructor(const char *drivername);

/*
 * Register a storage constructor function so that it is retrievable
 * given a specified name.
 */

e4_DLL bool e4_RegisterStorageConstructor(e4_StorageConstructor scp,
					  const char *drivername);

/*
 * Deregister a storage construction function.
 */

e4_DLL bool e4_DeregisterStorageConstructor(const char *drivername);

/*
 * Is there a driver registered under the given name?
 */

e4_DLL bool e4_NamedDriverExists(const char *drivername);

/*
 * These are the names of the built-in drivers:
 */

#define E4_METAKIT	"Metakit 2.0"

#endif	/* __E4_DRIVERS_H__ */
