/*
 * e4metacache.h --
 *
 *	This file provides defines for constants used in the meta-information
 *	cache maintained per storage.
 *
 * Copyright (c) 2000-2001, JYL Software Inc.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE, EVEN IF
 * JYL SOFTWARE INC. IS MADE AWARE OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__E4_METACACHE_H__
#define	__E4_METACACHE_H__

/*
 * The size of keys for the meta information cache:
 */

#define E4_META_KEYS		3

/*
 * This structure defines a key for meta information.
 */

typedef struct e4_MetaKey {
    int		nodeID;
    int		nameID;
    int		rankOrNth;
} e4_MetaKey;

/*
 * This structure defines the tuple stored for nameID-nth keys:
 */

typedef struct e4_VertexAndRank {
    int		vertexID;
    int		rank;
} e4_VertexAndRank;

/*
 * Keys for meta information consist of up to three elements:
 *
 * 1. Node ID.
 * 2. Name ID.
 * 3. Nth or rank.
 *
 * Most keys are of one of these two forms:
 *
 * {NodeID, NameID, nth}	The value is a tuple <vertexID, rank> for
 *				the vertex whose name is the nth occurrence
 *				of the name nameID in the node nodeID.
 * {NodeID, E4_RANKKEY, rank}	The value is the vertexID for the
 *				vertex whose rank is rank in the node
 *				nodeID.
 *
 * There are also a few special purpose keys, explained below.
 */

/*
 * This value in the NameID denotes the key is a rank key. The key is
 * of the form:
 *
 * {NodeID, E4_RANKKEY, rank}	The value is the vertexID for the
 *				vertex whose rank is rank in the node
 *				nodeID
 */

#define E4_RANKKEY		-1

/*
 * This value in the NameID denotes the key is a nodeID caching
 * sentinel. It is used simply to denote that some meta information
 * is cached about this node.
 */

#define E4_SENTINELKEY		-2

#endif	/* __E4_METACACHE_H__ */
