# util.tcl --
#
# Utilities used during testing TGraph:
#
# Copyright (c) 2000-2001, JYL Software Inc.
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE, EVEN IF
# JYL SOFTWARE INC. IS MADE AWARE OF THE POSSIBILITY OF SUCH DAMAGE.

# Ensure that the tgraph extension is loaded:

proc findLibraryExtensionAndPrefix {} {
    global tcl_platform

    if {[string equal windows $tcl_platform(platform)]} {
	return {"" ".dll"}
    }
    if {[string equal unix $tcl_platform(platform)]} {
	return {"lib" ".so"}
    }
    return {"" ".unknown"}
}

proc loadTGraph {} {
    set prefext [findLibraryExtensionAndPrefix]
    set libname [lindex $prefext 0]tgraph[lindex $prefext 1]
    load $libname
}

# Ensure that the named storage is empty and open for use:

proc ensureFreshStorage {name} {
    [tgraph::open $name] delete
    return [tgraph::open $name]
}

# Delete a storage and cause an error:

proc deleteAndError {storage msg} {
    $storage delete
    return -code error $msg
}

# Print an indication of activity:

proc showActive {} {
    puts -nonewline stderr "."
    flush stderr
}
