/*
 * testdelete.cpp --
 *
 * Tests of the deletion and callback facility.
 *
 * Copyright (c) 2000-2001, JYL Software Inc.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE, EVEN IF
 * JYL SOFTWARE INC. IS MADE AWARE OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#ifndef	_WIN32
#include <unistd.h>
#endif
#include "e4graph.h"
#include "test.h"

static int
test_delete1()
{
    e4_Marker m;
    e4_Node n;
    e4_Vertex v;
    int rank;

    /*
     * Test deletion of a vertex, a node and a marker.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test delete1 failed (step 1)\n");
	    return 1;
	}
	if (!s.GetMarker("m", m) || !m.IsValid()) {
	    fprintf(stderr, "test delete1 failed (step 2)\n");
	    return 1;
	}
	if (!m.Delete() || m.IsValid()) {
	    fprintf(stderr, "test delete1 failed (step 3)\n");
	    return 1;
	}
	if (!s.GetMarker("m", m) || !m.IsValid()) {
	    fprintf(stderr, "test delete1 failed (step 4)\n");
	    return 1;
	}
	if (!m.GetMarkedNode(n) || !n.IsValid()) {
	    fprintf(stderr, "test delete1 failed (step 5)\n");
	    return 1;
	}
	if (!n.Delete() || n.IsValid() || m.IsValid()) {
	    fprintf(stderr, "test delete1 failed (step 6)\n");
	    return 1;
	}
	if (!s.GetMarker("m", m) || !m.IsValid()) {
	    fprintf(stderr, "test delete1 failed (step 7)\n");
	    return 1;
	}
	if (!m.GetMarkedNode(n) || !n.IsValid()) {
	    fprintf(stderr, "test delete1 failed (step 8)\n");
	    return 1;
	}
	if (!s.GetMarker("m", m) || !m.IsValid()) {
	    fprintf(stderr, "test delete1 failed (step 9)\n");
	    return 1;
	}
	if (!m.GetMarkedNode(n) || !n.IsValid()) {
	    fprintf(stderr, "test delete1 failed (step 10)\n");
	    return 1;
	}
	if (!n.AddVertexRef("v", E4_IOLAST, rank, 0, v) || !v.IsValid()) {
	    fprintf(stderr, "test delete1 failed (step 11)\n");
	    return 1;
	}
	if (!v.Delete() || v.IsValid() || !n.IsValid() || !m.IsValid() ||
	    (n.VertexCount() != 0) || (n.OccurrenceCount() != 0) ||
	    (n.ParentCount() != 0) || (n.MarkerCount() != 1)) {
	    fprintf(stderr, "test delete1 failed (step 12)\n");
	    return 1;
	}
	if (!n.Delete() || n.IsValid() || m.IsValid() || v.IsValid()) {
	    fprintf(stderr, "test delete1 failed (step 13)\n");
	    return 1;
	}
	if (!s.IsEmpty() || !s.IsValid()) {
	    fprintf(stderr, "test delete1 failed (step 14)\n");
	    return 1;
	}

	s.Delete();
    }

    return 0;
}

static int
test_delete2()
{
    int rank, i;
    e4_Marker m;
    e4_Node n1, n2;
    e4_Vertex v1, v2;

    /*
     * n2 is a child of n1 and has a vertex v2. n1 has a vertex v1, whose
     * value is the node n2.
     *
     * delete v2, then n2, then check that v1 is invalid and the vertex count
     * of n1 goes down, then delete n1.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test delete2 failed (step 1)\n");
	    return 2;
	}
	if (!s.GetMarker("m", m) || !m.IsValid()) {
	    fprintf(stderr, "test delete2 failed (step 2)\n");
	    return 2;
	}
	if (!m.Delete() || m.IsValid()) {
	    fprintf(stderr, "test delete2 failed (step 3)\n");
	    return 2;
	}
	if (!s.GetMarker("m", m) || !m.IsValid()) {
	    fprintf(stderr, "test delete2 failed (step 4)\n");
	    return 2;
	}
	if (!m.GetMarkedNode(n1) || !n1.IsValid()) {
	    fprintf(stderr, "test delete2 failed (step 5)\n");
	    return 2;
	}
	if (!n1.AddNodeRef("v1", E4_IOFIRST, rank, n2, v1) ||
	    !n2.IsValid() || !v1.IsValid()) {
	    fprintf(stderr, "test delete2 failed (step 6)\n");
	    return 2;
	}
	for (i = 0; i < 10; i++) {
	    if (!n1.AddVertex("foo", E4_IOLAST, rank, i)) {
		fprintf(stderr, "test delete2 failed (step 7.1.%d)\n", i);
		return 2;
	    }
	    if (n1.VertexCount() != i + 2) {
		fprintf(stderr, "test delete2 failed (step 7.2.%d)\n", i);
		return 2;
	    }
	}
	if (!n2.AddVertexRef("v1", E4_IOFIRST, rank, 24, v2) || 
	    !v2.IsValid() || (n2.VertexCount() != 1)) {
	    fprintf(stderr, "test delete2 failed (step 8)\n");
	    return 2;
	}
	if (!v2.Delete() || v2.IsValid() || !n2.IsValid() || !v1.IsValid() ||
	    !n1.IsValid() || (n2.VertexCount() != 0) || 
	    (n1.VertexCount() != 11)) {
	    fprintf(stderr, "test delete2 failed (step 8)\n");
	    return 2;
	}
	if (!n2.Delete() || n2.IsValid() || v1.IsValid() || !n1.IsValid() ||
	    (n1.VertexCount() != 10)) {
	    fprintf(stderr, "test delete2 failed (step 9)\n");
	    return 2;
	}
	if (!n1.Delete() || n1.IsValid() || m.IsValid()) {
	    fprintf(stderr, "test delete2 failed (step 10)\n");
	    return 2;
	}
	if (!s.IsValid() || !s.IsEmpty()) {
	    fprintf(stderr, "test delete2 failed (step 11)\n");
	    return 2;
	}

	s.Delete();
    }

    return 0;
}

static int
test_delete3()
{
    int rank, i;
    e4_Marker m;
    e4_Node n1;
    e4_Node children[10];

    /*
     * n1 is a node and has ten child nodes.
     *
     * Delete the five first children, check that the remaining children
     * are valid, then delete n1 and check that all children are invalid.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test delete3 failed (step 1)\n");
	    return 3;
	}
	if (!s.GetMarker("m", m) || !m.IsValid()) {
	    fprintf(stderr, "test delete3 failed (step 2)\n");
	    return 3;
	}
	if (!m.Delete() || m.IsValid()) {
	    fprintf(stderr, "test delete3 failed (step 3)\n");
	    return 3;
	}
	if (!s.GetMarker("m", m) || !m.IsValid()) {
	    fprintf(stderr, "test delete3 failed (step 4)\n");
	    return 3;
	}
	if (!m.GetMarkedNode(n1) || !n1.IsValid()) {
	    fprintf(stderr, "test delete3 failed (step 5)\n");
	    return 3;
	}
	for (i = 0; i < 10; i++) {
	    if (!n1.AddNode("foo", E4_IOLAST, rank, children[i]) ||
		!children[i].IsValid()) {
		fprintf(stderr, "test delete3 failed (step 6.%d)\n", i);
		return 3;
	    }
	}
	if (!n1.IsValid() || (n1.VertexCount() != 10)) {
	    fprintf(stderr, "test delete3 failed (step 7)\n");
	    return 3;
	}
	for (i = 0; i < 5; i++) {
	    if (!children[i].Delete() || children[i].IsValid() ||
		!n1.IsValid() || (n1.VertexCount() != 9 - i)) {
		fprintf(stderr, "test delete3 failed (step 8.%d)\n", i);
		return 3;
	    }
	}
	for (i = 5; i < 10; i++) {
	    if (!children[i].IsValid()) {
		fprintf(stderr, "test delete3 failed (step 9.%d)\n", i);
		return 3;
	    }
	}
	if (!n1.Delete() || n1.IsValid() || m.IsValid()) {
	    fprintf(stderr, "test delete3 failed (step 10)\n");
	    return 3;
	}
	for (i = 0; i < 10; i++) {
	    if (children[i].IsValid()) {
		fprintf(stderr, "test delete3 failed (step 11.%d)\n", i);
		return 3;
	    }
	}
	if (!s.IsValid() || !s.IsEmpty()) {
	    fprintf(stderr, "test delete3 failed (step 12)\n");
	    return 3;
	}

	s.Delete();
    }

    return 0;
}

static int
test_delete4()
{
    int i, rank;
    e4_Marker m;
    e4_Node n1, n2;

    /*
     * n2 occurs in n1 ten times. Delete n2 and see if the vertex count
     * of n1 drops by ten.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test delete4 failed (step 1)\n");
	    return 4;
	}
	if (!s.GetMarker("m", m) || !m.IsValid()) {
	    fprintf(stderr, "test delete4 failed (step 2)\n");
	    return 4;
	}
	if (!m.Delete() || m.IsValid()) {
	    fprintf(stderr, "test delete4 failed (step 3)\n");
	    return 4;
	}
	if (!s.GetMarker("m", m) || !m.IsValid()) {
	    fprintf(stderr, "test delete4 failed (step 4)\n");
	    return 4;
	}
	if (!m.GetMarkedNode(n1) || !n1.IsValid()) {
	    fprintf(stderr, "test delete4 failed (step 5)\n");
	    return 4;
	}
	if (!n1.AddNode("foo", E4_IOLAST, rank, n2) || !n2.IsValid()) {
	    fprintf(stderr, "test delete4 failed (step 6)\n");
	    return 4;
	}
	for (i = 1; i < 10; i++) {
	    if (!n1.AddVertex("foo", E4_IOLAST, rank, n2) ||
		(n1.VertexCount() != i + 1)) {
		fprintf(stderr, "test delete4 failed (step 6.1.%d)\n", i);
		return 4;
	    }
	    if ((n2.OccurrenceCount() != i + 1) ||
		(n2.OccurrenceCount(n1) != i + 1)) {
		fprintf(stderr, "test delete4 failed (step 6.2.%d)\n", i);
		return 4;
	    }
	}
	for (i = 0; i < 10; i++) {
	    if (!n1.AddVertex("bar", E4_IOLAST, rank, i) ||
		(n1.VertexCount() != i + 11)) {
		fprintf(stderr, "test delete4 failed (step 7.%d)\n", i);
		return 4;
	    }
	}
	if (!n2.Delete() || n2.IsValid() || !n1.IsValid() ||
	    (n1.VertexCount() != 10)) {
	    fprintf(stderr, "test delete4 failed (step 8)\n");
	    return 4;
	}
	if (!n1.AddNode("foo", E4_IOLAST, rank, n2) || !n2.IsValid()) {
	    fprintf(stderr, "test delete4 failed (step 9)\n");
	    return 4;
	}
	for (i = 1; i < 10; i++) {
	    if (!n1.AddVertex("foo", E4_IOLAST, rank, n2) ||
		(n1.VertexCount() != i + 11)) {
		fprintf(stderr, "test delete4 failed (step 10.1.%d)\n", i);
		return 4;
	    }
	    if ((n2.OccurrenceCount() != i + 1) ||
		(n2.OccurrenceCount(n1) != i + 1)) {
		fprintf(stderr, "test delete4 failed (step 10.2.%d)\n", i);
		return 4;
	    }
	}
	if (!n1.Delete() || n1.IsValid() || n2.IsValid() || m.IsValid()) {
	    fprintf(stderr, "test delete4 failed (step 11)\n");
	    return 4;
	}

	s.Delete();
    }

    return 0;
}

static int
test_delete5()
{
    e4_Marker m1, m2, m3, m4, m5;
    e4_Node n;

    /*
     * The node n1 is marked with five markers. Deleting the node deletes
     * all five markers. Restore the situation and delete each marker in
     * turn -- the node should be deleted only on the deletion of the last
     * marker.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test delete5 failed (step 1)\n");
	    return 5;
	}
	if (!s.GetMarker("m1", m1) || !m1.IsValid()) {
	    fprintf(stderr, "test delete5 failed (step 2)\n");
	    return 5;
	}
	if (!m1.Delete() || m1.IsValid()) {
	    fprintf(stderr, "test delete5 failed (step 3)\n");
	    return 5;
	}
	if (!s.GetMarker("m1", m1) || !m1.IsValid()) {
	    fprintf(stderr, "test delete5 failed (step 4)\n");
	    return 5;
	}
	if (!m1.GetMarkedNode(n) || !n.IsValid()) {
	    fprintf(stderr, "test delete5 failed (step 5)\n");
	    return 5;
	}
	if (!n.AddMarker("m2", m2) || !m2.IsValid() || !n.IsValid() ||
	    (n.MarkerCount() != 2) ||
	    !n.AddMarker("m3", m3) || !m3.IsValid() || !n.IsValid() ||
	    (n.MarkerCount() != 3) ||
	    !n.AddMarker("m4", m4) || !m4.IsValid() || !n.IsValid() ||
	    (n.MarkerCount() != 4) ||
	    !n.AddMarker("m5", m5) || !m5.IsValid() || !n.IsValid() ||
	    (n.MarkerCount() != 5)) {
	    fprintf(stderr, "test delete5 failed (step 6)\n");
	    return 5;
	}
	if (!n.Delete() || n.IsValid() || m1.IsValid() || m2.IsValid() ||
	    m3.IsValid() || m4.IsValid() || m5.IsValid()) {
	    fprintf(stderr, "test delete5 failed (step 7)\n");
	    return 5;
	}
	if (!s.GetMarker("m1", m1) || !m1.IsValid()) {
	    fprintf(stderr, "test delete5 failed (step 8)\n");
	    return 5;
	}
	if (!m1.GetMarkedNode(n) || !n.IsValid()) {
	    fprintf(stderr, "test delete5 failed (step 9)\n");
	    return 5;
	}
	if (!n.AddMarker("m2", m2) || !m2.IsValid() || !n.IsValid() ||
	    (n.MarkerCount() != 2) ||
	    !n.AddMarker("m3", m3) || !m3.IsValid() || !n.IsValid() ||
	    (n.MarkerCount() != 3) ||
	    !n.AddMarker("m4", m4) || !m4.IsValid() || !n.IsValid() ||
	    (n.MarkerCount() != 4) ||
	    !n.AddMarker("m5", m5) || !m5.IsValid() || !n.IsValid() ||
	    (n.MarkerCount() != 5)) {
	    fprintf(stderr, "test delete5 failed (step 10)\n");
	    return 5;
	}
	if (!m5.Delete() || m5.IsValid() || !n.IsValid() || !m1.IsValid() ||
	    !m2.IsValid() || !m3.IsValid() || !m4.IsValid()) {
	    fprintf(stderr, "test delete5 failed (step 11)\n");
	    return 5;
	}
	if (!m4.Delete() || m4.IsValid() || !n.IsValid() || !m1.IsValid() ||
	    !m2.IsValid() || !m3.IsValid()) {
	    fprintf(stderr, "test delete5 failed (step 12)\n");
	    return 5;
	}
	if (!m3.Delete() || m3.IsValid() || !n.IsValid() || !m1.IsValid() ||
	    !m2.IsValid()) {
	    fprintf(stderr, "test delete5 failed (step 13)\n");
	    return 5;
	}
	if (!m2.Delete() || m2.IsValid() || !n.IsValid() || !m1.IsValid()) {
	    fprintf(stderr, "test delete5 failed (step 14)\n");
	    return 5;
	}
	if (!m1.Delete() || m1.IsValid() || n.IsValid()) {
	    fprintf(stderr, "test delete5 failed (step 15)\n");
	    return 5;
	}
	if (!s.IsValid() || !s.IsEmpty()) {
	    fprintf(stderr, "test delete5 failed (step 16)\n");
	    return 5;
	}
	s.Delete();
    }

    return 0;
}

static int
test_delete6()
{
    e4_Marker m;
    e4_Node n1, n2;
    int rank;

    /*
     * n2 is a child of n1 and n1 is a child of n2. Delete n2 and n1 should
     * stay valid because it is marked by the marker m. Recreate the situation,
     * then delete the marker m -- now both n1 and n2 should be invalid.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test delete6 failed (step 1)\n");
	    return 6;
	}
	if (!s.GetMarker("m", m) || !m.IsValid()) {
	    fprintf(stderr, "test delete6 failed (step 2)\n");
	    return 6;
	}
	if (!m.Delete() || m.IsValid()) {
	    fprintf(stderr, "test delete6 failed (step 3)\n");
	    return 6;
	}
	if (!s.GetMarker("m", m) || !m.IsValid()) {
	    fprintf(stderr, "test delete6 failed (step 4)\n");
	    return 6;
	}
	if (!m.GetMarkedNode(n1) || !n1.IsValid()) {
	    fprintf(stderr, "test delete6 failed (step 5)\n");
	    return 6;
	}
	if (!n1.AddNode("foo", E4_IOLAST, rank, n2) || !n2.IsValid()) {
	    fprintf(stderr, "test delete6 failed (step 6)\n");
	    return 6;
	}
	if (!n2.AddVertex("foo", E4_IOLAST, rank, n1) || !n1.IsValid() ||
	    !n2.IsValid() || 
	    (n1.OccurrenceCount() != 1) || (n1.OccurrenceCount(n2) != 1) ||
	    (n2.OccurrenceCount() != 1) || (n2.OccurrenceCount(n1) != 1)) {
	    fprintf(stderr, "test delete6 failed (step 7)\n");
	    return 6;
	}
	if (!n2.Delete() || n2.IsValid() || !n1.IsValid() ||
	    (n1.OccurrenceCount() != 0) || (n1.ParentCount() != 0) ||
	    (n1.MarkerCount() != 1)) {
	    fprintf(stderr, "test delete6 failed (step 8)\n");
	    return 6;
	}
	if (!n1.AddNode("foo", E4_IOLAST, rank, n2) || !n2.IsValid()) {
	    fprintf(stderr, "test delete6 failed (step 9)\n");
	    return 6;
	}
	if (!n2.AddVertex("foo", E4_IOLAST, rank, n1) || !n1.IsValid() ||
	    !n2.IsValid() || 
	    (n1.OccurrenceCount() != 1) || (n1.OccurrenceCount(n2) != 1) ||
	    (n2.OccurrenceCount() != 1) || (n2.OccurrenceCount(n1) != 1)) {
	    fprintf(stderr, "test delete6 failed (step 10)\n");
	    return 6;
	}
	if (!m.Delete() || m.IsValid() || n1.IsValid() || n2.IsValid()) {
	    fprintf(stderr, "test delete6 failed (step 11) %d %d\n",
		    n1.IsValid(), n2.IsValid());
	    return 6;
	}
	if (!s.IsValid() || !s.IsEmpty()) {
	    fprintf(stderr, "test delete6 failed (step 12)\n");
	    return 6;
	}
	s.Delete();
    }

    return 0;
}

static int
test_delete7()
{
    e4_Marker m1, m2;
    e4_Node n1, n2, n3;
    int rank;

    /*
     * n1 is marked by m1, n2 is marked by m2. n3 is a child of
     * n1 and n2, and a parent of n1 and n2.
     *
     * Delete m1, ensure that n1 is still valid.
     * Delete m2, ensure that n1, n2 and n3 become invalid.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test delete7 failed (step 1)\n");
	    return 7;
	}
	if (!s.GetMarker("m1", m1) || !m1.IsValid()) {
	    fprintf(stderr, "test delete7 failed (step 2)\n");
	    return 7;
	}
	if (!s.GetMarker("m2", m2) || !m1.IsValid()) {
	    fprintf(stderr, "test delete7 failed (step 3)\n");
	    return 7;
	}
	if (!m1.GetMarkedNode(n1) || !n1.IsValid()) {
	    fprintf(stderr, "test delete7 failed (step 4)\n");
	    return 7;
	}
	if (!m2.GetMarkedNode(n2) || !n2.IsValid()) {
	    fprintf(stderr, "test delete7 failed (step 5)\n");
	    return 7;
	}
	if (!n1.AddNode("n3", E4_IOLAST, rank, n3) || !n3.IsValid()) {
	    fprintf(stderr, "test delete7 failed (step 6)\n");
	    return 7;
	}
	if (!n2.AddVertex("n3", E4_IOLAST, rank, n3) || !n3.IsValid()) {
	    fprintf(stderr, "test delete7 failed (step 7)\n");
	    return 7;
	}
	if (n3.ParentCount() != 2) {
	    fprintf(stderr, "test delete7 failed (step 8)\n");
	    return 7;
	}
	if (!n3.AddVertex("n1", E4_IOLAST, rank, n1) || !n1.IsValid() ||
	    (n1.ParentCount() != 1)) {
	    fprintf(stderr, "test delete7 failed (step 9)\n");
	    return 7;
	}
	if (!n3.AddVertex("n2", E4_IOLAST, rank, n2) || !n2.IsValid() ||
	    (n2.ParentCount() != 1)) {
	    fprintf(stderr, "test delete7 failed (step 10)\n");
	    return 7;
	}
	if (!m1.Delete() || m1.IsValid() || !n1.IsValid() || !n2.IsValid() ||
	    !n3.IsValid() || (n1.MarkerCount() != 0) || 
	    (n1.ParentCount() != 1) || (n2.ParentCount() != 1) ||
	    (n2.MarkerCount() != 1) || (n3.ParentCount() != 2) ||
	    !m2.IsValid() || (s.MarkerCount() != 1)) {
	    fprintf(stderr, "test delete7 failed (step 11)\n");
	    return 7;
	}
	if (!m2.Delete() || m1.IsValid() || m2.IsValid() ||
	    n1.IsValid() || n2.IsValid()) {
	    fprintf(stderr, "test delete7 failed (step 12)\n");
	    return 7;
	}
	if (!s.IsValid() || !s.IsEmpty()) {
	    fprintf(stderr, "test delete7 failed (step 13)\n");
	    return 7;
	}
	s.Delete();
    }

    return 0;
}

static int
test_delete8()
{
    e4_Marker m;
    e4_Node n1, n2, ntmp;
    int rank, i;

    /*
     * n1 is marker by m. n1 has ten child nodes and each of these child nodes
     * has n2 as its child node. n2 has n1 as its child node.
     *
     * When m is deleted, both n1 and n2 become invalid.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test delete8 failed (step 1)\n");
	    return 8;
	}
	if (!s.GetMarker("m", m) || !m.IsValid()) {
	    fprintf(stderr, "test delete8 failed (step 2)\n");
	    return 8;
	}
	if (!m.GetMarkedNode(n1) || !n1.IsValid()) {
	    fprintf(stderr, "test delete8 failed (step 3)\n");
	    return 8;
	}
	n2 = invalidNode;
	for (i = 0; i < 10; i++) {
	    if (!n1.AddNode("x", E4_IOLAST, rank, ntmp) || !ntmp.IsValid()) {
		fprintf(stderr, "test delete8 failed (step 4.1.%d)\n", i);
		return 8;
	    }
	    if (n2 == invalidNode) {
		ntmp.AddNode("n2", E4_IOLAST, rank, n2);
	    } else {
		ntmp.AddVertex("n2", E4_IOLAST, rank, n2);
	    }
	    if (!n2.IsValid()) {
		fprintf(stderr, "test delete8 failed (step 4.2.%d)\n", i);
		return 8;
	    }
	    if (n2.ParentCount() != (i + 1)) {
		fprintf(stderr, "test delete8 failed (step 4.3.%d)\n", i);
		return 8;
	    }
	}
	if (!m.Delete() || m.IsValid() || n1.IsValid() || n2.IsValid()) {
	    fprintf(stderr, "test delete8 failed (step 5)\n");
	    return 8;
	}
	if (!s.IsValid() || !s.IsEmpty()) {
	    fprintf(stderr, "test delete8 failed (step 6)\n");
	    return 8;
	}

	s.Delete();
    }

    return 0;
}

static int
test_delete9()
{
    e4_Marker m;
    e4_Node n, p, c;
    int i, rank;

    /*
     * n is marked by m. Make a chain of 1001 nodes whose long whose ultimate
     * parent is n. Make the last child have n as its child, forming a cycle
     * one thousand and one nodes long.
     *
     * When m is deleted, n and all nodes up to the last node become invalid.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test delete9 failed (step 1)\n");
	    return 9;
	}
	if (!s.GetMarker("m", m) || !m.IsValid()) {
	    fprintf(stderr, "test delete9 failed (step 2)\n");
	    return 9;
	}
	if (!m.GetMarkedNode(n) || !n.IsValid()) {
	    fprintf(stderr, "test delete9 failed (step 3)\n");
	    return 9;
	}
	for (p = n, i = 0; i < 1000; i++) {
	    if (!p.AddNode("c", E4_IOLAST, rank, c) || !c.IsValid()) {
		fprintf(stderr, "test delete9 failed (step 4.1.%d)\n", i);
		return 9;
	    }
	    if ((p.VertexCount() != 1) || (c.ParentCount() != 1)) {
		fprintf(stderr, "test delete9 failed (step 4.2.%d)\n", i);
		return 9;
	    }
	    p = c;
	}
	if (!c.AddVertex("n", E4_IOLAST, rank, n) || !n.IsValid() ||
	    (n.ParentCount() != 1)) {
	    fprintf(stderr, "test delete9 failed (step 5)\n");
	    return 9;
	}
	if (!m.Delete() || m.IsValid() || c.IsValid() || n.IsValid()) {
	    fprintf(stderr, "test delete9 failed (step 6)\n");
	    return 9;
	}
	if (!s.IsValid() || !s.IsEmpty()) {
	    fprintf(stderr, "test delete9 failed (step 7)\n");
	    return 9;
	}

	s.Delete();
    }

    return 0;
}
    
int
test_delete()
{
    int result;

    fprintf(stderr, "Running deletion tests: ");
    result = test_delete1();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_delete2();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_delete3();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_delete4();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_delete5();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_delete6();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_delete7();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_delete8();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_delete9();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".\n");

    return 0;
}
