/*
 * testmarker.cpp --
 *
 * Tests of the e4_Marker class.
 *
 * Copyright (c) 2000-2001, JYL Software Inc.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE, EVEN IF
 * JYL SOFTWARE INC. IS MADE AWARE OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#ifndef	_WIN32
#include <unistd.h>
#endif
#include "e4graph.h"
#include "test.h"

static int
test_marker1()
{
    e4_Marker t;

    /*
     * Test that we can create a marker in a storage.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker1 failed (step 1)\n");
	    return 1;
	}
	if (s.MarkerCount() != 0) {
	    fprintf(stderr, "test marker1 failed (step 2), %d\n",
		    s.MarkerCount());
	    return 1;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test marker1 failed (step 3)\n");
	    return 1;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test marker1 failed (step 4)\n");
	    return 1;
	}
	if (s.MarkerCount() != 1) {
	    fprintf(stderr, "test marker1 failed (step 5)\n");
	    return 1;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test marker1 failed (step 6)\n");
	    return 1;
	}
    }
    if (t.IsValid()) {
	fprintf(stderr, "test marker1 failed (step 7)\n");
	return 1;
    }

    return 0;
}

static int
test_marker2()
{
    e4_Marker t;

    /*
     * Test that we can get the storage back from a marker.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker2 failed (step 1)\n");
	    return 2;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test marker2 failed (step 2)\n");
	    return 2;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test marker2 failed (step 3)\n");
	    return 2;
	}
	e4_Storage bs;
	if (!t.GetStorage(bs)) {
	    fprintf(stderr, "test marker2 failed (step 4)\n");
	    return 2;
	}
	if (!bs.IsValid()) {
	    fprintf(stderr, "test marker2 failed (step 5)\n");
	    return 2;
	}
	if (bs != s) {
	    fprintf(stderr, "test marker2 failed (step 6)\n");
	    return 2;
	}
	s.Delete();
	if (s.IsValid() || bs.IsValid()) {
	    fprintf(stderr, "test marker2 failed (step 7)\n");
	    return 2;
	}
    }
    if (t.IsValid()) {
	fprintf(stderr, "test marker2 failed (step 8)\n");
	return 2;
    }

    return 0;
}

static int
test_marker3()
{
    e4_Marker t;
    const char *nm;

    /*
     * Test that we can get the name of a marker.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker3 failed (step 1)\n");
	    return 3;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test marker3 failed (step 2)\n");
	    return 3;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test marker3 failed (step 4)\n");
	    return 3;
	}
	nm = t.GetName();
	if (nm == NULL) {
	    fprintf(stderr, "test marker3 failed (step 5)\n");
	    return 3;
	}
	if (strcmp(nm, "mymarker") != 0) {
	    fprintf(stderr, "test marker3 failed (step 6)\n");
	    return 3;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test marker3 failed (step 7)\n");
	    return 3;
	}
    }
    if (t.IsValid()) {
	fprintf(stderr, "test marker3 failed (step 8)\n");
	return 3;
    }
    if (t.GetName() != NULL) {
	fprintf(stderr, "test marker3 failed (step 9)\n");
	return 3;
    }

    return 0;
}

static int
test_marker4()
{
    e4_Marker t;
    e4_Node n;

    /*
     * Test that we can get the root node of a marker.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker4 failed (step 1)\n");
	    return 4;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test marker4 failed (step 2)\n");
	    return 4;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test marker4 failed (step 3)\n");
	    return 4;
	}
	if (!t.GetMarkedNode(n)) {
	    fprintf(stderr, "test marker4 failed (step 4)\n");
	    return 4;
	}
	if (!n.IsValid()) {
	    fprintf(stderr, "test marker4 failed (step 5)\n");
	    return 4;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test marker4 failed (step 6)\n");
	    return 4;
	}
    }
    if (t.IsValid()) {
	fprintf(stderr, "test marker4 failed (step 7)\n");
	return 4;
    }
    if (n.IsValid()) {
	fprintf(stderr, "test marker4 failed (step 8)\n");
	return 4;
    }
    if (t.GetMarkedNode(n)) {
	fprintf(stderr, "test marker4 failed (step 9)\n");
	return 4;
    }

    return 0;
}

static int
test_marker5()
{
    e4_Marker t1, t2;
    e4_Node n1, n2;

    /*
     * Test that the root node is different for each marker.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker5 failed (step 1)\n");
	    return 5;
	}
	if (!s.GetMarker("t1", t1)) {
	    fprintf(stderr, "test marker5 failed (step 2)\n");
	    return 5;
	}
	if (!t1.IsValid()) {
	    fprintf(stderr, "test marker5 failed (step 3)\n");
	    return 5;
	}
	if (!t1.GetMarkedNode(n1)) {
	    fprintf(stderr, "test marker5 failed (step 4)\n");
	    return 5;
	}
	if (!s.GetMarker("t2", t2)) {
	    fprintf(stderr, "test marker5 failed (step 5)\n");
	    return 5;
	}
	if (!t2.IsValid()) {
	    fprintf(stderr, "test marker5 failed (step 6)\n");
	    return 5;
	}
	if (!t2.GetMarkedNode(n2)) {
	    fprintf(stderr, "test marker5 failed (step 7)\n");
	    return 5;
	}
	if (!n1.IsValid() || !n2.IsValid()) {
	    fprintf(stderr, "test marker5 failed (step 8)\n");
	    return 5;
	}
	if (n1 == n2) {
	    fprintf(stderr, "test marker5 failed (step 9)\n");
	    return 5;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test marker5 failed (step 10)\n");
	    return 5;
	}
    }
    if (t1.IsValid() || t2.IsValid() || n1.IsValid() || n2.IsValid()) {
	fprintf(stderr, "test marker5 failed (step 11)\n");
	return 5;
    }

    return 0;
}

static int
test_marker6()
{
    e4_Marker t1, t2;
    e4_Node n1, n2;

    /*
     * Test that the same root node is returned when gotten from two
     * references to the same marker.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker6 failed (step 1)\n");
	    return 6;
	}
	if (!s.GetMarker("mymarker", t1)) {
	    fprintf(stderr, "test marker6 failed (step 2)\n");
	    return 6;
	}
	if (!t1.IsValid()) {
	    fprintf(stderr, "test marker6 failed (step 3)\n");
	    return 6;
	}
	if (!t1.GetMarkedNode(n1)) {
	    fprintf(stderr, "test marker6 failed (step 4)\n");
	    return 6;
	}
	if (!s.GetMarker("mymarker", t2)) {
	    fprintf(stderr, "test marker6 failed (step 5)\n");
	    return 6;
	}
	if (!t2.IsValid()) {
	    fprintf(stderr, "test marker6 failed (step 6)\n");
	    return 6;
	}
	if (!t2.GetMarkedNode(n2)) {
	    fprintf(stderr, "test marker6 failed (step 7)\n");
	    return 6;
	}
	if (!n1.IsValid() || !n2.IsValid()) {
	    fprintf(stderr, "test marker6 failed (step 8)\n");
	    return 6;
	}
	if (n1 != n2) {
	    fprintf(stderr, "test marker6 failed (step 9)\n");
	    return 6;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test marker6 failed (step 10)\n");
	    return 6;
	}
    }
    if (t1.IsValid() || t2.IsValid() || n1.IsValid() || n2.IsValid()) {
	fprintf(stderr, "test marker6 failed (step 11)\n");
	return 6;
    }
    
    return 0;
}

static int
test_marker7()
{
    e4_Marker t1, t2;

    /*
     * Test that different markers with the same name can co-exist in different
     * storages.
     */

    {
	e4_Storage s1("foo1.db", E4_METAKIT);
	if (!s1.IsValid()) {
	    fprintf(stderr, "test marker7 failed (step 1)\n");
	    return 7;
	}
	e4_Storage s2("foo2.db", E4_METAKIT);
	if (!s2.IsValid()) {
	    fprintf(stderr, "test marker7 failed (step 2)\n");
	    return 7;
	}
	if (!s1.GetMarker("mymarker", t1)) {
	    fprintf(stderr, "test marker7 failed (step 3)\n");
	    return 7;
	}
	if (!t1.IsValid()) {
	    fprintf(stderr, "test marker7 failed (step 4)\n");
	    return 7;
	}
	if (!s2.GetMarker("mymarker", t2)) {
	    fprintf(stderr, "test marker7 failed (step 5)\n");
	    return 7;
	}
	if (!t2.IsValid()) {
	    fprintf(stderr, "test marker7 failed (step 6)\n");
	    return 7;
	}
	if (t1 == t2) {
	    fprintf(stderr, "test marker7 failed (step 7)\n");
	    return 7;
	}
	s1.Delete();
	s2.Delete();
	if (s1.IsValid() || s2.IsValid()) {
	    fprintf(stderr, "test marker7 failed (step 8)\n");
	    return 7;
	}
    }
    if (t1.IsValid() || t2.IsValid()) {
	fprintf(stderr, "test marker7 failed (step 9)\n");
	return 7;
    }

    return 0;
}

static int
test_marker8()
{
    e4_Marker t;
    e4_MarkerUniqueID muid;

    /*
     * Test that we can get a unique ID from a marker and that the unique
     * ID is >= 0.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker8 failed (step 1)\n");
	    return 8;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test marker8 failed (step 2)\n");
	    return 8;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test marker8 failed (step 3)\n");
	    return 8;
	}
	if ((!t.GetUniqueID(muid)) || 
	    (muid.GetUniqueID() == E4_INVALIDUNIQUEID)) {
	    fprintf(stderr, "test marker8 failed (step 4)\n");
	    return 8;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test marker8 failed (step 5)\n");
	    return 8;
	}
    }
    if ((t.IsValid()) || (t.GetUniqueID(muid))) {
	fprintf(stderr, "test marker8 failed (step 6)\n");
	return 8;
    }

    return 0;
}

static int
test_marker9()
{
    e4_Marker t1, t2;
    e4_MarkerUniqueID muid1, muid2;

    /*
     * Test that the unique IDs from two markers in the same storage are indeed
     * different.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker9 failed (step 1)\n");
	    return 9;
	}
	if (!s.GetMarker("t1", t1)) {
	    fprintf(stderr, "test marker9 failed (step 2)\n");
	    return 9;
	}
	if (!t1.IsValid()) {
	    fprintf(stderr, "test marker9 failed (step 3)\n");
	    return 9;
	}
	if (!s.GetMarker("t2", t2)) {
	    fprintf(stderr, "test marker9 failed (step 4)\n");
	    return 9;
	}
	if (!t2.IsValid()) {
	    fprintf(stderr, "test marker9 failed (step 5)\n");
	    return 9;
	}
	if (!t1.GetUniqueID(muid1)) {
	    fprintf(stderr, "test marker9 failed (step 6)\n");
	    return 9;
	}
	if (!t2.GetUniqueID(muid2)) {
	    fprintf(stderr, "test marker9 failed (step 7)\n");
	    return 9;
	}
	if (muid1 == muid2) {
	    fprintf(stderr, "test marker9 failed (step 8)\n");
	    return 9;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test marker9 failed (step 9)\n");
	    return 9;
	}
    }
    if (t1.IsValid() || t2.IsValid()) {
	fprintf(stderr, "test marker9 failed (step 10)\n");
	return 9;
    }
    if ((t1.GetUniqueID(muid1)) || (t2.GetUniqueID(muid2))) {
	fprintf(stderr, "test marker9 failed (step 11)\n");
	return 9;
    }

    return 0;
}

static int
test_marker10()
{
    e4_Marker t;

    /*
     * Test that deletion of markers works.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker10 failed (step 1)\n");
	    return 10;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test marker10 failed (step 2)\n");
	    return 10;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test marker10 failed (step 3)\n");
	    return 10;
	}
	if (s.MarkerCount() != 1) {
	    fprintf(stderr, "test marker10 failed (step 4)\n");
	    return 10;
	}
	if (!t.Delete()) {
	    fprintf(stderr, "test marker10 failed (step 5)\n");
	    return 10;
	}
	if (s.MarkerCount() != 0) {
	    fprintf(stderr, "test marker10 failed (step 6)\n");
	    return 10;
	}
	if (t.IsValid()) {
	    fprintf(stderr, "test marker10 failed (step 7)\n");
	    return 10;
	}

	/*
	 * Explicitly commit.
	 */

	s.Commit();
    }
    if (t.IsValid()) {
	fprintf(stderr, "test marker10 failed (step 8)\n");
	return 10;
    }
    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker10 failed (step 9)\n");
	    return 10;
	}
	if (s.MarkerCount() != 0) {
	    fprintf(stderr, "test marker10 failed (step 10)\n");
	    return 10;
	}
	if (t.IsValid()) {
	    fprintf(stderr, "test marker10 failed (step 11)\n");
	    return 10;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test marker10 failed (step 12)\n");
	    return 10;
	}
    }

    return 0;
}

static int
test_marker11()
{
    e4_Marker t;
    e4_Node n;
    e4_Storage bs;

    /*
     * Test that deletion of markers works. After a marker is deleted
     * the only API that should work is IsValid.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker11 failed (step 1)\n");
	    return 11;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test marker11 failed (step 2)\n");
	    return 11;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test marker11 failed (step 3)\n");
	    return 11;
	}
	if (!t.Delete()) {
	    fprintf(stderr, "test marker11 failed (step 4)\n");
	    return 11;
	}
	if (s.MarkerCount() != 0) {
	    fprintf(stderr, "test marker11 failed (step 5)\n");
	    return 11;
	}
	if (t.GetName() != NULL) {
	    fprintf(stderr, "test marker11 failed (step 6)\n");
	    return 11;
	}
	if (t.GetMarkedNode(n)) {
	    fprintf(stderr, "test marker11 failed (step 7)\n");
	    return 11;
	}
	if (n.IsValid()) {
	    fprintf(stderr, "test marker11 failed (step 8)\n");
	    return 11;
	}
	if (t.GetStorage(bs)) {
	    fprintf(stderr, "test marker11 failed (step 9)\n");
	    return 11;
	}
	if (bs.IsValid()) {
	    fprintf(stderr, "test marker11 failed (step 10)\n");
	    return 11;
	}
	s.Delete();
	if (s.IsValid() || bs.IsValid()) {
	    fprintf(stderr, "test marker11 failed (step 11)\n");
	    return 11;
	}
    }
    if (t.IsValid() || bs.IsValid()) {
        fprintf(stderr, "test marker11 failed (step 12)\n");
        return 11;
    }

    return 0;
}

static int
test_marker12() 
{
    e4_Node n1, n2;
    e4_NodeUniqueID nuid1, nuid2;

    /*
     * Test whether the root is disconnected correctly if the owning 
     * marker goes out of scope. Test whether root is saved to permanent
     * storage correctly when marker goes out of scope.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test marker12 failed (step 1)\n");
	    return 12;
	}
	{
	    e4_Marker t;

	    if (!s.GetMarker("mymarker", t)) {
		fprintf(stderr, "test marker12 failed (step 2)\n");
		return 12;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test marker12 failed (step 3)\n");
		return 12;
	    }
	    if (!t.GetMarkedNode(n1)) {
		fprintf(stderr, "test marker12 failed (step 4)\n");
		return 12;
	    }
	    if (!n1.IsValid()) {
		fprintf(stderr, "test marker12 failed (step 5)\n");
		return 12;
	    }
	    if (!n1.GetUniqueID(nuid1)) {
		fprintf(stderr, "test marker12 failed (step 6)\n");
		return 12;
	    }
	}
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker12 failed (step 7)\n");
	    return 12;
	}
	if (s.MarkerCount() != 1) {
	    fprintf(stderr, "test marker12 failed (step 8)\n");
	    return 12;
	}
	if (!n1.IsValid()) {
	    fprintf(stderr, "test marker12 failed (step 9)\n");
	    return 12;
	}
	{
	    e4_Marker t;

	    if (!s.GetMarker("mymarker", t)) {
		fprintf(stderr, "test marker12 failed (step 10)\n");
		return 12;
	    }
	    if (s.MarkerCount() != 1) {
		fprintf(stderr, "test marker12 failed (step 11)\n");
		return 12;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test marker12 failed (step 12)\n");
		return 12;
	    }
	    if (!t.GetMarkedNode(n2)) {
		fprintf(stderr, "test marker12 failed (step 13)\n");
		return 12;
	    }
	    if (!n2.IsValid()) {
		fprintf(stderr, "test marker12 failed (step 14)\n");
		return 12;
	    }
	    if (!n2.GetUniqueID(nuid2)) {
		fprintf(stderr, "test marker12 failed (step 15)\n");
		return 12;
	    }
	}
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker12 failed (step 16)\n");
	    return 12;
	}
	if (s.MarkerCount() != 1) {
	    fprintf(stderr, "test marker12 failed (step 17)\n");
	    return 12;
	}
	if (!n2.IsValid()) {
	    fprintf(stderr, "test marker12 failed (step 18)\n");
	    return 12;
	}
	if (n1 != n2) {
	    fprintf(stderr, "test marker12 failed (step 19)\n");
	    return 12;
	}
	if (nuid1 != nuid2) {
	    fprintf(stderr, "test marker12 failed (step 20)\n");
	    return 12;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test marker12 failed (step 21)\n");
	    return 12;
	}
    }

    return 0;
}

static int
test_marker13()
{
    e4_Marker t1, t2;
    e4_MarkerUniqueID muid1, muid2;

    /*
     * Test that the same instance is returned on multiple retrievals of
     * a named marker from the storage.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker13 failed (step 1)\n");
	    return 13;
	}
	if (!s.GetMarker("mymarker", t1)) {
	    fprintf(stderr, "test marker13 failed (step 2)\n");
	    return 13;
	}
	if (!t1.IsValid()) {
	    fprintf(stderr, "test marker13 failed (step 3)\n");
	    return 13;
	}
	if (s.MarkerCount() != 1) {
	    fprintf(stderr, "test marker13 failed (step 4)\n");
	    return 13;
	}
	if (!s.GetMarker("mymarker", t2)) {
	    fprintf(stderr, "test marker13 failed (step 4)\n");
	    return 13;
	}
	if (!t2.IsValid()) {
	    fprintf(stderr, "test marker13 failed (step 5)\n");
	    return 13;
	}
	if (s.MarkerCount() != 1) {
	    fprintf(stderr, "test marker13 failed (step 6)\n");
	    return 13;
	}
	if (t1 != t2) {
	    fprintf(stderr, "test marker13 failed (step 7)\n");
	    return 13;
	}
	if (!t1.GetUniqueID(muid1)) {
	    fprintf(stderr, "test marker13 failed (step 8)\n");
	    return 13;
	}
	if (!t2.GetUniqueID(muid2)) {
	    fprintf(stderr, "test marker13 failed (step 9)\n");
	    return 13;
	}
	if (muid1 != muid2) {
	    fprintf(stderr, "test marker13 failed (step 10)\n");
	    return 13;
	}
	if (!t1.Delete()) {
	    fprintf(stderr, "test marker13 failed (step 11)\n");
	    return 13;
	}
	if (t1.IsValid() || t2.IsValid()) {
	    fprintf(stderr, "test marker13 failed (step 12)\n");
	    return 13;
	}
	if (t2.Delete()) {
	    fprintf(stderr, "test marker13 failed (step 13)\n");
	    return 13;
	}
	if ((t1.GetUniqueID(muid1)) || (t2.GetUniqueID(muid2))) {
	    fprintf(stderr, "test marker13 failed (step 14)\n");
	    return 13;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test marker13 failed (step 15)\n");
	    return 13;
	}
    }

    return 0;
}

static int
test_marker14()
{
    e4_Marker t;
    e4_Node n;

    /*
     * Test that we cannot retrieve instances of other items from an e4_Marker
     * after Delete().
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker14 failed (step 1)\n");
	    return 14;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test marker14 failed (step 2)\n");
	    return 14;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test marker14 failed (step 3)\n");
	    return 14;
	}
	if (!t.Delete()) {
	    fprintf(stderr, "test marker14 failed (step 4)\n");
	    return 14;
	}
	if (t.IsValid()) {
	    fprintf(stderr, "test marker14 failed (step 5)\n");
	    return 14;
	}
	if (t.GetMarkedNode(n)) {
	    fprintf(stderr, "test marker14 failed (step 6)\n");
	    return 14;
	}
	if (t.GetName() != NULL) {
	    fprintf(stderr, "test marker14 failed (step 7)\n");
	    return 14;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test marker14 failed (step 8)\n");
	    return 14;
	}
    }

    return 0;
}

static int
test_marker15()
{
    e4_Marker t;
    e4_MarkerUniqueID muid1, muid2;

    /*
     * Test that GetUniqueID returns the same value across sequential
     * opens of the storage.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker15 failed (step 1)\n");
	    return 15;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test marker15 failed (step 2)\n");
	    return 15;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test marker15 failed (step 3)\n");
	    return 15;
	}
	if (!t.GetUniqueID(muid1)) {
	    fprintf(stderr, "test marker15 failed (step 4)\n");
	    return 15;
	}
    }
    if (t.IsValid()) {
	fprintf(stderr, "test marker15 failed (step 5)\n");
	return 15;
    }
    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker15 failed (step 6)\n");
	    return 15;
	}
	if (t.IsValid()) {
	    fprintf(stderr, "test marker15 failed (step 7)\n");
	    return 15;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test marker15 failed (step 8)\n");
	    return 15;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test marker15 failed (step 9)\n");
	    return 15;
	}
	if (s.MarkerCount() != 1) {
	    fprintf(stderr, "test marker15 failed (step 10)\n");
	    return 15;
	}
	if (!t.GetUniqueID(muid2)) {
	    fprintf(stderr, "test marker15 failed (step 11)\n");
	    return 15;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test marker15 failed (step 12)\n");
	    return 15;
	}
    }
    if (t.IsValid()) {
	fprintf(stderr, "test marker15 failed (step 13)\n");
	return 15;
    }

    return 0;
}

static int
test_marker16()
{
    e4_Marker t;

    /*
     * Test e4_MarkerImpl::IsValid() under various circumstances.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker16 failed (step 1)\n");
	    return 16;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test marker16 failed (step 2)\n");
	    return 16;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test marker16 failed (step 3)\n");
	    return 16;
	}
    }
    if (t.IsValid()) {
	fprintf(stderr, "test marker16 failed (step 4)\n");
	return 16;
    }
    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker16 failed (step 5)\n");
	    return 16;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test marker16 failed (step 6)\n");
	    return 16;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test marker16 failed (step 7)\n");
	    return 16;
	}
	if (!t.Delete()) {
	    fprintf(stderr, "test marker16 failed (step 8)\n");
	    return 16;
	}
	if (t.IsValid()) {
	    fprintf(stderr, "test marker16 failed (step 9)\n");
	    return 16;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test marker16 failed (step 10)\n");
	    return 16;
	}
    }
    if (t.IsValid()) {
	fprintf(stderr, "test marker16 failed (step 11)\n");
	return 16;
    }

    return 0;
}

static int
test_marker17()
{
    e4_Marker t;
    e4_Node n;
    e4_NodeUniqueID nuid0, nuid1;
    e4_MarkerUniqueID muid0, muid1;
    int i;

    /*
     * This test checks that the storage for markers and nodes is
     * properly recycled (as denoted by reuse of the unique IDs for
     * these elements). This test is specific to the E4_METAKIT driver
     * because it relies on its property of reusing unique IDs.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test marker17 failed (step 1)\n");
	    return 17;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test marker17 failed (step 2)\n");
	    return 17;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test marker17 failed (step 3)\n");
	    return 17;
	}
	if ((!t.GetMarkedNode(n)) || (!n.IsValid())) {
	    fprintf(stderr, "test marker17 failed (step 4)\n");
	    return 17;
	}
	(void) t.GetUniqueID(muid0);
	(void) n.GetUniqueID(nuid0);
	for (i = 0; i < 255; i++) {
	    if (!t.Delete()) {
		fprintf(stderr, "test marker17 failed (step 5.1.1.%d)\n", i);
		return 17;
	    }
	    if (t.IsValid()) {
		fprintf(stderr, "test marker17 failed (step 5.1.2.%d)\n", i);
		return 17;
	    }
	    if (n.IsValid()) {
		fprintf(stderr, "test marker17 failed (step 5.1.3.%d)\n", i);
		return 17;
	    }
	    if ((!s.GetMarker("m", t)) || (!t.IsValid()) ||
		(!t.GetUniqueID(muid1)) || (muid0 != muid1)) {
		fprintf(stderr, "test marker17 failed (step 5.2.%d)\n", i);
		return 17;
	    }
	    if ((!t.GetMarkedNode(n)) || (!n.IsValid()) ||
		(!n.GetUniqueID(nuid1)) || (nuid0 != nuid1)) {
		fprintf(stderr, "test marker17 failed (step 5.3.%d)\n", i);
		return 17;
	    }
	}
	s.Delete();
	if ((s.IsValid()) || (t.IsValid()) || (n.IsValid())) {
	    fprintf(stderr, "test marker17 failed (step 8)\n");
	    return 17;
	}
    }

    return 0;
}

int
test_marker()
{
    int result;

    fprintf(stderr, "Running e4_Marker tests: ");
    result = test_marker1();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_marker2();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_marker3();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_marker4();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_marker5();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_marker6();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_marker7();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_marker8();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_marker9();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_marker10();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_marker11();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_marker12();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_marker13();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_marker14();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_marker15();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_marker16();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_marker17();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".\n");
    
    return 0;
}
