/*
 * testmvisitor.cpp --
 *
 * Tests of the e4_MarkerVisitor class.
 *
 * Copyright (c) 2000-2001, JYL Software Inc.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE, EVEN IF
 * JYL SOFTWARE INC. IS MADE AWARE OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#ifndef	_WIN32
#include <unistd.h>
#endif
#include "e4graph.h"
#include "test.h"

#define	TTV_MARKERLIMIT	256

static int
test_markervisitor1()
{
    e4_Marker t;
    int i;
    char buf[64];

    /*
     * Test marker visitor construction from a storage.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test markervisitor1 failed (step 1)\n");
	    return 1;
	}
	for (i = 0; i < TTV_MARKERLIMIT; i++) {
	    sprintf(buf, "marker%d", i);
	    if (!s.GetMarker(buf, t)) {
		fprintf(stderr, "test markervisitor1 failed (step 2.1.%d)\n", 
			i);
		return 1;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor1 failed (step 2.2.%d)\n", 
			i);
		return 1;
	    }
	}
	e4_MarkerVisitor tv(s);

	if (!tv.IsValid()) {
	    fprintf(stderr, "test markervisitor1 failed (step 3.1)\n");
	    return 1;
	}
	if (tv.IsDone()) {
	    fprintf(stderr, "test markervisitor1 failed (step 3.2)\n");
	    return 1;
	}
	for (i = 0;
	     (i < TTV_MARKERLIMIT) && (!tv.IsDone());
	     i++, (void) tv.NextMarker(t)) {
	    if (!tv.CurrentMarker(t)) {
		fprintf(stderr, "test markervisitor1 failed (step 4.1.%d)\n",
			i);
		return 1;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor1 failed (step 4.2.%d)\n",
			i);
		return 1;
	    }
	}
	if ((!tv.IsDone()) || (!tv.IsValid()) || (i != TTV_MARKERLIMIT)) {
	    fprintf(stderr, "test markervisitor1 failed (step 5)\n");
	    return 1;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test markervisitor1 failed (step 6)\n");
	    return 1;
	}
	if (tv.IsValid()) {
	    fprintf(stderr, "test markervisitor1 failed (step 7)\n");
	    return 1;
	}
    }

    return 0;
}

static int
test_markervisitor2()
{
    e4_Marker t;
    int i;
    char buf[64];

    /*
     * Test marker visitor construction from a storage using
     * e4_MarkerVisitor::SetStorage().
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test markervisitor2 failed (step 1)\n");
	    return 2;
	}
	for (i = 0; i < TTV_MARKERLIMIT; i++) {
	    sprintf(buf, "marker%d", i);
	    if (!s.GetMarker(buf, t)) {
		fprintf(stderr, "test markervisitor2 failed (step 2.1.%d)\n", 
			i);
		return 2;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor2 failed (step 2.2.%d)\n", 
			i);
		return 2;
	    }
	}
	e4_MarkerVisitor tv;

	if (tv.IsValid()) {
	    fprintf(stderr, "test markervisitor2 failed (step 3.1)\n");
	    return 2;
	}
	if (!tv.SetStorage(s)) {
	    fprintf(stderr, "test markervisitor2 failed (step 3.2)\n");
	    return 2;
	}
	if (!tv.IsValid()) {
	    fprintf(stderr, "test markervisitor2 failed (step 3.3)\n");
	    return 2;
	}
	if (tv.IsDone()) {
	    fprintf(stderr, "test markervisitor2 failed (step 3.4)\n");
	    return 2;
	}
	for (i = 0;
	     (i < TTV_MARKERLIMIT) && (!tv.IsDone());
	     i++, (void) tv.NextMarker(t)) {
	    if (!tv.CurrentMarker(t)) {
		fprintf(stderr, "test markervisitor2 failed (step 4.1.%d)\n", 
			i);
		return 2;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor2 failed (step 4.2.%d)\n", 
			i);
		return 2;
	    }
	}
	if ((!tv.IsDone()) || (i != TTV_MARKERLIMIT)) {
	    fprintf(stderr, "test markervisitor2 failed (step 5)\n");
	    return 2;
	}
	if (!tv.SetStorage(s)) {
	    fprintf(stderr, "test markervisitor2 failed (step 6.1)\n");
	    return 2;
	}
	if (!tv.IsValid()) {
	    fprintf(stderr, "test markervisitor2 failed (step 6.2)\n");
	    return 2;
	}
	for (i = 0;
	     (i < TTV_MARKERLIMIT) && (!tv.IsDone());
	     i++, (void) tv.NextMarker(t)) {
	    if (!tv.CurrentMarker(t)) {
		fprintf(stderr, "test markervisitor2 failed (step 7.1.%d)\n", 
			i);
		return 2;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor2 failed (step 7.2.%d)\n", 
			i);
		return 2;
	    }
	}
	if ((!tv.IsDone()) || (!tv.IsValid()) || (i != TTV_MARKERLIMIT)) {
	    fprintf(stderr, "test markervisitor2 failed (step 8)\n");
	    return 2;
	}

	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test markervisitor2 failed (step 9)\n");
	    return 2;
	}
	if (tv.IsValid()) {
	    fprintf(stderr, "test markervisitor2 failed (step 10)\n");
	    return 2;
	}

	/*
	 * Test that it is disallowed to set the visitor to an invalid
	 * storage.
	 */

	if (tv.SetStorage(invalidStorage)) {
	    fprintf(stderr, "test markervisitor2 failed (step 11)\n");
	    return 2;
	}
    }

    return 0;
}

static int
test_markervisitor3()
{
    e4_Marker t;
    int i;
    char buf[64];

    /*
     * Test marker visitor construction from a marker.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test markervisitor3 failed (step 1)\n");
	    return 3;
	}
	for (i = 0; i < TTV_MARKERLIMIT; i++) {
	    sprintf(buf, "marker%d", i);
	    if (!s.GetMarker(buf, t)) {
		fprintf(stderr, "test markervisitor3 failed (step 2.1.%d)\n", 
			i);
		return 3;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor3 failed (step 2.2.%d)\n", 
			i);
		return 3;
	    }
	}
	e4_MarkerVisitor tv(t);

	if (!tv.IsValid()) {
	    fprintf(stderr, "test markervisitor3 failed (step 3.1)\n");
	    return 3;
	}
	if (tv.IsDone()) {
	    fprintf(stderr, "test markervisitor3 failed (step 3.2)\n");
	    return 3;
	}
	for (i = 0;
	     (i < TTV_MARKERLIMIT) && (!tv.IsDone());
	     i++, (void) tv.NextMarker(t)) {
	    if (!tv.CurrentMarker(t)) {
		fprintf(stderr, "test markervisitor3 failed (step 4.1.%d)\n", 
			i);
		return 3;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor3 failed (step 4.2.%d)\n", 
			i);
		return 3;
	    }
	}
	if ((!tv.IsDone()) || (!tv.IsValid()) || (i != TTV_MARKERLIMIT)) {
	    fprintf(stderr, "test markervisitor3 failed (step 5)\n");
	    return 3;
	}
	s.Delete();
	if (s.IsValid() || tv.IsValid()) {
	    fprintf(stderr, "test markervisitor3 failed (step 6)\n");
	    return 3;
	}
    }

    return 0;
}

static int
test_markervisitor4()
{
    e4_Marker t, ts;
    int i;
    char buf[64];

    /*
     * Test marker visitor construction from a marker using 
     * e4_MarkerVisitor::SetMarker().
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test markervisitor4 failed (step 1)\n");
	    return 4;
	}
	for (i = 0; i < TTV_MARKERLIMIT; i++) {
	    sprintf(buf, "marker%d", i);
	    if (!s.GetMarker(buf, t)) {
		fprintf(stderr, "test markervisitor4 failed (step 2.1.%d)\n", 
			i);
		return 4;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor4 failed (step 2.2.%d)\n", 
			i);
		return 4;
	    }
	}

	ts = t;

	e4_MarkerVisitor tv;

	if (tv.IsValid()) {
	    fprintf(stderr, "test markervisitor4 failed (step 3.1)\n");
	    return 4;
	}
	if (!tv.SetMarker(ts)) {
	    fprintf(stderr, "test markervisitor4 failed (step 3.2)\n");
	    return 4;
	}
	if (!tv.IsValid()) {
	    fprintf(stderr, "test markervisitor4 failed (step 3.3)\n");
	    return 4;
	}
	if (tv.IsDone()) {
	    fprintf(stderr, "test markervisitor4 failed (step 3.4)\n");
	    return 4;
	}
	for (i = 0;
	     (i < TTV_MARKERLIMIT) && (!tv.IsDone());
	     i++, (void) tv.NextMarker(t)) {
	    if (!tv.CurrentMarker(t)) {
		fprintf(stderr, "test markervisitor4 failed (step 4.1.%d)\n", 
			i);
		return 4;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor4 failed (step 4.2.%d)\n", 
			i);
		return 4;
	    }
	}
	if ((!tv.IsDone()) || (i != TTV_MARKERLIMIT)) {
	    fprintf(stderr, "test markervisitor4 failed (step 5)\n");
	    return 4;
	}
	if (!tv.SetMarker(ts)) {
	    fprintf(stderr, "test markervisitor4 failed (step 6.1)\n");
	    return 4;
	}
	if (!tv.IsValid()) {
	    fprintf(stderr, "test markervisitor4 failed (step 6.2)\n");
	    return 4;
	}
	for (i = 0;
	     (i < TTV_MARKERLIMIT) && (!tv.IsDone());
	     i++, (void) tv.NextMarker(t)) {
	    if (!tv.CurrentMarker(t)) {
		fprintf(stderr, "test markervisitor4 failed (step 7.1.%d)\n", 
			i);
		return 4;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor4 failed (step 7.2.%d)\n", 
			i);
		return 4;
	    }
	}
	if ((!tv.IsDone()) || (!tv.IsValid()) || (i != TTV_MARKERLIMIT)) {
	    fprintf(stderr, "test markervisitor4 failed (step 8)\n");
	    return 4;
	}
	s.Delete();
	if (s.IsValid() || tv.IsValid()) {
	    fprintf(stderr, "test markervisitor4 failed (step 9)\n");
	    return 4;
	}

	/*
	 * Test that it is disallowed to set the visitor to an invalid
	 * marker.
	 */

	if (tv.SetMarker(invalidMarker)) {
	    fprintf(stderr, "test markervisitor4 failed (step 10)\n");
	    return 4;
	}
    }

    return 0;
}

static int
test_markervisitor5()
{
    e4_Marker t;
    e4_Node n;
    int i;
    char buf[64];

    /*
     * Test marker visitor construction from a node.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test markervisitor5 failed (step 1)\n");
	    return 5;
	}
	for (i = 0; i < TTV_MARKERLIMIT; i++) {
	    sprintf(buf, "marker%d", i);
	    if (!s.GetMarker(buf, t)) {
		fprintf(stderr, "test markervisitor5 failed (step 2.1.%d)\n", 
			i);
		return 5;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor5 failed (step 2.2.%d)\n", 
			i);
		return 5;
	    }
	    if (i == 30) {
		if (!t.GetMarkedNode(n)) {
		    fprintf(stderr, "test markervisitor5 failed (step 2.3)\n");
		    return 5;
		}
		if (!n.IsValid()) {
		    fprintf(stderr, "test markervisitor5 failed (step 2.4)\n");
		    return 5;
		}
	    }
	}

	e4_MarkerVisitor tv(n);

	if (!tv.IsValid()) {
	    fprintf(stderr, "test markervisitor5 failed (step 3.1)\n");
	    return 5;
	}
	if (tv.IsDone()) {
	    fprintf(stderr, "test markervisitor5 failed (step 3.2)\n");
	    return 5;
	}
	for (i = 0;
	     (i < TTV_MARKERLIMIT) && (!tv.IsDone());
	     i++, (void) tv.NextMarker(t)) {
	    if (!tv.CurrentMarker(t)) {
		fprintf(stderr, "test markervisitor5 failed (step 4.1.%d)\n", 
			i);
		return 5;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor5 failed (step 4.2.%d)\n", 
			i);
		return 5;
	    }
	}
	if ((!tv.IsDone()) || (!tv.IsValid()) || (i != TTV_MARKERLIMIT)) {
	    fprintf(stderr, "test markervisitor5 failed (step 5)\n");
	    return 5;
	}
	s.Delete();
	if (s.IsValid() || tv.IsValid()) {
	    fprintf(stderr, "test markervisitor5 failed (step 6)\n");
	    return 5;
	}
    }

    return 0;
}

static int
test_markervisitor6()
{
    e4_Marker t;
    e4_Node n;
    int i;
    char buf[64];

    /*
     * Test marker visitor construction from a node using 
     * e4_MarkerVisitor::SetNode().
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test markervisitor6 failed (step 1)\n");
	    return 6;
	}
	for (i = 0; i < TTV_MARKERLIMIT; i++) {
	    sprintf(buf, "marker%d", i);
	    if (!s.GetMarker(buf, t)) {
		fprintf(stderr, "test markervisitor6 failed (step 2.1.%d)\n", 
			i);
		return 6;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor6 failed (step 2.2.%d)\n", 
			i);
		return 6;
	    }
	    if (i == 30) {
		if (!t.GetMarkedNode(n)) {
		    fprintf(stderr, "test markervisitor5 failed (step 2.3)\n");
		    return 6;
		}
		if (!n.IsValid()) {
		    fprintf(stderr, "test markervisitor5 failed (step 2.4)\n");
		    return 6;
		}
	    }
	}

	e4_MarkerVisitor tv;

	if (tv.IsValid()) {
	    fprintf(stderr, "test markervisitor6 failed (step 3.1)\n");
	    return 6;
	}
	if (!tv.SetNode(n)) {
	    fprintf(stderr, "test markervisitor6 failed (step 3.2)\n");
	    return 6;
	}
	if (!tv.IsValid()) {
	    fprintf(stderr, "test markervisitor6 failed (step 3.3)\n");
	    return 6;
	}
	if (tv.IsDone()) {
	    fprintf(stderr, "test markervisitor6 failed (step 3.4)\n");
	    return 6;
	}
	for (i = 0;
	     (i < TTV_MARKERLIMIT) && (!tv.IsDone());
	     i++, (void) tv.NextMarker(t)) {
	    if (!tv.CurrentMarker(t)) {
		fprintf(stderr, "test markervisitor6 failed (step 4.1.%d)\n", 
			i);
		return 6;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor6 failed (step 4.2.%d)\n", 
			i);
		return 6;
	    }
	}
	if ((!tv.IsDone()) || (i != TTV_MARKERLIMIT)) {
	    fprintf(stderr, "test markervisitor6 failed (step 5)\n");
	    return 6;
	}
	if (!tv.SetNode(n)) {
	    fprintf(stderr, "test markervisitor6 failed (step 6.1)\n");
	    return 6;
	}
	if (!tv.IsValid()) {
	    fprintf(stderr, "test markervisitor6 failed (step 6.2)\n");
	    return 6;
	}
	for (i = 0;
	     (i < TTV_MARKERLIMIT) && (!tv.IsDone());
	     i++, (void) tv.NextMarker(t)) {
	    if (!tv.CurrentMarker(t)) {
		fprintf(stderr, "test markervisitor6 failed (step 7.1.%d)\n", 
			i);
		return 6;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor6 failed (step 7.2.%d)\n", 
			i);
		return 6;
	    }
	}
	if ((!tv.IsDone()) || (!tv.IsValid()) || (i != TTV_MARKERLIMIT)) {
	    fprintf(stderr, "test markervisitor6 failed (step 8)\n");
	    return 6;
	}

	s.Delete();
	if (s.IsValid() || tv.IsValid()) {
	    fprintf(stderr, "test markervisitor6 failed (step 9)\n");
	    return 6;
	}

	/*
	 * Test that it is disallowed to set the visitor to an invalid
	 * node.
	 */

	if (tv.SetNode(invalidNode)) {
	    fprintf(stderr, "test markervisitor6 failed (step 10)\n");
	    return 6;
	}
    }

    return 0;
}

static int
test_markervisitor7()
{
    e4_Marker t;
    e4_Node n;
    e4_Vertex f;
    int i, rank;
    char buf[64];

    /*
     * Test marker visitor construction from a vertex.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test markervisitor7 failed (step 1)\n");
	    return 7;
	}
	for (i = 0; i < TTV_MARKERLIMIT; i++) {
	    sprintf(buf, "marker%d", i);
	    if (!s.GetMarker(buf, t)) {
		fprintf(stderr, "test markervisitor7 failed (step 2.1.%d)\n", 
			i);
		return 7;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor7 failed (step 2.2.%d)\n", 
			i);
		return 7;
	    }
	    if (i == 30) {
		if (!t.GetMarkedNode(n)) {
		    fprintf(stderr, "test markervisitor7 failed (step 2.3)\n");
		    return 7;
		}
		if (!n.IsValid()) {
		    fprintf(stderr, "test markervisitor7 failed (step 2.4)\n");
		    return 7;
		}
		if (!n.AddVertexRef("vertex", E4_IOLAST, rank, i, f)) {
		    fprintf(stderr, "test markervisitor7 failed (step 2.5)\n");
		    return 7;
		}
		if (!f.IsValid()) {
		    fprintf(stderr, "test markervisitor7 failed (step 2.6)\n");
		    return 7;
		}
	    }
	}

	e4_MarkerVisitor tv(f);

	if (!tv.IsValid()) {
	    fprintf(stderr, "test markervisitor7 failed (step 3.1)\n");
	    return 7;
	}
	if (tv.IsDone()) {
	    fprintf(stderr, "test markervisitor7 failed (step 3.2)\n");
	    return 7;
	}
	for (i = 0;
	     (i < TTV_MARKERLIMIT) && (!tv.IsDone());
	     i++, (void) tv.NextMarker(t)) {
	    if (!tv.CurrentMarker(t)) {
		fprintf(stderr, "test markervisitor7 failed (step 4.1.%d)\n", 
			i);
		return 7;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor7 failed (step 4.2.%d)\n", 
			i);
		return 7;
	    }
	}
	if ((!tv.IsDone()) || (!tv.IsValid()) || (i != TTV_MARKERLIMIT)) {
	    fprintf(stderr, "test markervisitor7 failed (step 5)\n");
	    return 7;
	}
	s.Delete();
	if (s.IsValid() || tv.IsValid()) {
	    fprintf(stderr, "test markervisitor7 failed (step 6)\n");
	    return 7;
	}
    }

    return 0;
}

static int
test_markervisitor8()
{
    e4_Marker t;
    e4_Node n;
    e4_Vertex f;
    int i, rank;
    char buf[64];

    /*
     * Test marker visitor construction from a node using 
     * e4_MarkerVisitor::SetNode().
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test markervisitor8 failed (step 1)\n");
	    return 8;
	}
	for (i = 0; i < TTV_MARKERLIMIT; i++) {
	    sprintf(buf, "marker%d", i);
	    if (!s.GetMarker(buf, t)) {
		fprintf(stderr, "test markervisitor8 failed (step 2.1.%d)\n", 
			i);
		return 8;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor8 failed (step 2.2.%d)\n", 
			i);
		return 8;
	    }
	    if (i == 30) {
		if (!t.GetMarkedNode(n)) {
		    fprintf(stderr, "test markervisitor8 failed (step 2.3)\n");
		    return 8;
		}
		if (!n.IsValid()) {
		    fprintf(stderr, "test markervisitor8 failed (step 2.4)\n");
		    return 8;
		}
		if (!n.AddVertexRef("vertex", E4_IOLAST, rank, i, f)) {
		    fprintf(stderr, "test markervisitor8 failed (step 2.5)\n");
		    return 8;
		}
		if (!f.IsValid()) {
		    fprintf(stderr, "test markervisitor8 failed (step 2.6)\n");
		    return 8;
		}
	    }
	}

	e4_MarkerVisitor tv;

	if (tv.IsValid()) {
	    fprintf(stderr, "test markervisitor8 failed (step 3.1)\n");
	    return 8;
	}
	if (!tv.SetVertex(f)) {
	    fprintf(stderr, "test markervisitor8 failed (step 3.2)\n");
	    return 8;
	}
	if (!tv.IsValid()) {
	    fprintf(stderr, "test markervisitor8 failed (step 3.3)\n");
	    return 8;
	}
	if (tv.IsDone()) {
	    fprintf(stderr, "test markervisitor8 failed (step 3.4)\n");
	    return 8;
	}
	for (i = 0;
	     (i < TTV_MARKERLIMIT) && (!tv.IsDone());
	     i++, (void) tv.NextMarker(t)) {
	    if (!tv.CurrentMarker(t)) {
		fprintf(stderr, "test markervisitor8 failed (step 4.1.%d)\n", 
			i);
		return 8;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor8 failed (step 4.2.%d)\n", 
			i);
		return 8;
	    }
	}
	if ((!tv.IsDone()) || (i != TTV_MARKERLIMIT)) {
	    fprintf(stderr, "test markervisitor8 failed (step 5)\n");
	    return 8;
	}
	if (!tv.SetVertex(f)) {
	    fprintf(stderr, "test markervisitor8 failed (step 6.1)\n");
	    return 8;
	}
	if (!tv.IsValid()) {
	    fprintf(stderr, "test markervisitor8 failed (step 6.2)\n");
	    return 8;
	}
	for (i = 0;
	     (i < TTV_MARKERLIMIT) && (!tv.IsDone());
	     i++, (void) tv.NextMarker(t)) {
	    if (!tv.CurrentMarker(t)) {
		fprintf(stderr, "test markervisitor8 failed (step 7.1.%d)\n", 
			i);
		return 8;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor8 failed (step 7.2.%d)\n", 
			i);
		return 8;
	    }
	}
	if ((!tv.IsDone()) || (!tv.IsValid()) || (i != TTV_MARKERLIMIT)) {
	    fprintf(stderr, "test markervisitor8 failed (step 8)\n");
	    return 8;
	}

	s.Delete();
	if (s.IsValid() || tv.IsValid()) {
	    fprintf(stderr, "test markervisitor8 failed (step9)\n");
	    return 8;
	}

	/*
	 * Test that it is disallowed to set the visitor to an invalid
	 * node.
	 */

	if (tv.SetVertex(invalidVertex)) {
	    fprintf(stderr, "test markervisitor8 failed (step 10)\n");
	    return 8;
	}
    }

    return 0;
}

static int
test_markervisitor9()
{
    e4_Marker t;
    e4_Node n;
    e4_Vertex f;
    int i, rank;
    char buf[64];

    /*
     * Test marker visitor construction from a node using 
     * e4_MarkerVisitor::SetNode().
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test markervisitor9 failed (step 1)\n");
	    return 9;
	}
	for (i = 0; i < TTV_MARKERLIMIT; i++) {
	    sprintf(buf, "marker%d", i);
	    if (!s.GetMarker(buf, t)) {
		fprintf(stderr, "test markervisitor9 failed (step 2.1.%d)\n", 
			i);
		return 9;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor9 failed (step 2.2.%d)\n", 
			i);
		return 9;
	    }
	    if (i == 30) {
		if (!t.GetMarkedNode(n)) {
		    fprintf(stderr, "test markervisitor9 failed (step 2.3)\n");
		    return 9;
		}
		if (!n.IsValid()) {
		    fprintf(stderr, "test markervisitor9 failed (step 2.4)\n");
		    return 9;
		}
		if (!n.AddVertexRef("vertex", E4_IOLAST, rank, i, f)) {
		    fprintf(stderr, "test markervisitor9 failed (step 2.5)\n");
		    return 9;
		}
		if (!f.IsValid()) {
		    fprintf(stderr, "test markervisitor9 failed (step 2.6)\n");
		    return 9;
		}
	    }
	}
        e4_MarkerVisitor v1(s);
        e4_MarkerVisitor v2 = v1;
        
        if (v1 != v2) {
            fprintf(stderr, "test markervisitor9 failed (step 3)\n");
            return 9;
        }
        for (i = 0; i < 10; i++, v1.NextMarker(t)) {
            if (!v1.CurrentMarker(t)) {
                break;
            }
        }
        if (v1 == v2) {
            fprintf(stderr, "test markervisitor9 failed (step 4)\n");
            return 9;
        }
        if ((v1.IsDone()) || (i != 10)) {
            fprintf(stderr, "test markervisitor9 failed (step 5)\n");
            return 9;
        }
        for (i = 0; i < 10; i++, v2.NextMarker(t)) {
            if (!v2.CurrentMarker(t)) {
                break;
            }
        }
        if (v1 != v2) {
            fprintf(stderr, "test markervisitor9 failed (step 6)\n");
            return 9;
        }
        if ((v2.IsDone()) || (i != 10)) {
            fprintf(stderr, "test markervisitor9 failed (step 7)\n");
            return 9;
        }
        v1.SetStorage(s);
        if (v1 == v2) {
            fprintf(stderr, "test markervisitor9 failed (step 8)\n");
            return 9;
        }
        for (i = 0; i < 10; i++, v1.NextMarker(t)) {
            if (!v1.CurrentMarker(t)) {
                break;
            }
        }
        if (v1 != v2) {
            fprintf(stderr, "test markervisitor9 failed (step 9)\n");
            return 9;
        }
        if ((v1.IsDone()) || (!v1.IsValid()) || (i != 10)) {
            fprintf(stderr, "test markervisitor9 failed (step 10)\n");
            return 9;
        }
	s.Delete();
	if (s.IsValid() || v1.IsValid() || v2.IsValid()) {
	    fprintf(stderr, "test markervisitor9 failed (step 11)\n");
	    return 9;
	}
    }
    
    return 0;
}

static int
test_markervisitor10()
{
    e4_Marker t;
    int i;
    char buf[64];

    /*
     * Test marker visitor construction from a storage.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test markervisitor10 failed (step 1)\n");
	    return 10;
	}
	for (i = 0; i < TTV_MARKERLIMIT; i++) {
	    sprintf(buf, "marker%d", i);
	    if (!s.GetMarker(buf, t)) {
		fprintf(stderr, "test markervisitor10 failed (step 2.1.%d)\n", 
			i);
		return 10;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor10 failed (step 2.2.%d)\n", 
			i);
		return 10;
	    }
	}
	e4_MarkerVisitor tv(s);

	if (!tv.IsValid()) {
	    fprintf(stderr, "test markervisitor10 failed (step 3.1)\n");
	    return 10;
	}
	if (tv.IsDone()) {
	    fprintf(stderr, "test markervisitor10 failed (step 3.2)\n");
	    return 10;
	}
	for (i = 0;
	     (i < TTV_MARKERLIMIT / 2) && (!tv.IsDone());
	     i++, (void) tv.NextMarker(t)) {
	    if (!tv.CurrentMarker(t)) {
		fprintf(stderr, "test markervisitor10 failed (step 4.1.%d)\n",
			i);
		return 10;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test markervisitor10 failed (step 4.2.%d)\n",
			i);
		return 10;
	    }
	}
	if (tv.IsDone()) {
	    fprintf(stderr, "test markervisitor10 failed (step 5)\n");
	    return 10;
	}
	if (!tv.IsValid()) {
	    fprintf(stderr, "test markervisitor10 failed (step 6)\n");
	    return 10;
	}
	s.Delete();
	if (!tv.IsDone()) {
	    fprintf(stderr, "test markervisitor10 failed (step 7)\n");
	    return 10;
	}
	if (tv.IsValid()) {
	    fprintf(stderr, "test markervisitor10 failed (step 8)\n");
	    return 10;
	}
    }

    return 0;
}

int
test_markervisitor()
{
    int result = 0;

    fprintf(stderr, "Running e4_MarkerVisitor tests: ");
    result = test_markervisitor1();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_markervisitor2();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_markervisitor3();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_markervisitor4();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_markervisitor5();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_markervisitor6();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_markervisitor7();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_markervisitor8();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_markervisitor9();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_markervisitor10();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".\n");
    return 0;
}
