/*
 * testnode.cpp --
 *
 * Tests of the e4_Node class.
 *
 * Copyright (c) 2000-2001, JYL Software Inc.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE, EVEN IF
 * JYL SOFTWARE INC. IS MADE AWARE OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#ifndef	_WIN32
#include <unistd.h>
#endif
#include "e4graph.h"
#include "test.h"

#define TN_VERTEXLIMIT	256

static int
test_node1()
{
    e4_Marker t;
    e4_Node n;

    /*
     * Test that we can get a root node from an e4_StorageImpl.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test node1 failed (step 1)\n");
	    return 1;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test node1 failed (step 2)\n");
	    return 1;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test node1 failed (step 3)\n");
	    return 1;
	}
	if (!t.GetMarkedNode(n)) {
	    fprintf(stderr, "test node1 failed (step 4)\n");
	    return 1;
	}
	if (!n.IsValid()) {
	    fprintf(stderr, "test node1 failed (step 5)\n");
	    return 1;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node1 failed (step 6)\n");
	    return 1;
	}
    }
    if (n.IsValid()) {
	fprintf(stderr, "test node1 failed (step 7)\n");
	return 1;
    }
    if (t.IsValid()) {
	fprintf(stderr, "test node1 failed (step 8)\n");
	return 1;
    }

    return 0;
}

static int
test_node4()
{
    e4_Marker t;
    e4_Node n;
    int rank, i;

    /*
     * Test e4_Node::VertexCount() under various circumstances.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test node4 failed (step 1)\n");
	    return 4;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test node4 failed (step 2)\n");
	    return 4;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test node4 failed (step 3)\n");
	    return 4;
	}
	if (!t.GetMarkedNode(n)) {
	    fprintf(stderr, "test node4 failed (step 4)\n");
	    return 4;
	}
	if (!n.IsValid()) {
	    fprintf(stderr, "test node4 failed (step 5)\n");
	    return 4;
	}
	if (n.VertexCount() != 0) {
	    fprintf(stderr, "test node4 failed (step 6)\n");
	    return 4;
	}
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    if (!n.AddVertex("f", E4_IOLAST, rank, (int) 34)) {
		fprintf(stderr, "test node4 failed (step 7.%d.1)\n", i+1);
		return 4;
	    }
	    if (n.VertexCount() != i + 1) {
		fprintf(stderr, "test node4 failed (step 7.%d.2)\n", i+1);
		return 4;
	    }
	}
	s.Commit();
    }
    if (n.IsValid() || t.IsValid()) {
	fprintf(stderr, "test node4 failed (step 8)\n");
	return 4;
    }
    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test node4 failed (step 9)\n");
	    return 4;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test node4 failed (step 10)\n");
	    return 4;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test node4 failed (step 11)\n");
	    return 4;
	}
	if (!t.GetMarkedNode(n)) {
	    fprintf(stderr, "test node4 failed (step 12)\n");
	    return 4;
	}
	if (n.VertexCount() != TN_VERTEXLIMIT) {
	    fprintf(stderr, "test node4 failed (step 13)\n");
	    return 4;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node4 failed (step 14)\n");
	    return 4;
	}
    }
    if (n.IsValid() || t.IsValid()) {
	fprintf(stderr, "test node4 failed (step 15)\n");
	return 4;
    }
    if (n.VertexCount() == TN_VERTEXLIMIT) {
	fprintf(stderr, "test node4 failed (step 16)\n");
	return 4;
    }

    return 0;
}


static int
test_node5()
{
    e4_Marker t;
    e4_Node n1, n2, n3;
    int rank;

    /*
     * Test the e4_Node::SetNthNode and e4_Node::GetParent APIs.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test node5 failed (step 1)\n");
	    return 5;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test node5 failed (step 2)\n");
	    return 5;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test node5 failed (step 3)\n");
	    return 5;
	}
	if (!t.GetMarkedNode(n1)) {
	    fprintf(stderr, "test node5 failed (step 4)\n");
	    return 5;
	}
	if (!n1.IsValid()) {
	    fprintf(stderr, "test node5 failed (step 5)\n");
	    return 5;
	}
	if (!n1.AddVertex("f", E4_IOLAST, rank, (int) 33)) {
	    fprintf(stderr, "test node5 failed (step 6)\n");
	    return 5;
	}
	if (n1.VertexCount() != 1) {
	    fprintf(stderr, "test node5 failed (step 7)\n");
	    return 5;
	}
	if (!n1.SetNthNode("f", 1, n2)) {
	    fprintf(stderr, "test node5 failed (step 8)\n");
	    return 5;
	}
	if (!n2.IsValid()) {
	    fprintf(stderr, "test node5 failed (step 9)\n");
	    return 5;
	}
	if (n2.VertexCount() != 0) {
	    fprintf(stderr, "test node5 failed (step 10)\n");
	    return 5;
	}
	if (n1.VertexCount() != 1) {
	    fprintf(stderr, "test node5 failed (step 11)\n");
	    return 5;
	}
	if (!n2.GetParent(n3)) {
	    fprintf(stderr, "test node5 failed (step 12)\n");
	    return 5;
	}
	if (!n3.IsValid()) {
	    fprintf(stderr, "test node5 failed (step 13)\n");
	    return 5;
	}
	if (n3 != n1) {
	    fprintf(stderr, "test node5 failed (step 14)\n");
	    return 5;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node5 failed (step 15)\n");
	    return 5;
	}
    }
    if (t.IsValid() || n1.IsValid() || n2.IsValid() || n3.IsValid()) {
	fprintf(stderr, "test node5 failed (step 16)\n");
	return 5;
    }
    if (n2.GetParent(n3)) {
	fprintf(stderr, "test node5 failed (step 17)\n");
	return 5;
    }

    return 0;
}

static int
test_node6()
{
    e4_Marker t;
    e4_Node n1, n2, n3;
    int rank;

    /*
     * Test the e4_Node::SetNthNode and e4_Node::GetMarkedNode APIs.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test node6 failed (step 1)\n");
	    return 6;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test node6 failed (step 2)\n");
	    return 6;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test node6 failed (step 3)\n");
	    return 6;
	}
	if (!t.GetMarkedNode(n1)) {
	    fprintf(stderr, "test node6 failed (step 4)\n");
	    return 6;
	}
	if (!n1.IsValid()) {
	    fprintf(stderr, "test node6 failed (step 5)\n");
	    return 6;
	}
	if (!n1.AddVertex("f", E4_IOLAST, rank, (int) 33)) {
	    fprintf(stderr, "test node6 failed (step 6)\n");
	    return 6;
	}
	if (n1.VertexCount() != 1) {
	    fprintf(stderr, "test node6 failed (step 7)\n");
	    return 6;
	}
	if (!n1.SetNthNode("f", 1, n2)) {
	    fprintf(stderr, "test node6 failed (step 8)\n");
	    return 6;
	}
	if (!n2.IsValid()) {
	    fprintf(stderr, "test node6 failed (step 9)\n");
	    return 6;
	}
	if (n2.VertexCount() != 0) {
	    fprintf(stderr, "test node6 failed (step 10)\n");
	    return 6;
	}
	if (n1.VertexCount() != 1) {
	    fprintf(stderr, "test node6 failed (step 11)\n");
	    return 6;
	}
	if (!n2.AddVertex("f", E4_IOLAST, rank, (int) 33)) {
	    fprintf(stderr, "test node6 failed (step 12)\n");
	    return 6;
	}
	if (n2.VertexCount() != 1) {
	    fprintf(stderr, "test node6 failed (step 13)\n");
	    return 6;
	}
	if (!n2.SetNthNode("f", 1, n3)) {
	    fprintf(stderr, "test node6 failed (step 14)\n");
	    return 6;
	}
	if (!n3.IsValid()) {
	    fprintf(stderr, "test node6 failed (step 15)\n");
	    return 6;
	}
	if (n3.VertexCount() != 0) {
	    fprintf(stderr, "test node6 failed (step 16)\n");
	    return 6;
	}

	/*
	 * Steps 17, 18 and 19 have been removed.
	 */

	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node6 failed (step 20)\n");
	    return 6;
	}
    }
    if (t.IsValid() || n1.IsValid() || n2.IsValid() || n3.IsValid()) {
	fprintf(stderr, "test node6 failed (step 21)\n");
	return 6;
    }

    return 0;
}

static int
test_node7()
{
    e4_Marker t;
    e4_Node n1, n2, n3;
    e4_Vertex v1, v2, v3;
    int rank;

    /*
     * Test e4_Node::SetNode.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test node7 failed (step 1)\n");
	    return 7;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test node7 failed (step 2)\n");
	    return 7;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test node7 failed (step 3)\n");
	    return 7;
	}
	if (!t.GetMarkedNode(n1)) {
	    fprintf(stderr, "test node7 failed (step 4)\n");
	    return 7;
	}
	if (!n1.IsValid()) {
	    fprintf(stderr, "test node7 failed (step 5)\n");
	    return 7;
	}
	if (!n1.AddVertex("f1", E4_IOFIRST, rank, 0)) {
	    fprintf(stderr, "test node7 failed (step 6)\n");
	    return 7;
	}
	if (!n1.SetNode("f1", n2)) {
	    fprintf(stderr, "test node7 failed (step 7)\n");
	    return 7;
	}
	if (!n2.IsValid()) {
	    fprintf(stderr, "test node7 failed (step 8)\n");
	    return 7;
	}
	if (!n2.GetParent(n3)) {
	    fprintf(stderr, "test node7 failed (step 9)\n");
	    return 7;
	}
	if (!n3.IsValid()) {
	    fprintf(stderr, "test node7 failed (step 10)\n");
	    return 7;
	}
	if (n3 != n1) {
	    fprintf(stderr, "test node7 failed (step 11)\n");
	    return 7;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node7 failed (step 12)\n");
	    return 7;
	}
    }
    if (t.IsValid() || n1.IsValid() || n2.IsValid() || n3.IsValid()) {
	fprintf(stderr, "test node7 failed (step 13)\n");
	return 7;
    }
    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test node7 failed (step 14)\n");
	    return 7;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test node7 failed (step 15)\n");
	    return 7;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test node7 failed (step 16)\n");
	    return 7;
	}
	if (!t.GetMarkedNode(n1)) {
	    fprintf(stderr, "test node7 failed (step 17)\n");
	    return 7;
	}
	if (!n1.IsValid()) {
	    fprintf(stderr, "test node7 failed (step 18)\n");
	    return 7;
	}
	if ((!n1.AddNodeRef("f1", E4_IOFIRST, rank, n2, v2)) ||
	    (!n2.IsValid()) || (!v2.IsValid())) {
	    fprintf(stderr, "test node7 failed (step 19)\n");
	    return 7;
	}
	if (!n2.GetParent(n3)) {
	    fprintf(stderr, "test node7 failed (step 20)\n");
	    return 7;
	}
	if (!n3.IsValid()) {
	    fprintf(stderr, "test node7 failed (step 21)\n");
	    return 7;
	}
	if (n3 != n1) {
	    fprintf(stderr, "test node7 failed (step 22)\n");
	    return 7;
	}
	if ((!v2.GetNode(n3)) || (!n3.IsValid()) || (n3 != n1)) {
	    fprintf(stderr, "test node7 failed (step 23)\n");
	    return 7;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node7 failed (step 24)\n");
	    return 7;
	}
    }
    if (t.IsValid() || n1.IsValid() || n2.IsValid() || n3.IsValid() ||
	v2.IsValid()) {
	fprintf(stderr, "test node7 failed (step 25)\n");
	return 7;
    }


    return 0;
}

static int
test_node8()
{
    e4_Marker t;
    e4_Node n1, n2;
    e4_Node subnodes[TN_VERTEXLIMIT];
    int i;
    int rank;
    char buf[32];

    /*
     * Test e4_Node::AddNode.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test node8 failed (step 1)\n");
	    return 8;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test node8 failed (step 2)\n");
	    return 8;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test node8 failed (step 3)\n");
	    return 8;
	}
	if (!t.GetMarkedNode(n1)) {
	    fprintf(stderr, "test node8 failed (step 4)\n");
	    return 8;
	}
	if (!n1.IsValid()) {
	    fprintf(stderr, "test node8 failed (step 5)\n");
	    return 8;
	}
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "n%d", i);
	    if (!n1.AddNode(buf, E4_IOLAST, rank, subnodes[i])) {
	        fprintf(stderr, "test node8 failed (step 6.1.%d)\n", i);
		return 8;
	    }
	    if (!subnodes[i].IsValid()) {
		fprintf(stderr, "test node8 failed (step 6.2.%d)\n", i);
		return 8;
	    }
	    if (rank != i + 1) {
		fprintf(stderr, "test node8 failed (step 6.3.%d)\n", i);
		return 8;
	    }
	    if (n1.VertexCount() != i + 1) {
		fprintf(stderr, "test node8 failed (step 6.4.%d)\n", i);
		return 8;
	    }
	    if (subnodes[i].VertexCount() != 0) {
	        fprintf(stderr, "test node8 failed (step 6.5.%d)\n", i);
		return 8;
	    }
	    if (!subnodes[i].GetParent(n2)) {
		fprintf(stderr, "test node8 failed (step 6.6.%d)\n", i);
		return 8;
	    }
	    if (!n2.IsValid()) {
		fprintf(stderr, "test node8 failed (step 6.7.%d)\n", i);
		return 8;
	    }
	    if (n1 != n2) {
		fprintf(stderr, "test node8 failed (step 6.8.%d)\n", i);
		return 8;
	    }
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node8 failed (step 7)\n");
	    return 8;
	}
    }
    if (t.IsValid() || n1.IsValid() || n2.IsValid()) {
	fprintf(stderr, "test node8 failed (step 8)\n");
	return 8;
    }

    return 0;
}

static int
test_node9()
{
    e4_Marker t;
    e4_Node n1, n2, n3, n4;
    int i;
    int rank;
    char buf[32];

    /*
     * Test e4_Node::SetNodeByRank, e4_Node::GetVertexByRank,
     * e4_Node::GetParent.
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node9 failed (step 1)\n");
            return 9;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node9 failed (step 2)\n");
            return 9;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node9 failed (step 3)\n");
            return 9;
        }
        if (!t.GetMarkedNode(n1)) {
            fprintf(stderr, "test node9 failed (step 4)\n");
            return 9;
        }
        if (!n1.IsValid()) {
            fprintf(stderr, "test node9 failed (step 5)\n");
            return 9;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
            sprintf(buf, "f%d", i);
            if (!n1.AddVertex(buf, E4_IOLAST, rank, i)) {
                fprintf(stderr, "test node9 failed (step 6.1.%d)\n", i);
                return 9;
            }
            if (rank != i + 1) {
                fprintf(stderr, "test node9 failed (step 6.2.%d)\n", i);
                return 9;
            }
            if (n1.VertexCount() != i + 1) {
                fprintf(stderr, "test node9 failed (step 6.3.%d)\n", i);
                return 9;
            }
        }
        if (!n1.SetNodeByRank(22, n2)) {
            fprintf(stderr, "test node9 failed (step 7)\n");
            return 9;
        }
        if (!n2.IsValid()) {
            fprintf(stderr, "test node9 failed (step 8)\n");
            return 9;
        }
        if (n1.VertexCount() != TN_VERTEXLIMIT) {
            fprintf(stderr, "test node9 failed (step 9)\n");
            return 9;
        }
        if (!n2.GetParent(n3)) {
            fprintf(stderr, "test node9 failed (step 10)\n");
            return 9;
        }
        if (!n3.IsValid()) {
            fprintf(stderr, "test node9 failed (step 11)\n");
            return 9;
        }
        if (n1 != n3) {
            fprintf(stderr, "test node9 failed (step 12)\n");
            return 9;
        }
        if (!n1.GetVertexByRank(22, n4)) {
            fprintf(stderr, "test node9 failed (step 13)\n");
            return 9;
        }
        if (!n4.IsValid()) {
            fprintf(stderr, "test node9 failed (step 14)\n");
            return 9;
        }
        if (n4 != n2) {
            fprintf(stderr, "test node9 failed (step 15)\n");
            return 9;
        }
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node9 failed (step 16)\n");
	    return 9;
	}
    }
    if (t.IsValid() || n1.IsValid() || n2.IsValid() || n3.IsValid() ||
        n4.IsValid()) {
        fprintf(stderr, "test node9 failed (step 17)\n");
        return 9;
    }

    return 0;
}

static int
test_node10()
{
    e4_Marker t;
    e4_Node n1;
    int i, j;
    int rank;
    e4_Value v;

    /*
     * Test e4_Node::SetNthVertex, e4_Node::GetNthVertex with e4_Value 
     * argument, and e4_Node::VertexTypeByRank.
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node10 failed (step 1)\n");
            return 10;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node10 failed (step 2)\n");
            return 10;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node10 failed (step 3)\n");
            return 10;
        }
        if (!t.GetMarkedNode(n1)) {
            fprintf(stderr, "test node10 failed (step 4)\n");
            return 10;
        }
        if (!n1.IsValid()) {
            fprintf(stderr, "test node10 failed (step 5)\n");
            return 10;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
            if (!n1.AddVertex("f1", E4_IOLAST, rank, i)) {
                fprintf(stderr, "test node10 failed (step 6.1.%d)\n", i);
                return 10;
            }
            if (n1.VertexCount() != i + 1) {
                fprintf(stderr, "test node10 failed (step 6.2.%d)\n", i);
                return 10;
            }
            if (rank != i + 1) {
                fprintf(stderr, "test node10 failed (step 6.3.%d)\n", i);
                return 10;
            }
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
            j = i % (int) E4_VTLASTVERTEXTYPE;
            switch ((e4_VertexType) j) {
            case E4_VTNODE:
            case E4_VTINT:
                if (!n1.SetNthVertex("f1", i + 1, i)) {
                    fprintf(stderr, "test node10 failed (step 7.1.%d)\n", i);
                    return 10;
                }
                if (n1.VertexTypeByRank(i + 1) != E4_VTINT) {
                    fprintf(stderr, "test node10 failed (step 7.2.%d)\n", i);
                    return 10;
                }
                if (!n1.GetNthVertex("f1", i + 1, v)) {
                    fprintf(stderr, "test node10 failed (step 7.3.%d)\n", i);
                    return 10;
                }
                if (v.vertexType != E4_VTINT) {
                    fprintf(stderr, "test node10 failed (step 7.4.%d)\n", i);
                    return 10;
                }
                if (v.u.i != i) {
                    fprintf(stderr, "test node10 failed (step 7.5.%d, %d)\n", 
			    i, v.u.i);
                    return 10;
                }
                break;
            case E4_VTDOUBLE:
                if (!n1.SetNthVertex("f1", i + 1, (double) 3.145678)) {
                    fprintf(stderr, "test node10 failed (step 7.1.%d)\n", i);
                    return 10;
                }
                if (n1.VertexTypeByRank(i + 1) != E4_VTDOUBLE) {
                    fprintf(stderr, "test node10 failed (step 7.2.%d)\n", i);
                    return 10;
                }
                if (!n1.GetNthVertex("f1", i + 1, v)) {
                    fprintf(stderr, "test node10 failed (step 7.3.%d)\n", i);
                    return 10;
                }
                if (v.vertexType != E4_VTDOUBLE) {
                    fprintf(stderr, "test node10 failed (step 7.4.%d)\n", i);
                    return 10;
                }
                break;
            case E4_VTSTRING:
                if (!n1.SetNthVertex("f1", i + 1, "hello there")) {
                    fprintf(stderr, "test node10 failed (step 7.1.%d)\n", i);
                    return 10;
                }
                if (n1.VertexTypeByRank(i + 1) != E4_VTSTRING) {
                    fprintf(stderr, "test node10 failed (step 7.2.%d)\n", i);
                    return 10;
                }
                if (!n1.GetNthVertex("f1", i + 1, v)) {
                    fprintf(stderr, "test node10 failed (step 7.3.%d)\n", i);
                    return 10;
                }
                if (v.vertexType != E4_VTSTRING) {
                    fprintf(stderr, "test node10 failed (step 7.4.%d)\n", i);
                    return 10;
                }
                if (strcmp(v.u.s, "hello there") != 0) {
                    fprintf(stderr, "test node10 failed (step 7.5.%d, %s)\n", 
			    i, v.u.s);
                    return 10;
                }
                break;
            case E4_VTBINARY:
                if (!n1.SetNthVertex("f1", i + 1, (const void *) "hello there",
                                    12)) {
                    fprintf(stderr, "test node10 failed (step 7.1.%d)\n", i);
                    return 10;
                }
                if (n1.VertexTypeByRank(i + 1) != E4_VTBINARY) {
                    fprintf(stderr, "test node10 failed (step 7.2.%d)\n", i);
                    return 10;
                }
                if (!n1.GetNthVertex("f1", i + 1, v)) {
                    fprintf(stderr, "test node10 failed (step 7.3.%d)\n", i);
                    return 10;
                }
                if (v.vertexType != E4_VTBINARY) {
                    fprintf(stderr, "test node10 failed (step 7.4.%d)\n", i);
                    return 10;
                }
                if (strcmp((const char *) v.u.b.bytes, "hello there") != 0) {
                    fprintf(stderr, "test node10 failed (step 7.5.%d)\n", i);
                    return 10;
                }
                if (v.u.b.nbytes != 12) {
                    fprintf(stderr, "test node10 failed (step 7.6.%d)\n", i);
                    return 10;
                }
                break;
            default:
                fprintf(stderr, "test node10 failed (step 7.%d)\n", i);
                return 10;
            }
        }
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node10 failed (step 8)\n");
	    return 10;
	}
    }

    return 0;
}

static int
test_node11()
{
    e4_Marker t;
    e4_Node n;
    int i, j, k, rank;
    double f;
    const char *ss;
    const void *v;
    int nbytes;

    /*
     * Test e4_Node::SetNthVertex, e4_Node::GetNthVertex 
     * with various types, and e4_Node::VertexTypeByRank.
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node11 failed (step 1)\n");
            return 11;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node11 failed (step 2)\n");
            return 11;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node11 failed (step 3)\n");
            return 11;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node11 failed (step 4)\n");
            return 11;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node11 failed (step 5)\n");
            return 11;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
            if (!n.AddVertex("f1", E4_IOLAST, rank, i)) {
                fprintf(stderr, "test node11 failed (step 6.1.%d)\n", i);
                return 11;
            }
            if (n.VertexCount() != i + 1) {
                fprintf(stderr, "test node11 failed (step 6.2.%d)\n", i);
                return 11;
            }
            if (rank != i + 1) {
                fprintf(stderr, "test node11 failed (step 6.3.%d)\n", i);
                return 11;
            }
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
            j = i % (int) E4_VTLASTVERTEXTYPE;
            switch ((e4_VertexType) j) {
            case E4_VTNODE:
            case E4_VTINT:
                if (!n.SetNthVertex("f1", i + 1, i)) {
                    fprintf(stderr, "test node11 failed (step 7.1.%d)\n", i);
                    return 11;
                }
                if (n.VertexTypeByRank(i + 1) != E4_VTINT) {
                    fprintf(stderr, "test node11 failed (step 7.2.%d)\n", i);
                    return 11;
                }
                if (!n.GetNthVertex("f1", i + 1, k)) {
                    fprintf(stderr, "test node11 failed (step 7.3.%d)\n", i);
                    return 11;
                }
                if (k != i) {
                    fprintf(stderr, "test node11 failed (step 7.4.%d)\n", i);
                    return 11;
                }
                break;
            case E4_VTDOUBLE:
                if (!n.SetNthVertex("f1", i + 1, (double) 3.145678)) {
                    fprintf(stderr, "test node11 failed (step 7.1.%d)\n", i);
                    return 11;
                }
                if (n.VertexTypeByRank(i + 1) != E4_VTDOUBLE) {
                    fprintf(stderr, "test node11 failed (step 7.2.%d)\n", i);
                    return 11;
                }
                if (!n.GetNthVertex("f1", i + 1, f)) {
                    fprintf(stderr, "test node11 failed (step 7.3.%d)\n", i);
                    return 11;
                }
                break;
            case E4_VTSTRING:
                if (!n.SetNthVertex("f1", i + 1, "hello there")) {
                    fprintf(stderr, "test node11 failed (step 7.1.%d)\n", i);
                    return 11;
                }
                if (n.VertexTypeByRank(i + 1) != E4_VTSTRING) {
                    fprintf(stderr, "test node11 failed (step 7.2.%d)\n", i);
                    return 11;
                }
                if (!n.GetNthVertex("f1", i + 1, ss)) {
                    fprintf(stderr, "test node11 failed (step 7.3.%d)\n", i);
                    return 11;
                }
                if (strcmp("hello there", ss) != 0) {
                    fprintf(stderr, "test node11 failed (step 7.4.%d)\n", i);
                    return 11;
                }
                break;
            case E4_VTBINARY:
                if (!n.SetNthVertex("f1", i + 1, (const void *) "hello there",
                                   12)) {
                    fprintf(stderr, "test node11 failed (step 7.1.%d)\n", i);
                    return 11;
                }
                if (n.VertexTypeByRank(i + 1) != E4_VTBINARY) {
                    fprintf(stderr, "test node11 failed (step 7.2.%d)\n", i);
                    return 11;
                }
                if (!n.GetNthVertex("f1", i + 1, v, nbytes)) {
                    fprintf(stderr, "test node11 failed (step 7.3.%d)\n", i);
                    return 11;
                }
                if (strcmp((const char *) v, "hello there") != 0) {
                    fprintf(stderr, "test node11 failed (step 7.4.%d)\n", i);
                    return 11;
                }
                if (nbytes != 12) {
                    fprintf(stderr, "test node11 failed (step 7.5.%d)\n", i);
                    return 11;
                }
                break;
            default:
                fprintf(stderr, "test node11 failed (step 7.%d)\n", i);
                return 11;
            }
        }
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node11 failed (step 8)\n");
	    return 11;
	}
    }

    return 0;
}

static int
test_node12()
{
    e4_Marker t;
    e4_Node n;
    int i, j, k, rank;
    double f;
    const char *ss;
    const void *v;
    int nbytes;
    char nb[32];

    /*
     * Test e4_Node::SetVertex, e4_Node::GetVertexByRank with 
     * various types, and e4_Node::VertexTypeByRank.
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node12 failed (step 1)\n");
            return 12;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node12 failed (step 2)\n");
            return 12;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node12 failed (step 3)\n");
            return 12;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node12 failed (step 4)\n");
            return 12;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node12 failed (step 5)\n");
            return 12;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
            sprintf(nb, "f%d", i);
            if (!n.AddVertex(nb, E4_IOLAST, rank, i)) {
                fprintf(stderr, "test node12 failed (step 6.1.%d)\n", i);
                return 12;
            }
            if (n.VertexCount() != i + 1) {
                fprintf(stderr, "test node12 failed (step 6.2.%d)\n", i);
                return 12;
            }
            if (rank != i + 1) {
                fprintf(stderr, "test node12 failed (step 6.3.%d)\n", i);
                return 12;
            }
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
            sprintf(nb, "f%d", i);
            j = i % (int) E4_VTLASTVERTEXTYPE;
            switch ((e4_VertexType) j) {
            case E4_VTNODE:
            case E4_VTINT:
                if (!n.SetVertex(nb, i)) {
                    fprintf(stderr, "test node12 failed (step 7.1.%d)\n", i);
                    return 12;
                }
                if (n.VertexTypeByRank(i + 1) != E4_VTINT) {
                    fprintf(stderr, "test node12 failed (step 7.2.%d)\n", i);
                    return 12;
                }
                if (!n.GetVertexByRank(i + 1, k)) {
                    fprintf(stderr, "test node12 failed (step 7.3.%d)\n", i);
                    return 12;
                }
                if (k != i) {
                    fprintf(stderr, "test node12 failed (step 7.4.%d)\n", i);
                    return 12;
                }
                break;
            case E4_VTDOUBLE:
                if (!n.SetVertex(nb, (double) 3.145678)) {
                    fprintf(stderr, "test node12 failed (step 7.1.%d)\n", i);
                    return 12;
                }
                if (n.VertexTypeByRank(i + 1) != E4_VTDOUBLE) {
                    fprintf(stderr, "test node12 failed (step 7.2.%d)\n", i);
                    return 12;
                }
                if (!n.GetVertexByRank(i + 1, f)) {
                    fprintf(stderr, "test node12 failed (step 7.3.%d)\n", i);
                    return 12;
                }
                break;
            case E4_VTSTRING:
                if (!n.SetVertex(nb, "hello there")) {
                    fprintf(stderr, "test node12 failed (step 7.1.%d)\n", i);
                    return 12;
                }
                if (n.VertexTypeByRank(i + 1) != E4_VTSTRING) {
                    fprintf(stderr, "test node12 failed (step 7.2.%d)\n", i);
                    return 12;
                }
                if (!n.GetVertexByRank(i + 1, ss)) {
                    fprintf(stderr, "test node12 failed (step 7.3.%d)\n", i);
                    return 12;
                }
                if (strcmp("hello there", ss) != 0) {
                    fprintf(stderr, "test node12 failed (step 7.4.%d)\n", i);
                    return 12;
                }
                break;
            case E4_VTBINARY:
                if (!n.SetVertex(nb, (const void *) "hello there", 12)) {
                    fprintf(stderr, "test node12 failed (step 7.1.%d)\n", i);
                    return 12;
                }
                if (n.VertexTypeByRank(i + 1) != E4_VTBINARY) {
                    fprintf(stderr, "test node12 failed (step 7.2.%d)\n", i);
                    return 12;
                }
                if (!n.GetVertexByRank(i + 1, v, nbytes)) {
                    fprintf(stderr, "test node12 failed (step 7.3.%d)\n", i);
                    return 12;
                }
                if (strcmp((const char *) v, "hello there") != 0) {
                    fprintf(stderr, "test node12 failed (step 7.4.%d)\n", i);
                    return 12;
                }
                if (nbytes != 12) {
                    fprintf(stderr, "test node12 failed (step 7.5.%d)\n", i);
                    return 12;
                }
                break;
            default:
                fprintf(stderr, "test node12 failed (step 7.%d)\n", i);
                return 12;
            }
        }
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node12 failed (step 8)\n");
	    return 12;
	}
    }

    return 0;
}

static int
test_node13()
{
    e4_Marker t;
    e4_Node n;
    int i, j, rank;
    e4_Value v;
    char nb[32];

    /*
     * Test e4_Node::SetVertex, e4_Node::GetVertexByRank with e4_Value
     * and e4_Node::VertexTypeByRank.
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node13 failed (step 1)\n");
            return 13;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node13 failed (step 2)\n");
            return 13;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node13 failed (step 3)\n");
            return 13;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node13 failed (step 4)\n");
            return 13;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node13 failed (step 5)\n");
            return 13;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
            sprintf(nb, "f%d", i);
            if (!n.AddVertex(nb, E4_IOLAST, rank, i)) {
                fprintf(stderr, "test node13 failed (step 6.1.%d)\n", i);
                return 13;
            }
            if (n.VertexCount() != i + 1) {
                fprintf(stderr, "test node13 failed (step 6.2.%d)\n", i);
                return 13;
            }
            if (rank != i + 1) {
                fprintf(stderr, "test node13 failed (step 6.3.%d)\n", i);
                return 13;
            }
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
            sprintf(nb, "f%d", i);
            j = i % (int) E4_VTLASTVERTEXTYPE;
            switch ((e4_VertexType) j) {
            case E4_VTNODE:
            case E4_VTINT:
                if (!n.SetVertex(nb, i)) {
                    fprintf(stderr, "test node13 failed (step 7.1.%d)\n", i);
                    return 13;
                }
                if (n.VertexTypeByRank(i + 1) != E4_VTINT) {
                    fprintf(stderr, "test node13 failed (step 7.2.%d)\n", i);
                    return 13;
                }
                if (!n.GetVertexByRank(i + 1, v)) {
                    fprintf(stderr, "test node13 failed (step 7.3.%d)\n", i);
                    return 13;
                }
                if (v.vertexType != E4_VTINT) {
                    fprintf(stderr, "test node13 failed (step 7.4.%d)\n", i);
                    return 13;
                }
                if (v.u.i != i) {
                    fprintf(stderr, "test node13 failed (step 7.5.%d)\n", i);
                    return 13;
                }
                break;
            case E4_VTDOUBLE:
                if (!n.SetVertex(nb, (double) 3.145678)) {
                    fprintf(stderr, "test node13 failed (step 7.1.%d)\n", i);
                    return 13;
                }
                if (n.VertexTypeByRank(i + 1) != E4_VTDOUBLE) {
                    fprintf(stderr, "test node13 failed (step 7.2.%d)\n", i);
		return 13;
                }
                if (!n.GetVertexByRank(i + 1, v)) {
                    fprintf(stderr, "test node13 failed (step 7.3.%d)\n", i);
                    return 13;
                }
                if (v.vertexType != E4_VTDOUBLE) {
                    fprintf(stderr, "test node13 failed (step 7.5.%d)\n", i);
                    return 13;
                }
                break;
            case E4_VTSTRING:
                if (!n.SetVertex(nb, "hello there")) {
                    fprintf(stderr, "test node13 failed (step 7.1.%d)\n", i);
                    return 13;
                }
                if (n.VertexTypeByRank(i + 1) != E4_VTSTRING) {
                    fprintf(stderr, "test node13 failed (step 7.2.%d)\n", i);
                    return 13;
                }
                if (!n.GetVertexByRank(i + 1, v)) {
                    fprintf(stderr, "test node13 failed (step 7.3.%d)\n", i);
                    return 13;
                }
                if (v.vertexType != E4_VTSTRING) {
                    fprintf(stderr, "test node13 failed (step 7.4.%d)\n", i);
                    return 13;
                }
                if (strcmp("hello there", v.u.s) != 0) {
                    fprintf(stderr, "test node13 failed (step 7.5.%d)\n", i);
                    return 13;
                }
                break;
            case E4_VTBINARY:
                if (!n.SetVertex(nb, (const void *) "hello there", 12)) {
                    fprintf(stderr, "test node13 failed (step 7.1.%d)\n", i);
                    return 13;
                }
                if (n.VertexTypeByRank(i + 1) != E4_VTBINARY) {
                    fprintf(stderr, "test node13 failed (step 7.2.%d)\n", i);
                    return 13;
                }
                if (!n.GetVertexByRank(i + 1, v)) {
                    fprintf(stderr, "test node13 failed (step 7.3.%d)\n", i);
                    return 13;
                }
                if (v.vertexType != E4_VTBINARY) {
                    fprintf(stderr, "test node13 failed (step 7.4.%d)\n", i);
                    return 13;
                }
                if (v.u.b.nbytes != 12) {
                    fprintf(stderr, "test node13 failed (step 7.5.%d)\n", i);
                    return 13;
                }
                if (strcmp((const char *) v.u.b.bytes, "hello there") != 0) {
                    fprintf(stderr, "test node13 failed (step 7.6.%d)\n", i);
                    return 13;
                }
                break;
            default:
                fprintf(stderr, "test node13 failed (step 7.%d)\n", i);
                return 13;
            }
        }
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node13 failed (step 8)\n");
	    return 13;
	}
    }

    return 0;
}

static int
test_node14()
{
    e4_Marker t;
    e4_Node n;
    int i, j, k;
    double f;
    const char *ss;
    const void *bytes;
    int nbytes;
    int rank;

    /*
     * Test e4_Node::SetVertexByRank, e4_Node::GetNthVertex
     * and e4_Node::VertexTypeByRank.
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node14 failed (step 1)\n");
            return 14;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node14 failed (step 2)\n");
            return 14;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node14 failed (step 3)\n");
            return 14;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node14 failed (step 4)\n");
            return 14;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node14 failed (step 5)\n");
            return 14;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
            if (!n.AddVertex("f1", E4_IOLAST, rank, i)) {
                fprintf(stderr, "test node14 failed (step 6.1.%d)\n", i);
                return 14;
            }
            if (n.VertexCount() != i + 1) {
                fprintf(stderr, "test node14 failed (step 6.2.%d)\n", i);
                return 14;
            }
            if (rank != i + 1) {
                fprintf(stderr, "test node14 failed (step 6.3.%d)\n", i);
                return 14;
            }
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
            j = i % (int) E4_VTLASTVERTEXTYPE;
            switch ((e4_VertexType) j) {
            case E4_VTNODE:
            case E4_VTINT:
                if (!n.SetVertexByRank(i + 1, i)) {
                    fprintf(stderr, "test node14 failed (step 7.1.%d)\n", i);
                    return 14;
                }
                if (n.VertexTypeByRank(i + 1) != E4_VTINT) {
                    fprintf(stderr, "test node14 failed (step 7.2.%d)\n", i);
                    return 14;
                }
                if (!n.GetNthVertex("f1", i + 1, k)) {
                    fprintf(stderr, "test node14 failed (step 7.3.%d)\n", i);
                    return 14;
                }
                if (k != i) {
                    fprintf(stderr, "test node14 failed (step 7.4.%d)\n", i);
                    return 14;
                }
                break;
            case E4_VTDOUBLE:
                if (!n.SetVertexByRank(i + 1, (double) 3.14567)) {
                    fprintf(stderr, "test node14 failed (step 7.1.%d)\n", i);
                    return 14;
                }
                if (n.VertexTypeByRank(i + 1) != E4_VTDOUBLE) {
                    fprintf(stderr, "test node14 failed (step 7.2.%d)\n", i);
                    return 14;
                }
                if (!n.GetNthVertex("f1", i + 1, f)) {
                    fprintf(stderr, "test node14 failed (step 7.3.%d)\n", i);
                    return 14;
                }
                break;
            case E4_VTSTRING:
                if (!n.SetVertexByRank(i + 1, "hello there")) {
                    fprintf(stderr, "test node14 failed (step 7.1.%d)\n", i);
                    return 14;
                }
                if (n.VertexTypeByRank(i + 1) != E4_VTSTRING) {
                    fprintf(stderr, "test node14 failed (step 7.2.%d)\n", i);
                    return 14;
                }
                if (!n.GetNthVertex("f1", i + 1, ss)) {
                    fprintf(stderr, "test node14 failed (step 7.3.%d)\n", i);
                    return 14;
                }
                if (strcmp(ss, "hello there") != 0) {
                    fprintf(stderr, "test node14 failed (step 7.4.%d)\n", i);
                    return 14;
                }
                break;
            case E4_VTBINARY:
                if (!n.SetVertexByRank(i + 1, (const void *) "hello there",
                                      12)) {
                    fprintf(stderr, "test node14 failed (step 7.1.%d)\n", i);
                    return 14;
                }
                if (n.VertexTypeByRank(i + 1) != E4_VTBINARY) {
                    fprintf(stderr, "test node14 failed (step 7.2.%d)\n", i);
                    return 14;
                }
                if (!n.GetNthVertex("f1", i + 1, bytes, nbytes)) {
                    fprintf(stderr, "test node14 failed (step 7.3.%d)\n", i);
                    return 14;
                }
                if (strcmp((const char *) bytes, "hello there") != 0) {
                    fprintf(stderr, "test node14 failed (step 7.4.%d)\n", i);
                    return 14;
                }
                if (nbytes != 12) {
                    fprintf(stderr, "test node14 failed (step 7.5.%d)\n", i);
                    return 14;
                }
                break;
            default:
                fprintf(stderr, "test node14 failed (step 7.%d)\n", i);
                return 14;
            }
        }
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node14 failed (step 8)\n");
	    return 14;
	}
    }

    return 0;
}

static int
test_node15()
{
    e4_Marker t;
    e4_Node n;
    int i, j, k;
    double f;
    const char *ss;
    const void *bytes;
    int nbytes;
    int rank;

    /*
     * Testing e4_Node::AddVertex.
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);

        if (!s.IsValid()) {
            fprintf(stderr, "test node15 failed (step 1)\n");
            return 15;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node15 failed (step 2)\n");
            return 15;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node15 failed (step 3)\n");
            return 15;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node15 failed (step 4)\n");
            return 15;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node15 failed (step 5)\n");
            return 15;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
            j = i % (int) E4_VTLASTVERTEXTYPE;
            switch ((e4_VertexType) j) {
            case E4_VTNODE:
            case E4_VTINT:
                if (!n.AddVertex("f1", E4_IOLAST, rank, i)) {
                    fprintf(stderr, "test node15 failed (step 6.1.%d)\n", i);
                    return 15;
                }
                if (!n.GetVertexByRank(rank, k)) {
                    fprintf(stderr, "test node15 failed (step 6.2.%d)\n", i);
                    return 15;
                }
                if (k != i) {
                    fprintf(stderr, "test node15 failed (step 6.3.%d)\n", i);
                    return 15;
                }
                break;
            case E4_VTDOUBLE:
                if (!n.AddVertex("f1", E4_IOLAST, rank, (double) 3.145678)) {
                    fprintf(stderr, "test node15 failed (step 6.1.%d)\n", i);
                    return 15;
                }
                if (!n.GetVertexByRank(rank, f)) {
                    fprintf(stderr, "test node15 failed (step 6.2.%d)\n", i);
                    return 15;
                }
                break;
            case E4_VTSTRING:
                if (!n.AddVertex("f1", E4_IOLAST, rank, "hello there")) {
                    fprintf(stderr, "test node15 failed (step 6.1.%d)\n", i);
                    return 15;
                }
                if (!n.GetVertexByRank(rank, ss)) {
                    fprintf(stderr, "test node15 failed (step 6.2.%d)\n", i);
                    return 15;
                }
                if (strcmp(ss, "hello there") != 0) {
                    fprintf(stderr, "test node15 failed (step 6.3.%d)\n", i);
                    return 15;
                }
                break;
            case E4_VTBINARY:
                if (!n.AddVertex("f1", E4_IOLAST, rank,
                                (const void *) "hello there", 12)) {
                    fprintf(stderr, "test node15 failed (step 6.1.%d)\n", i);
                    return 15;
                }
                if (!n.GetVertexByRank(rank, bytes, nbytes)) {
                    fprintf(stderr, "test node15 failed (step 6.2.%d)\n", i);
                    return 15;
                }
                if (strcmp((const char *) bytes, "hello there") != 0) {
                    fprintf(stderr, "test node15 failed (step 6.3.%d)\n", i);
                    return 15;
                }
                if (nbytes != 12) {
                    fprintf(stderr, "test node15 failed (step 6.4.%d)\n", i);
                    return 15;
                }
                break;
            }
        }
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node15 failed (step 7)\n");
	    return 15;
	}
    }

    return 0;
}

static int
test_node16()
{
    e4_Marker t;
    e4_Node n;
    e4_Vertex f;
    int i, j, rank;

    /*
     * Test e4_Node::AddVertexRef.
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node16 failed (step 1)\n");
            return 16;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node16 failed (step 2)\n");
            return 16;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node16 failed (step 3)\n");
            return 16;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node16 failed (step 4)\n");
            return 16;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node16 failed (step 5)\n");
            return 16;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
            j = i % (int) E4_VTLASTVERTEXTYPE;
            switch ((e4_VertexType) j) {
            case E4_VTNODE:
            case E4_VTINT:
                if (!n.AddVertexRef("f1", E4_IOLAST, rank, i, f)) {
                    fprintf(stderr, "test node16 failed (step 6.1.%d)\n", i);
                    return 16;
                }
                if (!f.IsValid()) {
                    fprintf(stderr, "test node16 failed (step 6.2.%d)\n", i);
                    return 16;
                }
                if (rank != i + 1) {
                    fprintf(stderr, "test node16 failed (step 6.3.%d)\n", i);
                    return 16;
                }
                if (n.VertexCount() != i + 1) {
                    fprintf(stderr, "test node16 failed (step 6.4.%d)\n", i);
                    return 16;
                }
                break;
            case E4_VTDOUBLE:
                if (!n.AddVertexRef("f1", E4_IOLAST, rank, (double) 3.16, f)) {
                    fprintf(stderr, "test node16 failed (step 6.1.%d)\n", i);
                    return 16;
                }
                if (!f.IsValid()) {
                    fprintf(stderr, "test node16 failed (step 6.2.%d)\n", i);
                    return 16;
                }
                if (rank != i + 1) {
                    fprintf(stderr, "test node16 failed (step 6.3.%d)\n", i);
                    return 16;
                }
                if (n.VertexCount() != i + 1) {
                    fprintf(stderr, "test node16 failed (step 6.4.%d)\n", i);
                    return 16;
                }
                break;
            case E4_VTSTRING:
                if (!n.AddVertexRef("f1", E4_IOLAST, rank, "hello there", f)) {
                    fprintf(stderr, "test node16 failed (step 6.1.%d)\n", i);
                    return 16;
                }
                if (!f.IsValid()) {
                    fprintf(stderr, "test node16 failed (step 6.2.%d)\n", i);
                    return 16;
                }
                if (rank != i + 1) {
                    fprintf(stderr, "test node16 failed (step 6.3.%d)\n", i);
                    return 16;
                }
                if (n.VertexCount() != i + 1) {
                    fprintf(stderr, "test node16 failed (step 6.4.%d)\n", i);
                    return 16;
                }
                break;
            case E4_VTBINARY:
                if (!n.AddVertexRef("f1", E4_IOLAST, rank,
                                   (const void *) "hello there", 12, f)) {
                    fprintf(stderr, "test node16 failed (step 6.1.%d)\n", i);
                    return 16;
                }
                if (!f.IsValid()) {
                    fprintf(stderr, "test node16 failed (step 6.2.%d)\n", i);
                    return 16;
                }
                if (rank != i + 1) {
                    fprintf(stderr, "test node16 failed (step 6.3.%d)\n", i);
                    return 16;
                }
                if (n.VertexCount() != i + 1) {
                    fprintf(stderr, "test node16 failed (step 6.4.%d)\n", i);
                    return 16;
                }
                break;
            default:
                fprintf(stderr, "test node16 failed (step 6.%d)\n", i);
                return 16;
            }
        }
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node16 failed (step 7)\n");
	    return 16;
	}
    }

    return 0;
}

static int
test_node17()
{
    e4_Marker t;
    e4_Node n1, n2, n3;
    e4_Vertex f;
    int rank;

    /*
     * Testing e4_Node::AddNodeRef.
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node17 failed (step 1)\n");
            return 17;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node17 failed (step 2)\n");
            return 17;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node17 failed (step 3)\n");
            return 17;
        }
        if (!t.GetMarkedNode(n1)) {
            fprintf(stderr, "test node17 failed (step 4)\n");
            return 17;
        }
        if (!n1.IsValid()) {
            fprintf(stderr, "test node17 failed (step 5)\n");
            return 17;
        }
        if (!n1.AddNodeRef("f1", E4_IOFIRST, rank, n2, f)) {
            fprintf(stderr, "test node17 failed (step 6)\n");
            return 17;
        }
        if (!n2.IsValid()) {
            fprintf(stderr, "test node17 failed (step 7)\n");
            return 17;
        }
        if (!n2.GetParent(n3)) {
            fprintf(stderr, "test node17 failed (step 8)\n");
            return 17;
        }
        if (!n3.IsValid()) {
            fprintf(stderr, "test node17 failed (step 9)\n");
            return 17;
        }
        if (n1 != n3) {
            fprintf(stderr, "test node17 failed (step 10)\n");
            return 17;
        }
        if (!f.IsValid()) {
            fprintf(stderr, "test node17 failed (step 11)\n");
            return 17;
        }
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node17 failed (step 12)\n");
	    return 17;
	}
    }

    return 0;
}
    
static int
test_node18()
{
    e4_Marker t;
    e4_Node n;
    int i, j;
    int rank;
    char buf[32];

    /*
     * Testing e4_Node::GetVertex().
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test node18 failed (step 1)\n");
	    return 18;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test node18 failed (step 2)\n");
	    return 18;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test node18 failed (step 3)\n");
	    return 18;
	}
	if (!t.GetMarkedNode(n)) {
	    fprintf(stderr, "test node18 failed (step 4)\n");
	    return 18;
	}
	if (!n.IsValid()) {
	    fprintf(stderr, "test node18 failed (step 5)\n");
	    return 18;
	}
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.AddVertex(buf, E4_IOLAST, rank, i + 1)) {
		fprintf(stderr, "test node18 failed (step 6.1.%d)\n", i);
		return 18;
	    }
	    if (rank != i + 1) {
		fprintf(stderr, "test node18 failed (step 6.2.%d)\n", i);
		return 18;
	    }
	    if (n.VertexCount() != i + 1) {
		fprintf(stderr, "test node18 failed (step 6.3.%d)\n", i);
		return 18;
	    }
	}
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.GetVertex(buf, j)) {
		fprintf(stderr, "test node18 failed (step 7.1.%d)\n", i);
		return 18;
	    }
	    if (j != i + 1) {
		fprintf(stderr, "test node18 failed (step 7.2.%d)\n", i);
		return 18;
	    }
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node18 failed (step 8)\n");
	    return 18;
	}
    }

    return 0;
}
    
static int
test_node19()
{
    e4_Marker t;
    e4_Node n;
    int i, j;
    int rank;
    char buf[32];

    /*
     * Testing e4_Node::DeleteVertex().
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test node19 failed (step 1)\n");
	    return 19;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test node19 failed (step 2)\n");
	    return 19;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test node19 failed (step 3)\n");
	    return 19;
	}
	if (!t.GetMarkedNode(n)) {
	    fprintf(stderr, "test node19 failed (step 4)\n");
	    return 19;
	}
	if (!n.IsValid()) {
	    fprintf(stderr, "test node19 failed (step 5)\n");
	    return 19;
	}
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.AddVertex(buf, E4_IOLAST, rank, i + 1)) {
		fprintf(stderr, "test node19 failed (step 6.1.%d)\n", i);
		return 19;
	    }
	    if (rank != i + 1) {
		fprintf(stderr, "test node19 failed (step 6.2.%d)\n", i);
		return 19;
	    }
	    if (n.VertexCount() != i + 1) {
		fprintf(stderr, "test node19 failed (step 6.3.%d)\n", i);
		return 19;
	    }
	}
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.GetVertex(buf, j)) {
		fprintf(stderr, "test node19 failed (step 7.1.%d)\n", i);
		return 19;
	    }
	    if (j != i + 1) {
		fprintf(stderr, "test node19 failed (step 7.2.%d)\n", i);
		return 19;
	    }
	}
	for (i = TN_VERTEXLIMIT; i > 0; i--) {
	    sprintf(buf, "f%d", i);
	    if (!n.DeleteVertex(buf)) {
		fprintf(stderr, "test node19 failed (step 8.1.%d)\n", i);
		return 19;
	    }
	    if (n.GetVertex(buf, j)) {
		fprintf(stderr, "test node19 failed (step 8.2.%d), %d %d %s\n",
			i, j, n.VertexCount(), buf);
		return 19;
	    }
	    if (!n.IsValid()) {
	      fprintf(stderr, "Smashed node\n");
	    }
	    if (n.VertexCount() != i - 1) {
	        fprintf(stderr, "test node19 failed (step 8.3.%d) %d\n", 
			i, n.VertexCount());
		return 19;
	    }
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node19 failed (step 9)\n");
	    return 19;
	}
    }

    return 0;
}
    
static int
test_node20()
{
    e4_Marker t;
    e4_Node n;
    int i, j;
    int rank;
    char buf[32];

    /*
     * Testing e4_Node::DeleteVertexByRank().
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test node20 failed (step 1)\n");
	    return 20;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test node20 failed (step 2)\n");
	    return 20;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test node20 failed (step 3)\n");
	    return 20;
	}
	if (!t.GetMarkedNode(n)) {
	    fprintf(stderr, "test node20 failed (step 4)\n");
	    return 20;
	}
	if (!n.IsValid()) {
	    fprintf(stderr, "test node20 failed (step 5)\n");
	    return 20;
	}
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.AddVertex(buf, E4_IOLAST, rank, i + 1)) {
		fprintf(stderr, "test node20 failed (step 6.1.%d)\n", i);
		return 20;
	    }
	    if (rank != i + 1) {
		fprintf(stderr, "test node20 failed (step 6.2.%d)\n", i);
		return 20;
	    }
	    if (n.VertexCount() != i + 1) {
		fprintf(stderr, "test node20 failed (step 6.3.%d)\n", i);
		return 20;
	    }
	}
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.GetVertex(buf, j)) {
		fprintf(stderr, "test node20 failed (step 7.1.%d)\n", i);
		return 20;
	    }
	    if (j != i + 1) {
		fprintf(stderr, "test node20 failed (step 7.2.%d)\n", i);
		return 20;
	    }
	}
	for (i = TN_VERTEXLIMIT; i > 0; i--) {
	    sprintf(buf, "f%d", i);
	    if (!n.DeleteVertexByRank(i)) {
		fprintf(stderr, "test node20 failed (step 8.1.%d)\n", i);
		return 20;
	    }
	    if (n.GetVertex(buf, j)) {
		fprintf(stderr, "test node20 failed (step 8.2.%d), %d, %d\n", 
			i, j, n.VertexCount());
		return 20;
	    }
	    if (n.VertexCount() != i - 1) {
	        fprintf(stderr, "test node20 failed (step 8.3.%d)\n", i);
		return 20;
	    }
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node20 failed (step 9)\n");
	    return 20;
	}
    }

    return 0;
}
    
static int
test_node21()
{
    e4_Marker t;
    e4_Node n;
    e4_Vertex f;
    int i, j;
    int rank;
    char buf[32];
    char *ss;

    /*
     * Testing e4_Node::GetVertexRef().
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test node21 failed (step 1)\n");
	    return 21;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test node21 failed (step 2)\n");
	    return 21;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test node21 failed (step 3)\n");
	    return 21;
	}
	if (!t.GetMarkedNode(n)) {
	    fprintf(stderr, "test node21 failed (step 4)\n");
	    return 21;
	}
	if (!n.IsValid()) {
	    fprintf(stderr, "test node21 failed (step 5)\n");
	    return 21;
	}
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.AddVertex(buf, E4_IOLAST, rank, i + 1)) {
		fprintf(stderr, "test node21 failed (step 6.1.%d)\n", i);
		return 21;
	    }
	    if (rank != i + 1) {
		fprintf(stderr, "test node21 failed (step 6.2.%d)\n", i);
		return 21;
	    }
	    if (n.VertexCount() != i + 1) {
		fprintf(stderr, "test node21 failed (step 6.3.%d)\n", i);
		return 21;
	    }
	}
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.GetVertex(buf, j)) {
		fprintf(stderr, "test node21 failed (step 7.1.%d)\n", i);
		return 21;
	    }
	    if (j != i + 1) {
		fprintf(stderr, "test node21 failed (step 7.2.%d)\n", i);
		return 21;
	    }
	}
	for (i = TN_VERTEXLIMIT; i > 0; i--) {
	    sprintf(buf, "f%d", i);
	    if (!n.GetVertexRef(buf, f)) {
		fprintf(stderr, "test node21 failed (step 8.1.%d)\n", i);
		return 21;
	    }
	    if (!f.IsValid()) {
		fprintf(stderr, "test node21 failed (step 8.2.%d)\n", i);
		return 21;
	    }
	    ss = (char *) f.Name();
	    if (ss == NULL) {
		fprintf(stderr, "test node21 failed (step 8.3.%d)\n", i);
		return 21;
	    }
	    if (strcmp(buf, ss) != 0) {
		fprintf(stderr, "test node21 failed (step 8.4.%d, %s %s)\n",
			i, buf, ss);
		return 21;
	    }
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node21 failed (step 9)\n");
	    return 21;
	}
    }

    return 0;
}

static int
test_node22()
{
    e4_Marker t;
    e4_Node n;
    int i, j, rank;

    /*
     * Test e4_Node::VertexType.
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node22 failed (step 1)\n");
            return 22;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node22 failed (step 2)\n");
            return 22;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node22 failed (step 3)\n");
            return 22;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node22 failed (step 4)\n");
            return 22;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node22 failed (step 5)\n");
            return 22;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
            j = i % (int) E4_VTLASTVERTEXTYPE;
            switch ((e4_VertexType) j) {
            case E4_VTNODE:
            case E4_VTINT:
                if (!n.AddVertex("f1", E4_IOLAST, rank, i)) {
                    fprintf(stderr, "test node22 failed (step 6.1.%d)\n", i);
                    return 22;
                }
                if (rank != i + 1) {
                    fprintf(stderr, "test node22 failed (step 6.2.%d)\n", i);
                    return 22;
                }
                if (n.VertexCount() != i + 1) {
                    fprintf(stderr, "test node22 failed (step 6.3.%d)\n", i);
                    return 22;
                }
                break;
            case E4_VTDOUBLE:
                if (!n.AddVertex("f1", E4_IOLAST, rank, (double) 3.22)) {
                    fprintf(stderr, "test node22 failed (step 6.1.%d)\n", i);
                    return 22;
                }
                if (rank != i + 1) {
                    fprintf(stderr, "test node22 failed (step 6.2.%d)\n", i);
                    return 22;
                }
                if (n.VertexCount() != i + 1) {
                    fprintf(stderr, "test node22 failed (step 6.3.%d)\n", i);
                    return 22;
                }
                break;
            case E4_VTSTRING:
                if (!n.AddVertex("f1", E4_IOLAST, rank, "hello there")) {
                    fprintf(stderr, "test node22 failed (step 6.1.%d)\n", i);
                    return 22;
                }
                if (rank != i + 1) {
                    fprintf(stderr, "test node22 failed (step 6.2.%d)\n", i);
                    return 22;
                }
                if (n.VertexCount() != i + 1) {
                    fprintf(stderr, "test node22 failed (step 6.3.%d)\n", i);
                    return 22;
                }
                break;
            case E4_VTBINARY:
                if (!n.AddVertex("f1", E4_IOLAST, rank,
				(const void *) "hello there", 12)) {
                    fprintf(stderr, "test node22 failed (step 6.1.%d)\n", i);
                    return 22;
                }
                if (rank != i + 1) {
                    fprintf(stderr, "test node22 failed (step 6.2.%d)\n", i);
                    return 22;
                }
                if (n.VertexCount() != i + 1) {
                    fprintf(stderr, "test node22 failed (step 6.3.%d)\n", i);
                    return 22;
                }
                break;
            default:
                fprintf(stderr, "test node22 failed (step 6.%d)\n", i);
                return 22;
            }
        }
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
            j = i % (int) E4_VTLASTVERTEXTYPE;
	    if ((e4_VertexType) j == E4_VTNODE) {
		j = (int) E4_VTINT;
	    }
	    if ((e4_VertexType) j != n.VertexType("f1", i + 1)) {
	        fprintf(stderr, "test node22 failed (step 7.%d)\n", i);
		return 22;
	    }
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node22 failed (step 8)\n");
	    return 22;
	}
    }

    return 0;
}

static int
test_node23()
{
    e4_Marker t;
    e4_Node n;
    int i, j, rank;

    /*
     * Test e4_Node::VertexTypeByRank.
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node23 failed (step 1)\n");
            return 23;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node23 failed (step 2)\n");
            return 23;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node23 failed (step 3)\n");
            return 23;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node23 failed (step 4)\n");
            return 23;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node23 failed (step 5)\n");
            return 23;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
            j = i % (int) E4_VTLASTVERTEXTYPE;
            switch ((e4_VertexType) j) {
            case E4_VTNODE:
            case E4_VTINT:
                if (!n.AddVertex("f1", E4_IOLAST, rank, i)) {
                    fprintf(stderr, "test node23 failed (step 6.1.%d)\n", i);
                    return 23;
                }
                if (rank != i + 1) {
                    fprintf(stderr, "test node23 failed (step 6.2.%d)\n", i);
                    return 23;
                }
                if (n.VertexCount() != i + 1) {
                    fprintf(stderr, "test node23 failed (step 6.3.%d)\n", i);
                    return 23;
                }
                break;
            case E4_VTDOUBLE:
                if (!n.AddVertex("f1", E4_IOLAST, rank, (double) 3.23)) {
                    fprintf(stderr, "test node23 failed (step 6.1.%d)\n", i);
                    return 23;
                }
                if (rank != i + 1) {
                    fprintf(stderr, "test node23 failed (step 6.2.%d)\n", i);
                    return 23;
                }
                if (n.VertexCount() != i + 1) {
                    fprintf(stderr, "test node23 failed (step 6.3.%d)\n", i);
                    return 23;
                }
                break;
            case E4_VTSTRING:
                if (!n.AddVertex("f1", E4_IOLAST, rank, "hello there")) {
                    fprintf(stderr, "test node23 failed (step 6.1.%d)\n", i);
                    return 23;
                }
                if (rank != i + 1) {
                    fprintf(stderr, "test node23 failed (step 6.2.%d)\n", i);
                    return 23;
                }
                if (n.VertexCount() != i + 1) {
                    fprintf(stderr, "test node23 failed (step 6.3.%d)\n", i);
                    return 23;
                }
                break;
            case E4_VTBINARY:
                if (!n.AddVertex("f1", E4_IOLAST, rank,
				(const void *) "hello there", 12)) {
                    fprintf(stderr, "test node23 failed (step 6.1.%d)\n", i);
                    return 23;
                }
                if (rank != i + 1) {
                    fprintf(stderr, "test node23 failed (step 6.2.%d)\n", i);
                    return 23;
                }
                if (n.VertexCount() != i + 1) {
                    fprintf(stderr, "test node23 failed (step 6.3.%d)\n", i);
                    return 23;
                }
                break;
            default:
                fprintf(stderr, "test node23 failed (step 6.%d)\n", i);
                return 23;
            }
        }
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
            j = i % (int) E4_VTLASTVERTEXTYPE;
	    if ((e4_VertexType) j == E4_VTNODE) {
		j = (int) E4_VTINT;
	    }
	    if ((e4_VertexType) j != n.VertexTypeByRank(i + 1)) {
	        fprintf(stderr, "test node23 failed (step 7.%d)\n", i);
		return 23;
	    }
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node23 failed (step 8)\n");
	    return 23;
	}
    }

    return 0;
}

static int
test_node24()
{
    e4_Marker t;
    e4_Node n;
    int i, rank;
    char buf[32];
    const char *ss;

    /*
     * Test e4_Node::VertexName().
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node24 failed (step 1)\n");
            return 24;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node24 failed (step 2)\n");
            return 24;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node24 failed (step 3)\n");
            return 24;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node24 failed (step 4)\n");
            return 24;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node24 failed (step 5)\n");
            return 24;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.AddVertex(buf, E4_IOLAST, rank, i)) {
		fprintf(stderr, "test node24 failed (step 6.1.%d)\n", i);
		return 24;
	    }
	    if (rank != i + 1) {
	        fprintf(stderr, "test node24 failed (step 6.2.%d)\n", i);
		return 24;
	    }
	    if (n.VertexCount() != i + 1) {
	        fprintf(stderr, "test node24 failed (step 6.3.%d)\n", i);
		return 24;
	    }
        }
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    ss = n.VertexName(i + 1);
	    if (ss == NULL) {
		fprintf(stderr, "test node24 failed (step 7.1.%d)\n", i);
		return 24;
	    }
	    if (strcmp(ss, buf) != 0) {
		fprintf(stderr, "test node24 failed (step 7.2.%d)\n", i);
		return 24;
	    }
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node24 failed (step 8)\n");
	    return 24;
	}
    }

    return 0;
}

static int
test_node25()
{
    e4_Marker t;
    e4_Node n;
    int i, rank;
    char buf[32];
    const char *ss;

    /*
     * Test e4_Node::RenameVertex().
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node25 failed (step 1)\n");
            return 25;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node25 failed (step 2)\n");
            return 25;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node25 failed (step 3)\n");
            return 25;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node25 failed (step 4)\n");
            return 25;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node25 failed (step 5)\n");
            return 25;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.AddVertex(buf, E4_IOLAST, rank, i)) {
		fprintf(stderr, "test node25 failed (step 6.1.%d)\n", i);
		return 25;
	    }
	    if (rank != i + 1) {
	        fprintf(stderr, "test node25 failed (step 6.2.%d)\n", i);
		return 25;
	    }
	    if (n.VertexCount() != i + 1) {
	        fprintf(stderr, "test node25 failed (step 6.3.%d)\n", i);
		return 25;
	    }
        }
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    ss = n.VertexName(i + 1);
	    if (ss == NULL) {
		fprintf(stderr, "test node25 failed (step 7.1.%d)\n", i);
		return 25;
	    }
	    if (strcmp(ss, buf) != 0) {
		fprintf(stderr, "test node25 failed (step 7.2.%d)\n", i);
		return 25;
	    }
	}
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "g%d", i + 1);
	    if (!n.RenameVertex(i + 1, buf)) {
		fprintf(stderr, "test node25 failed (step 8)\n");
		return 25;
	    }
	}
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "g%d", i + 1);
	    ss = n.VertexName(i + 1);
	    if (ss == NULL) {
		fprintf(stderr, "test node25 failed (step 9.1.%d)\n", i);
		return 25;
	    }
	    if (strcmp(ss, buf) != 0) {
		fprintf(stderr, "test node25 failed (step 9.2.%d)\n", i);
		return 25;
	    }
	}
    }
    if (n.IsValid() || t.IsValid()) {
	fprintf(stderr, "test node25 failed (step 10)\n");
	return 25;
    }
    {
        e4_Storage s("foo.db", E4_METAKIT);

        if (!s.IsValid()) {
            fprintf(stderr, "test node25 failed (step 11)\n");
            return 25;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node25 failed (step 12)\n");
            return 25;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node25 failed (step 13)\n");
            return 25;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node25 failed (step 14)\n");
            return 25;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node25 failed (step 15)\n");
            return 25;
        }
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "g%d", i + 1);
	    ss = n.VertexName(i + 1);
	    if (ss == NULL) {
		fprintf(stderr, "test node25 failed (step 16.1.%d)\n", i);
		return 25;
	    }
	    if (strcmp(ss, buf) != 0) {
		fprintf(stderr, "test node25 failed (step 16.2.%d)\n", i);
		return 25;
	    }
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node25 failed (step 17)\n");
	    return 25;
	}
    }

    return 0;
}

static int
test_node26()
{
    e4_Marker t;
    e4_Node n;
    int i, rank;
    char buf[32];

    /*
     * Test e4_Node::VertexRank().
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node26 failed (step 1)\n");
            return 26;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node26 failed (step 2)\n");
            return 26;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node26 failed (step 3)\n");
            return 26;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node26 failed (step 4)\n");
            return 26;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node26 failed (step 5)\n");
            return 26;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.AddVertex(buf, E4_IOLAST, rank, i)) {
		fprintf(stderr, "test node26 failed (step 6.1.%d)\n", i);
		return 26;
	    }
	    if (rank != i + 1) {
	        fprintf(stderr, "test node26 failed (step 6.2.%d)\n", i);
		return 26;
	    }
	    if (n.VertexCount() != i + 1) {
	        fprintf(stderr, "test node26 failed (step 6.3.%d)\n", i);
		return 26;
	    }
        }
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    rank = n.VertexRank(buf);
	    if (rank == E4_VERTEXNOTFOUND) {
		fprintf(stderr, "test node26 failed (step 7.1.%d)\n", i);
		return 26;
	    }
	    if (rank != i + 1) {
		fprintf(stderr, "test node26 failed (step 7.2.%d)\n", i);
		return 26;
	    }
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node26 failed (step 8)\n");
	    return 26;
	}
    }

    return 0;
}

static int
test_node27()
{
    e4_Marker t;
    e4_Node n;
    int i, rank;
    char buf[32];

    /*
     * Test e4_Node::Exists().
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node27 failed (step 1)\n");
            return 27;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node27 failed (step 2)\n");
            return 27;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node27 failed (step 3)\n");
            return 27;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node27 failed (step 4)\n");
            return 27;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node27 failed (step 5)\n");
            return 27;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.AddVertex(buf, E4_IOLAST, rank, i)) {
		fprintf(stderr, "test node27 failed (step 6.1.%d)\n", i);
		return 27;
	    }
	    if (rank != i + 1) {
	        fprintf(stderr, "test node27 failed (step 6.2.%d)\n", i);
		return 27;
	    }
	    if (n.VertexCount() != i + 1) {
	        fprintf(stderr, "test node27 failed (step 6.3.%d)\n", i);
		return 27;
	    }
        }
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.Exists(buf)) {
		fprintf(stderr, "test node27 failed (step 7.%d)\n", i);
		return 27;
	    }
	}
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "g%d", i + 1);
	    if (n.Exists(buf)) {
		fprintf(stderr, "test node27 failed (step 8.%d)\n", i);
		return 27;
	    }
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node27 failed (step 9)\n");
	    return 27;
	}
    }

    return 0;
}

static int
test_node29()
{
    e4_Marker t;
    e4_Node n, sn;
    int i, rank;
    char buf[32];

    /*
     * Test e4_Node::GetRankInParent(), e4_Node::IsRoot().
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node29 failed (step 1)\n");
            return 29;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node29 failed (step 2)\n");
            return 29;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node29 failed (step 3)\n");
            return 29;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node29 failed (step 4)\n");
            return 29;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node29 failed (step 5)\n");
            return 29;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.AddVertex(buf, E4_IOLAST, rank, i)) {
		fprintf(stderr, "test node29 failed (step 6.1.%d)\n", i);
		return 29;
	    }
	    if (rank != i + 1) {
	        fprintf(stderr, "test node29 failed (step 6.2.%d)\n", i);
		return 29;
	    }
	    if (n.VertexCount() != i + 1) {
	        fprintf(stderr, "test node29 failed (step 6.3.%d)\n", i);
		return 29;
	    }
        }
	if (!n.SetNodeByRank(25, sn)) {
	    fprintf(stderr, "test node29 failed (step 7)\n");
	    return 29;
	}
	if (!sn.IsValid()) {
	    fprintf(stderr, "test node29 failed (step 8)\n");
	    return 29;
	}
	rank = sn.GetRankInParent();
	if (rank == E4_ROOTINDEX) {
	    fprintf(stderr, "test node29 failed (step 9)\n");
	    return 29;
	}
	if (rank != 25) {
	    fprintf(stderr, "test node29 failed (step 10)\n");
	    return 29;
	}
	if (sn.IsRoot()) {
	    fprintf(stderr, "test node29 failed (step 11)\n");
	    return 29;
	}
	if (!n.IsRoot()) {
	    fprintf(stderr, "test node29 failed (step 12)\n");
	    return 29;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node29 failed (step 13)\n");
	    return 29;
	}
    }

    return 0;
}

static int
test_node30()
{
    e4_Marker t;
    e4_Node n, sn;
    int i, rank;
    char buf[32];
    const char *ss;

    /*
     * Test e4_Node::GetNameInParent().
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node30 failed (step 1)\n");
            return 30;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node30 failed (step 2)\n");
            return 30;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node30 failed (step 3)\n");
            return 30;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node30 failed (step 4)\n");
            return 30;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node30 failed (step 5)\n");
            return 30;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.AddVertex(buf, E4_IOLAST, rank, i)) {
		fprintf(stderr, "test node30 failed (step 6.1.%d)\n", i);
		return 30;
	    }
	    if (rank != i + 1) {
	        fprintf(stderr, "test node30 failed (step 6.2.%d)\n", i);
		return 30;
	    }
	    if (n.VertexCount() != i + 1) {
	        fprintf(stderr, "test node30 failed (step 6.3.%d)\n", i);
		return 30;
	    }
        }
	if (!n.SetNodeByRank(25, sn)) {
	    fprintf(stderr, "test node30 failed (step 7)\n");
	    return 30;
	}
	if (!sn.IsValid()) {
	    fprintf(stderr, "test node30 failed (step 8)\n");
	    return 30;
	}
	ss = sn.GetNameInParent();
	if (ss == NULL) {
	    fprintf(stderr, "test node30 failed (step 9)\n");
	    return 30;
	}
	if (strcmp(ss, "f25") != 0) {
	    fprintf(stderr, "test node30 failed (step 10)\n");
	    return 30;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node30 failed (step 11)\n");
	    return 30;
	}
    }

    return 0;
}

static int
test_node31()
{
    e4_Marker t;
    e4_Node n, sn;
    int i, rank;
    char buf[32];

    /*
     * Test e4_Node::GetStorage().
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node31 failed (step 1)\n");
            return 31;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node31 failed (step 2)\n");
            return 31;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node31 failed (step 3)\n");
            return 31;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node31 failed (step 4)\n");
            return 31;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node31 failed (step 5)\n");
            return 31;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.AddVertex(buf, E4_IOLAST, rank, i)) {
		fprintf(stderr, "test node31 failed (step 6.1.%d)\n", i);
		return 31;
	    }
	    if (rank != i + 1) {
	        fprintf(stderr, "test node31 failed (step 6.2.%d)\n", i);
		return 31;
	    }
	    if (n.VertexCount() != i + 1) {
	        fprintf(stderr, "test node31 failed (step 6.3.%d)\n", i);
		return 31;
	    }
        }
	if (!n.SetNodeByRank(25, sn)) {
	    fprintf(stderr, "test node31 failed (step 7)\n");
	    return 31;
	}
	if (!sn.IsValid()) {
	    fprintf(stderr, "test node31 failed (step 8)\n");
	    return 31;
	}
	e4_Storage s1;
	if (!n.GetStorage(s1)) {
	    fprintf(stderr, "test node31 failed (step 9)\n");
	    return 31;
	}
	if (!s1.IsValid()) {
	    fprintf(stderr, "test node31 failed (step 10)\n");
	    return 31;
	}
	if (s1 != s) {
	    fprintf(stderr, "test node31 failed (step 11)\n");
	    return 31;
	}
	if (!sn.GetStorage(s1)) {
	    fprintf(stderr, "test node31 failed (step 12)\n");
	    return 31;
	}
	if (!s1.IsValid()) {
	    fprintf(stderr, "test node31 failed (step 13)\n");
	    return 31;
	}
	if (s1 != s) {
	    fprintf(stderr, "test node31 failed (step 14)\n");
	    return 31;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node31 failed (step 15)\n");
	    return 31;
	}
    }

    return 0;
}

static int
test_node32()
{
    e4_Marker t;
    e4_Node n, sn;
    e4_NodeUniqueID nuid1, nuid2, tnuid;
    int i, rank;
    char buf[32];

    /*
     * Test e4_Node::GetUniqueID().
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node32 failed (step 1)\n");
            return 32;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node32 failed (step 2)\n");
            return 32;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node32 failed (step 3)\n");
            return 32;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node32 failed (step 4)\n");
            return 32;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node32 failed (step 5)\n");
            return 32;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.AddVertex(buf, E4_IOLAST, rank, i)) {
		fprintf(stderr, "test node32 failed (step 6.1.%d)\n", i);
		return 32;
	    }
	    if (rank != i + 1) {
	        fprintf(stderr, "test node32 failed (step 6.2.%d)\n", i);
		return 32;
	    }
	    if (n.VertexCount() != i + 1) {
	        fprintf(stderr, "test node32 failed (step 6.3.%d)\n", i);
		return 32;
	    }
        }
	if (!n.SetNodeByRank(25, sn)) {
	    fprintf(stderr, "test node32 failed (step 7)\n");
	    return 32;
	}
	if (!sn.IsValid()) {
	    fprintf(stderr, "test node32 failed (step 8)\n");
	    return 32;
	}
	(void) n.GetUniqueID(nuid1);
	(void) sn.GetUniqueID(nuid2);
	if (nuid1 == nuid2) {
	    fprintf(stderr, "test node32 failed (step 9)\n");
	    return 32;
	}
    }
    if (n.IsValid() || sn.IsValid() || t.IsValid()) {
	fprintf(stderr, "test node32 failed (step 10)\n");
	return 32;
    }
    {
        e4_Storage s("foo.db", E4_METAKIT);

        if (!s.IsValid()) {
            fprintf(stderr, "test node32 failed (step 11)\n");
            return 32;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node32 failed (step 12)\n");
            return 32;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node32 failed (step 13)\n");
            return 32;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node32 failed (step 14)\n");
            return 32;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node32 failed (step 15)\n");
            return 32;
        }
	if (!n.GetVertexByRank(25, sn)) {
	    fprintf(stderr, "test node32 failed (step 16)\n");
	    return 32;
	}
	if (!sn.IsValid()) {
	    fprintf(stderr, "test node32 failed (step 17)\n");
	    return 32;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node32 failed (step 18)\n");
	    return 32;
	}
    }

    return 0;
}

static int
test_node33()
{
    e4_Marker t;
    e4_Node n, sn;
    e4_Vertex v;
    int i, rank;
    char buf[32];
    e4_Node ssns[TN_VERTEXLIMIT];

    /*
     * Testing e4_Node::Delete(), e4_Node::IsValid().
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node33 failed (step 1)\n");
            return 33;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node33 failed (step 2)\n");
            return 33;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node33 failed (step 3)\n");
            return 33;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node33 failed (step 4)\n");
            return 33;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node33 failed (step 5)\n");
            return 33;
        }
        for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!n.AddVertex(buf, E4_IOLAST, rank, i)) {
		fprintf(stderr, "test node33 failed (step 6.1.%d)\n", i);
		return 33;
	    }
	    if (rank != i + 1) {
	        fprintf(stderr, "test node33 failed (step 6.2.%d)\n", i);
		return 33;
	    }
	    if (n.VertexCount() != i + 1) {
	        fprintf(stderr, "test node33 failed (step 6.3.%d)\n", i);
		return 33;
	    }
        }
	if (!n.SetNodeByRank(11, sn)) {
	    fprintf(stderr, "test node33 failed (step 7)\n");
	    return 33;
	}
	if (!n.GetVertexRefByRank(11, v) || (!v.IsValid())) {
	    fprintf(stderr, "test node33 failed (step 7.1)\n");
	    return 33;
	}
	if (!sn.IsValid()) {
	    fprintf(stderr, "test node33 failed (step 8)\n");
	    return 33;
	}
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    sprintf(buf, "f%d", i + 1);
	    if (!sn.AddNode(buf, E4_IOLAST, rank, ssns[i])) {
		fprintf(stderr, "test node33 failed (step 9.1.%d)\n", i);
		return 33;
	    }
	    if (!ssns[i].IsValid()) {
		fprintf(stderr, "test node33 failed (step 9.2.%d)\n", i);
		return 33;
	    }
	    if (sn.VertexCount() != i + 1) {
		fprintf(stderr, "test node33 failed (step 9.3.%d)\n", i);
		return 33;
	    }
	    if (rank != i + 1) {
		fprintf(stderr, "test node33 failed (step 9.4.%d)\n", i);
		return 33;
	    }
	}
	if (!sn.Delete()) {
	    fprintf(stderr, "test node33 failed (step 10)\n");
	    return 33;
	}

	if (ssns[23].IsValid()) {
	  fprintf(stderr, "Delete of sn failed to delete nested nodes\n");
	}
	if (!n.IsValid()) {
	  fprintf(stderr, "Smashed up\n");
	}

	if (v.IsValid()) {
	    fprintf(stderr, "test node33 failed (step 10.1)\n");
	    return 33;
	}
	if (sn.IsValid()) {
	    fprintf(stderr, "test node33 failed (step 11)\n");
	    return 33;
	}
	if (n.VertexCount() != TN_VERTEXLIMIT - 1) {
	    fprintf(stderr, "test node33 failed (step 12), %d %d\n",
		    n.VertexCount(), TN_VERTEXLIMIT - 1);
	    return 33;
	}
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
  	    if (ssns[i].IsValid()) {
		fprintf(stderr, "test node33 failed (step 13.%d)\n", i);
		return 33;
	    }
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node33 failed (step 14)\n");
	    return 33;
	}
    }

    return 0;
}

static int
test_node34()
{
    e4_Marker t;
    e4_Node n, sn, dn, hn, root;
    int i, j, rank;
    char buf[32];

    /*
     * Testing e4_Node::Delete(), e4_Node::IsValid().
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node34 failed (step 1)\n");
            return 34;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node34 failed (step 2)\n");
            return 34;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node34 failed (step 3)\n");
            return 34;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node34 failed (step 4)\n");
            return 34;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node34 failed (step 5)\n");
            return 34;
        }
	root = n;
	for (j = 0; j < TN_VERTEXLIMIT; j++) {
	    if (j == 22) {
		hn = n;
	    }
	    if (j == 55) {
		dn = n;
	    }
	    for (i = 0; i < TN_VERTEXLIMIT; i++) {
		sprintf(buf, "f%d", i + 1);
		if (!n.AddVertex(buf, E4_IOLAST, rank, i)) {
		    fprintf(stderr, "test node34 failed (step 6.1.%d.%d)\n", 
			    j, i);
		    return 34;
		}
		if (rank != i + 1) {
		    fprintf(stderr, "test node34 failed (step 6.2.%d.%d)\n",
			    j, i);
		    return 34;
		}
		if (n.VertexCount() != i + 1) {
		    fprintf(stderr, "test node34 failed (step 6.3.%d.%d)\n", 
			    j, i);
		    return 34;
		}
	    }
	    if (!n.SetNodeByRank(11, sn)) {
		fprintf(stderr, "test node34 failed (step 7.%d)\n", j);
		return 34;
	    }
	    if (!sn.IsValid()) {
		fprintf(stderr, "test node34 failed (step 8.%d)\n", j);
		return 34;
	    }
	    n = sn;
	}
	if (!hn.IsValid()) {
	    fprintf(stderr, "test node34 failed (step 9)\n");
	    return 34;
	}
	if (!dn.IsValid()) {
	    fprintf(stderr, "test node34 failed (step 10)\n");
	    return 34;
	}
	if (!dn.Delete()) {
	    fprintf(stderr, "test node34 failed (step 11)\n");
	    return 34;
	}
	if (!hn.IsValid()) {
	    fprintf(stderr, "test node34 failed (step 12)\n");
	    return 34;
	}
	if (!hn.Delete()) {
	    fprintf(stderr, "test node34 failed (step 13)\n");
	    return 34;
	}
	if (!root.IsValid()) {
	    fprintf(stderr, "test node34 failed (step 14)\n");
	    return 34;
	}
	if (root.VertexCount() != TN_VERTEXLIMIT) {
	    fprintf(stderr, "test node34 failed (step 15)\n");
	    return 34;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node34 failed (step 16)\n");
	    return 34;
	}
    }

    return 0;
}

static int
test_node35()
{
    e4_Marker t;
    e4_Node n;
    int i, rank;

    /*
     * Test e4_Node::Delete on the root of a marker.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test node35 failed (step 1)\n");
            return 35;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node35 failed (step 2)\n");
            return 35;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node35 failed (step 3)\n");
            return 35;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node35 failed (step 4)\n");
            return 35;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node35 failed (step 5)\n");
            return 35;
        }
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    if (!n.AddVertex("f1", E4_IOLAST, rank, i)) {
		fprintf(stderr, "test node35 failed (step 6.1.%d)\n", i);
		return 35;
	    }
	    if (rank != i + 1) {
		fprintf(stderr, "test node35 failed (step 6.2.%d)\n", i);
		return 35;
	    }
	    if (n.VertexCount() != i + 1) {
		fprintf(stderr, "test node35 failed (step 6.3.%d)\n", i);
		return 35;
	    }
	}
	if (!n.Delete()) {
	    fprintf(stderr, "test node35 failed (step 7)\n");
	    return 35;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node35 failed (step 8)\n");
	    return 35;
	}
    }

    return 0;
}

static int
test_node36()
{
    e4_Marker t;
    e4_Node n, n1;
    int i, rank;

    /*
     * Test e4_Node::Delete on the root of a marker.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

        if (!s.IsValid()) {
            fprintf(stderr, "test node36 failed (step 1)\n");
            return 36;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node36 failed (step 2)\n");
            return 36;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node36 failed (step 3)\n");
            return 36;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node36 failed (step 4)\n");
            return 36;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node36 failed (step 5)\n");
            return 36;
        }
	for (i = 0; i < TN_VERTEXLIMIT; i++) {
	    if (!n.AddNode("f1", E4_IOLAST, rank, n1)) {
		fprintf(stderr, "test node36 failed (step 6.1.%d)\n", i);
		return 36;
	    }
	    n1 = invalidNode;
	    if (n1.IsValid()) {
		fprintf(stderr, "test node36 failed (step 6.2.%d)\n", i);
		return 36;
	    }
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node36 failed (step 7)\n");
	    return 36;
	}
    }

    return 0;
}

static int
test_node37()
{
    e4_Marker t;
    e4_Node n, n1, n2, n3;
    e4_Vertex f;
    int i, rank;

    /*
     * Test e4_Node::MoveVertex to move a vertex between two nodes.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

        if (!s.IsValid()) {
            fprintf(stderr, "test node37 failed (step 1)\n");
            return 37;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node37 failed (step 2)\n");
            return 37;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node37 failed (step 3)\n");
            return 37;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node37 failed (step 4)\n");
            return 37;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node37 failed (step 5)\n");
            return 37;
        }
	if ((!n.AddNode("n1", E4_IOLAST, rank, n1)) || (!n1.IsValid())) {
	    fprintf(stderr, "test node37 failed (step 6)\n");
	    return 37;
	}
	if ((!n.AddNode("n2", E4_IOLAST, rank, n2)) || (!n2.IsValid())) {
	    fprintf(stderr, "test node37 failed (step 7)\n");
	    return 37;
	}
	if ((n1.VertexCount() != 0) || (n2.VertexCount() != 0)) {
	    fprintf(stderr, "test node37 failed (step 8)\n");
	    return 37;
	}
	if ((!n1.AddVertexRef("f1", E4_IOLAST, rank, 37, f)) || 
	    (!f.IsValid())) {
	    fprintf(stderr, "test node37 failed (step 9)\n");
	    return 37;
	}
	for (i = 0; i < 32; i++) {
	    if (!n1.AddVertex("int", E4_IOLAST, rank, i)) {
		fprintf(stderr, "test node37 failed (step 10.%d)\n", i);
		return 37;
	    }
	}
	if (n1.VertexCount() != 33) {
	    fprintf(stderr, "test node37 failed (step 11)\n");
	    return 37;
	}
	for (i = 0; i < 32; i++) {
	    if (!n2.AddVertex("int", E4_IOLAST, rank, i)) {
		fprintf(stderr, "test node37 failed (step 12.%d)\n", i);
		return 37;
	    }
	}
	if (n2.VertexCount() != 32) {
	    fprintf(stderr, "test node37 failed (step 13)\n");
	    return 37;
	}
	if (f.Rank() != 1) {
	    fprintf(stderr, "test node37 failed (step 14)\n");
	    return 37;
	}
	if ((!f.GetNode(n3)) || (!n3.IsValid()) || (n3 != n1)) {
	    fprintf(stderr, "test node37 failed (step 15)\n");
	    return 37;
	}

	/*
	 * Move f from n1 to n2.
	 */

	if (!n2.MoveVertex(f, E4_IOLAST, rank)) {
	    fprintf(stderr, "test node37 failed (step 16)\n");
	    return 37;
	}
	if (n1.VertexCount() != 32) {
	    fprintf(stderr, "test node37 failed (step 17.1)\n");
	    return 37;
	}
	if (n2.VertexCount() != 33) {
	    fprintf(stderr, "test node37 failed (step 17.2)\n");
	    return 37;
	}
	if (!f.IsValid()) {
	    fprintf(stderr, "test node37 failed (step 17.3)\n");
	    return 37;
	}
	if (!f.GetNode(n3)) {
	    fprintf(stderr, "test node37 failed (step 17.4)\n");
	    return 37;
	}
	if (!n3.IsValid()) {
	    fprintf(stderr, "test node37 failed (step 17.5)\n");
	    return 37;
	}
	if (n3 != n2) {
	    fprintf(stderr, "test node37 failed (step 17.6)\n");
	    return 37;
	}
	if (n3 == n1) {
	    fprintf(stderr, "test node37 failed (step 17.7)\n");
	    return 37;
	}
	if (f.Rank() != 33) {
	    fprintf(stderr, "test node37 failed (step 17.8), %d\n", f.Rank());
	    return 37;
	}

	/*
	 * Move f from last rank in n2 to rank 17 in n2.
	 */

	rank = 17;
	if (!n2.MoveVertex(f, E4_IOAT, rank)) {
	    fprintf(stderr, "test node37 failed (step 20)\n");
	    return 37;
	}
	if ((n2.VertexCount() != 33) || (f.Rank() != 17) ||
	    (n1.VertexCount() != 32)) {
	    fprintf(stderr, "test node37 failed (step 21)\n");
	    return 37;
	}
	if ((!f.GetNode(n3)) || (!n3.IsValid()) || (n3 != n2) ||
	    (n3 == n1)) {
	    fprintf(stderr, "test node37 failed (step 22)\n");
	    return 37;
	}

	/*
	 * Move f from rank 17 in n2 to rank 24 in n1.
	 */

	rank = 24;
	if (!n1.MoveVertex(f, E4_IOAT, rank)) {
	    fprintf(stderr, "test node37 failed (step 23)\n");
	    return 37;
	}
	if ((n1.VertexCount() != 33) || (f.Rank() != 24) ||
	    (n2.VertexCount() != 32)) {
	    fprintf(stderr, "test node37 failed (step 24)\n");
	    return 37;
	}
	if ((!f.GetNode(n3)) || (!n3.IsValid()) || (n3 != n1) ||
	    (n3 == n2)) {
	    fprintf(stderr, "test node37 failed (step 25)\n");
	    return 37;
	}
	s.Delete();
	if ((s.IsValid()) || (n1.IsValid()) || (n2.IsValid()) ||
	    (f.IsValid())) {
	    fprintf(stderr, "test node37 failed (step 26)\n");
	    return 37;
	}
    }

    return 0;
}

static int
test_node38()
{
    e4_Marker t, to;
    e4_Node n, n1, n2, n3;
    e4_Node no;
    e4_Vertex f;
    int rank;

    /*
     * Test e4_Node::MoveVertex() with various illegal combinations.
     * All of these illegal attempts should be caught.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

        if (!s.IsValid()) {
            fprintf(stderr, "test node38 failed (step 1)\n");
            return 38;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node38 failed (step 2)\n");
            return 38;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node38 failed (step 3)\n");
            return 38;
        }
        if (!t.GetMarkedNode(n)) {
            fprintf(stderr, "test node38 failed (step 4)\n");
            return 38;
        }
        if (!n.IsValid()) {
            fprintf(stderr, "test node38 failed (step 5)\n");
            return 38;
        }
	if ((!n.AddNode("n1", E4_IOLAST, rank, n1)) || (!n1.IsValid())) {
	    fprintf(stderr, "test node38 failed (step 6)\n");
	    return 38;
	}
	if ((!n.AddNode("n2", E4_IOLAST, rank, n2)) || (!n2.IsValid())) {
	    fprintf(stderr, "test node38 failed (step 7)\n");
	    return 38;
	}
	if ((!n1.AddVertexRef("f", E4_IOLAST, rank, 27, f)) ||
	    (!f.IsValid())) {
	    fprintf(stderr, "test node38 failed (step 8)\n");
	    return 38;
	}
	if ((n1.VertexCount() != 1) || (f.Rank() != 1)) {
	    fprintf(stderr, "test node38 failed (step 9)\n");
	    return 38;
	}
	e4_Storage so("another.db", E4_METAKIT);
	if (!so.IsValid()) {
	    fprintf(stderr, "test node38 failed (step 10)\n");
	    return 38;
	}
	if ((!so.GetMarker("mymarker", to)) || (!to.IsValid())) {
	    fprintf(stderr, "test node38 failed (step 11)\n");
	    return 38;
	}
	if ((!to.GetMarkedNode(no)) || (!no.IsValid())) {
	    fprintf(stderr, "test node38 failed (step 12)\n");
	    return 38;
	}

	/*
	 * Attempt to move a vertex into another storage.
	 */

	if (no.MoveVertex(f, E4_IOLAST, rank)) {
	    fprintf(stderr, "test node38 failed (step 13)\n");
	    return 38;
	}
	if ((!f.IsValid()) || (!f.GetNode(n3)) || (!n3.IsValid()) ||
	    (n3 == no) || (n3 != n1) || (n1.VertexCount() != 1) ||
	    (f.Rank() != 1)) {
	    fprintf(stderr, "test node38 failed (step 14)\n");
	    return 38;
	}
	so.Delete();
	if (so.IsValid() || (to.IsValid()) || (no.IsValid())) {
	    fprintf(stderr, "test node38 failed (step 14.1)\n");
	    return 38;
	}

	/*
	 * Attempt to move an invalid vertex between two nodes.
	 */

	n3 = invalidNode;
	if ((!f.Delete()) || (f.IsValid()) || (f.GetNode(n3)) || 
	    (n3.IsValid())) {
	    fprintf(stderr, "test node38 failed (step 15)\n");
	    return 38;
	}
	if (n2.MoveVertex(f, E4_IOLAST, rank)) {
	    fprintf(stderr, "test node38 failed (step 16)\n");
	    return 38;
	}

	/*
	 * Attempt to move a vertex to an invalid node.
	 */

	if ((!n1.AddVertexRef("f", E4_IOLAST, rank, 27, f)) || (!f.IsValid()) ||
	    (!f.GetNode(n3)) || (!n3.IsValid()) || (n3 != n1) ||
	    (f.Rank() != 1) || (n1.VertexCount() != 1)) {
	    fprintf(stderr, "test node38 failed (step 17)\n");
	    return 38;
	}
	if ((!n2.Delete()) || (n2.IsValid())) {
	    fprintf(stderr, "test node38 failed (step 18)\n");
	    return 38;
	}
	if ((n2.MoveVertex(f, E4_IOLAST, rank)) || (!f.IsValid()) ||
	    (!f.GetNode(n3)) || (!n3.IsValid()) || (n3 != n1) ||
	    (n3 == n2)) {
	    fprintf(stderr, "test node38 failed (step 19)\n");
	    return 38;
	}
	s.Delete();
	if ((s.IsValid()) || (n.IsValid()) || (n1.IsValid()) || 
	    (n2.IsValid()) || (n3.IsValid()) || (t.IsValid())) {
	    fprintf(stderr, "test node38 failed (step 20)\n");
	    return 38;
	}
    }

    return 0;
}

static int
test_node39()
{
    int rank, i;
    e4_Marker t;
    e4_Node r, n1, n2, n3, n4;

    /*
     * Test e4_Node::AddVertex with a node value, e4_Node::ParentCount
     * and e4_Node::OccurrenceCount.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

        if (!s.IsValid()) {
            fprintf(stderr, "test node39 failed (step 1)\n");
            return 39;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node39 failed (step 2)\n");
            return 39;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node39 failed (step 3)\n");
            return 39;
        }
        if (!t.GetMarkedNode(r)) {
            fprintf(stderr, "test node39 failed (step 4)\n");
            return 39;
        }
        if (!r.IsValid()) {
            fprintf(stderr, "test node39 failed (step 5)\n");
            return 39;
        }
	if ((!r.AddNode("n1", E4_IOLAST, rank, n1)) || (!n1.IsValid())) {
	    fprintf(stderr, "test node39 failed (step 6)\n");
	    return 39;
	}
	if ((!r.AddNode("n2", E4_IOLAST, rank, n2)) || (!n2.IsValid())) {
	    fprintf(stderr, "test node39 failed (step 7)\n");
	    return 39;
	}
	if ((!n1.AddNode("n3", E4_IOLAST, rank, n3)) || (!n3.IsValid())) {
	    fprintf(stderr, "test node39 failed (step 8)\n");
	    return 39;
	}
	if (n3.ParentCount() != 1) {
	    fprintf(stderr, "test node39 failed (step 9)\n");
	    return 39;
	}
	if ((!n2.AddVertex("n3", E4_IOLAST, rank, n3)) || (!n3.IsValid()) ||
	    (n2.VertexCount() != 1) || (n3.ParentCount() != 2)) {
	    fprintf(stderr, "test node39 failed (step 10)\n");
	    return 39;
	}

	/*
	 * "n3" now occurs in both "n1" and "n2". Check that all is sane.
	 */

	if ((n3.OccurrenceCount(n1) != 1) ||
	    (n3.OccurrenceCount(n2) != 1) ||
	    (n3.OccurrenceCount(r) != E4_NODENOTFOUND) ||
	    (n3.OccurrenceCount() != 2) ||
	    (n3.ParentCount() != 2)) {
	    fprintf(stderr, "test node39 failed (step 11)\n");
	    return 39;
	}

	/*
	 * Add "n3" some more times to "n1" and "n2".
	 */

	for (i = 0; i < 32; i++) {
	    if ((i % 2) == 1) {
		if (!n1.AddVertex("n3", E4_IOLAST, rank, n3)) {
		    fprintf(stderr, "test node39 failed (step 12.1.%d)\n", i);
		    return 39;
		}
	    } else {
		if (!n2.AddVertex("n3", E4_IOLAST, rank, n3)) {
		    fprintf(stderr, "test node39 failed (step 12.2.%d)\n", i);
		    return 39;
		}
	    }
	    if (n3.ParentCount() != 2) {
		fprintf(stderr, "test node39 failed (step 12.3.%d)\n", i);
		return 39;
	    }
	}

	/*
	 * Ensure that things are still correct.
	 */

	if ((n3.OccurrenceCount() != 34) ||
	    (n3.OccurrenceCount(n1) != 17) ||
	    (n3.OccurrenceCount(n2) != 17)) {
	    fprintf(stderr, "test node39 failed (step 13)\n");
	    return 39;
	}

	/*
	 * Check parent ranks in "n3".
	 */

	if ((n3.ParentRank(n1) != 1) ||
	    (n3.ParentRank(n2) != 2) ||
	    (n3.ParentRank(r) != E4_NODENOTFOUND)) {
	    fprintf(stderr, "test node39 failed (step 14)\n");
	    return 39;
	}

	/*
	 * Attempt to retrieve individual parents.
	 */

	n4 = invalidNode;
	if ((!n3.GetParent(1, n4)) || (!n4.IsValid()) ||
	    (n4 != n1) || (n4 == n2)) {
	    fprintf(stderr, "test node39 failed (step 15)\n");
	    return 39;
	}

	n4 = invalidNode;
	if ((!n3.GetParent(2, n4)) || (!n4.IsValid()) ||
	    (n4 == n1) || (n4 != n2)) {
	    fprintf(stderr, "test node39 failed (step 16)\n");
	    return 39;
	}

	/*
	 * Delete "n1". See if the parent count and occurrence count
	 * of "n3" are updated.
	 */

	if (!n1.Delete()) {
	    fprintf(stderr, "test node39 failed (step 17)\n");
	    return 39;
	}
	n4 = invalidNode;
	if ((!n3.IsValid()) || 
	    (n3.OccurrenceCount() != 17) ||
	    (n3.OccurrenceCount(n2) != 17) ||
	    (n1.IsValid()) ||
	    (n3.OccurrenceCount(n1) != E4_NODENOTFOUND) ||
	    (n3.ParentCount() != 1) ||
	    (!n3.GetParent(1, n4)) ||
	    (!n4.IsValid()) ||
	    (n4 != n2)) {
	    fprintf(stderr, "test node39 failed (step 18)\n");
	    return 39;
	}

	/*
	 * Delete "n2". See that "n3" becomes invalid too.
	 */

	if (!n2.Delete()) {
	    fprintf(stderr, "test node39 failed (step 19)\n");
	    return 39;
	}
	if (n3.IsValid()) {
	    fprintf(stderr, "test node39 failed (step 20)\n");
	    return 39;
	}

	/*
	 * Delete the storage.
	 */

	s.Delete();
	if ((s.IsValid()) || (t.IsValid()) || (r.IsValid()) ||
	    (n1.IsValid()) || (n2.IsValid()) || (n3.IsValid()) ||
	    (n4.IsValid())) {
	    fprintf(stderr, "test node39 failed (step 21)\n");
	    return 39;
	}
    }

    return 0;
}

static int
test_node40()
{
    e4_Marker t;
    e4_Node r, n1, n2, n3, n4, n5;
    e4_Vertex f1, f2;
    int rank;

    /*
     * Test e4_Node::AddVertexRef() with a node value, e4_Node::ParentCount
     * and e4_Node::OccurrenceCount.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

        if (!s.IsValid()) {
            fprintf(stderr, "test node40 failed (step 1)\n");
            return 40;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test node40 failed (step 2)\n");
            return 40;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node40 failed (step 3)\n");
            return 40;
        }
        if (!t.GetMarkedNode(r)) {
            fprintf(stderr, "test node40 failed (step 4)\n");
            return 40;
        }
        if (!r.IsValid()) {
            fprintf(stderr, "test node40 failed (step 5)\n");
            return 40;
        }
	if ((!r.AddNode("n1", E4_IOLAST, rank, n1)) || (!n1.IsValid())) {
	    fprintf(stderr, "test node40 failed (step 6)\n");
	    return 40;
	}
	if ((!r.AddNode("n2", E4_IOLAST, rank, n2)) || (!n2.IsValid())) {
	    fprintf(stderr, "test node40 failed (step 7)\n");
	    return 40;
	}
	if ((!n1.AddNode("n3", E4_IOLAST, rank, n3)) || (!n3.IsValid())) {
	    fprintf(stderr, "test node40 failed (step 8)\n");
	    return 40;
	}
	if (n3.ParentCount() != 1) {
	    fprintf(stderr, "test node40 failed (step 9)\n");
	    return 40;
	}
	if ((!n2.AddVertex("n3", E4_IOLAST, rank, n3)) || (!n3.IsValid()) ||
	    (n2.VertexCount() != 1) || (n3.ParentCount() != 2)) {
	    fprintf(stderr, "test node40 failed (step 10)\n");
	    return 40;
	}

	/*
	 * Try e4_Node::AddVertexRef.
	 */

	if ((!n2.AddVertexRef("n3", E4_IOLAST, rank, n3, f2)) || 
	    (!f2.IsValid()) ||
	    (n2.VertexCount() != 2) || (n3.ParentCount() != 2) ||
	    (n3.OccurrenceCount() != 3) || (f2.Rank() != 2)) {
	    fprintf(stderr, "test node40 failed (step 11)\n");
	    return 40;
	}
	if ((!n1.AddVertexRef("n3", E4_IOLAST, rank, n3, f1)) ||
	    (!f1.IsValid()) ||
	    (n1.VertexCount() != 2) || (n3.ParentCount() != 2) ||
	    (n3.OccurrenceCount() != 4) || (f1.Rank() != 2)) {
	    fprintf(stderr, "test node40 failed (step 12)\n");
	    return 40;
	}
	if ((n3.OccurrenceCount(n1) != 2) || (n3.OccurrenceCount(n2) != 2)) {
	    fprintf(stderr, "test node40 failed (step 13)\n");
	    return 40;
	}

	/*
	 * OK, now make an "n4" child of "r" and move "f1" and "f2"
	 * to it. Avter the move of "f1", "n3" should have a new parent,
	 * "n4", and that should be the last parent in the list. After the
	 * move of "f2" to "n4", no parent changes should happen but the
	 * occurrence count of "n2" and "n4" in the parent list of "n3"
	 * should change.
	 */

	if ((!r.AddNode("n4", E4_IOLAST, rank, n4)) || (!n4.IsValid()) ||
	    (r.VertexCount() != 3) || (n4.VertexCount() != 0)) {
	    fprintf(stderr, "test node40 failed (step 14)\n");
	    return 40;
	}
	if ((!n4.MoveVertex(f1, E4_IOLAST, rank)) || (!f1.IsValid()) ||
	    (n4.VertexCount() != 1)) {
	    fprintf(stderr, "test node40 failed (step 15)\n");
	    return 40;
	}
	if (n3.ParentCount() != 3) {
	    fprintf(stderr, "test node40 failed (step 16.1)\n");
	    return 40;
	}
	if (n3.OccurrenceCount() != 4) {
	    fprintf(stderr, "test node40 failed (step 16.2), %d\n",
		    n3.OccurrenceCount());
	    return 40;
	}
	if (n3.OccurrenceCount(n4) != 1) {
	    fprintf(stderr, "test node40 failed (step 16.3)\n");
	    return 40;
	}
	if ((n3.ParentCount() != 3) || (n3.OccurrenceCount() != 4) ||
	    (n3.OccurrenceCount(n4) != 1) || (n3.OccurrenceCount(n1) != 1) ||
	    (!n3.GetParent(3, n5)) || (!n5.IsValid()) || (n4 != n5)) {
	}
	if ((!n4.MoveVertex(f2, E4_IOLAST, rank)) || (!f2.IsValid()) ||
	    (n4.VertexCount() != 2)) {
	    fprintf(stderr, "test node40 failed (step 17)\n");
	    return 40;
	}
	if ((n3.ParentCount() != 3) || (n3.OccurrenceCount() != 4) ||
	    (n3.OccurrenceCount(n4) != 2) || (n3.OccurrenceCount(n2) != 1) ||
	    (!n3.GetParent(1, n5)) || (!n5.IsValid()) || (n1 != n5)) {
	    fprintf(stderr, "test node40 failed (step 17)\n");
	    return 40;
	}

	s.Delete();
	if ((s.IsValid()) || (t.IsValid()) || (r.IsValid()) || 
	    (n1.IsValid()) || (n2.IsValid()) || (n3.IsValid()) ||
	    (f1.IsValid()) || (f2.IsValid()) || (n4.IsValid()) ||
	    (n5.IsValid())) {
	    fprintf(stderr, "test node40 failed (step 22)\n");
	    return 40;
	}
    }

    return 0;
}

static int
test_node41()
{
    e4_Marker t1, t2;
    e4_Node r, n1, n2, n3;
    int rank;

    /*
     * Test e4_Node::MarkWith, e4_Node::MarkerCount, e4_Node::Unmark.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

        if (!s.IsValid()) {
            fprintf(stderr, "test node41 failed (step 1)\n");
            return 41;
        }
        if (!s.GetMarker("t1", t1)) {
            fprintf(stderr, "test node41 failed (step 2)\n");
            return 41;
        }
        if (!t1.IsValid()) {
            fprintf(stderr, "test node41 failed (step 3)\n");
            return 41;
        }
        if (!t1.GetMarkedNode(r)) {
            fprintf(stderr, "test node41 failed (step 4)\n");
            return 41;
        }
        if (!r.IsValid()) {
            fprintf(stderr, "test node41 failed (step 5)\n");
            return 41;
        }
	if ((!r.AddNode("n1", E4_IOLAST, rank, n1)) || (!n1.IsValid())) {
	    fprintf(stderr, "test node41 failed (step 6)\n");
	    return 41;
	}
	if ((r.MarkerCount() != 1) || (n1.MarkerCount() != 0)) {
	    fprintf(stderr, "test node41 failed (step 7)\n");
	    return 41;
	}
	if ((!s.GetMarker("t2", t2)) || (!t2.IsValid())) {
	    fprintf(stderr, "test node41 failed (step 8)\n");
	    return 41;
	}
	if ((!n1.MarkWith(t2)) || (n1.MarkerCount() != 1)) {
	    fprintf(stderr, "test node41 failed (step 9)\n");
	    return 41;
	}
	if ((!t2.GetMarkedNode(n2)) || (!n2.IsValid()) ||
	    (n2 != n1) || (n1.MarkerCount() != 1) || (r.MarkerCount() != 1)) {
	    fprintf(stderr, "test node41 failed (step 10)\n");
	    return 41;
	}
	if (!n1.Unmark(t2)) {
	    fprintf(stderr, "test node41 failed (step 11.a)\n");
	    return 41;
	}
	if (t2.IsValid()) {
	    fprintf(stderr, "test node41 failed (step 11.b)\n");
	    return 41;
	}
	if (n1.MarkerCount() != 0) {
	    fprintf(stderr, "test node41 failed (step 11.c), %d != 0\n",
		    n1.MarkerCount());
	    return 41;
	}
	if (!n1.IsValid()) {
	    fprintf(stderr, "test node41 failed (step 11.d)\n");
	    return 41;
	}
	if (!n2.IsValid()) {
	    fprintf(stderr, "test node41 failed (step 11.e)\n");
	    return 41;
	}
	if (n1.ParentCount() != 1) {
	    fprintf(stderr, "test node41 failed (step 11.f), %d != 1\n",
		    n1.ParentCount());
	    return 41;
	}

	/*
	 * Recreate "t2" and mark "n1".
	 */

	if ((!s.GetMarker("t2", t2)) || (!t2.IsValid())) {
	    fprintf(stderr, "test node41 failed (step 12)\n");
	    return 41;
	}
	if ((!n1.MarkWith(t2)) || (n1.MarkerCount() != 1) ||
	    (n2.MarkerCount() != 1) || (!t2.IsValid())) {
	    fprintf(stderr, "test node41 failed (step 13)\n");
	    return 41;
	}

	/*
	 * Unmark the root. The "n1" node should stay valid because
	 * it is still reachable.
	 */

	if ((!r.Unmark(t1)) || (t1.IsValid()) || (r.IsValid()) ||
	    (!n1.IsValid()) || (n1.MarkerCount() != 1) ||
	    (!n2.IsValid()) || (n2.MarkerCount() != 1)) {
	    fprintf(stderr, "test node41 failed (step 14)\n");
	    return 41;
	}

	/*
	 * Trying to mark a node with an invalid marker should fail.
	 */

	if ((n1.MarkWith(t1)) || (t1.IsValid()) || (n1.MarkerCount() != 1)) {
	    fprintf(stderr, "test node41 failed (step 15)\n");
	    return 15;
	}

	/*
	 * Recreate "t1" and mark "n1" with it. Test that initially
	 * it marks a completely new node.
	 */

	if ((!s.GetMarker("t1", t1)) || (!t1.IsValid())) {
	    fprintf(stderr, "test node41 failed (step 16)\n");
	    return 41;
	}
	if ((!t1.GetMarkedNode(n3)) || (!n3.IsValid()) || (n3 == n1) ||
	    (n3 == n2)) {
	    fprintf(stderr, "test node41 failed (step 17)\n");
	    return 41;
	}
	if ((!n1.MarkWith(t1)) || (n1.MarkerCount() != 2) ||
	    (n2.MarkerCount() != 2) || (!t1.IsValid())) {
	    fprintf(stderr, "test node41 failed (step 18)\n");
	    return 41;
	}

	/*
	 * Marking a node with a marker that is already marking
	 * that node is OK, and does nothing.
	 */

	if ((!n1.MarkWith(t2)) || (n1.MarkerCount() != 2) ||
	    (!t2.IsValid()) || (!t2.GetMarkedNode(n3)) || (!n3.IsValid()) ||
	    (n3 != n1)) {
	    fprintf(stderr, "test node41 failed (step 19)\n");
	    return 41;
	}
	s.Delete();
	if ((s.IsValid()) || (t1.IsValid()) || (t2.IsValid()) || 
	    (r.IsValid()) || (n1.IsValid()) || (n2.IsValid())) {
	    fprintf(stderr, "test node41 failed (step 20)\n");
	    return 41;
	}
    }

    return 0;
}

static int
test_node42()
{
    e4_Marker t1, t2, t3;
    e4_Node r, n1, n2;
    int rank, i;
    char buf[32];

    /*
     * Test e4_Node::MarkWith, e4_Node::MarkerCount, e4_Node::GetMark.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

        if (!s.IsValid()) {
            fprintf(stderr, "test node42 failed (step 1)\n");
            return 42;
        }
        if (!s.GetMarker("tree", t1)) {
            fprintf(stderr, "test node42 failed (step 2)\n");
            return 42;
        }
        if (!t1.IsValid()) {
            fprintf(stderr, "test node42 failed (step 3)\n");
            return 42;
        }
        if (!t1.GetMarkedNode(r)) {
            fprintf(stderr, "test node42 failed (step 4)\n");
            return 42;
        }
        if (!r.IsValid()) {
            fprintf(stderr, "test node42 failed (step 5)\n");
            return 42;
        }
	if ((!r.AddNode("n1", E4_IOLAST, rank, n1)) || (!n1.IsValid())) {
	    fprintf(stderr, "test node42 failed (step 6)\n");
	    return 42;
	}
	for (i = 1; i < 5; i++) {
	    sprintf(buf, "mark%d", i);
	    if ((!s.GetMarker(buf, t2)) || (!t2.IsValid())) {
		fprintf(stderr, "test node42 failed (step 7.1.%d)\n", i);
		return 42;
	    }
	    if ((!n1.MarkWith(t2)) || (!t2.IsValid()) ||
		(n1.MarkerCount() != i) || (!n1.GetMarker(i, t3)) ||
		(!t3.IsValid()) || (t3 != t2) || (!t3.GetMarkedNode(n2)) ||
		(!n2.IsValid()) || (n2 != n1)) {
		fprintf(stderr, "test node42 failed (step 7.2.%d)\n", i);
		return 42;
	    }
	    if (s.MarkerCount() != i + 1) {
		fprintf(stderr, "test node42 failed (step 7.3.%d)\n", i);
		return 42;
	    }
	}
	if (s.MarkerCount() != 5) {
	    fprintf(stderr, "test node42 failed (step 8)\n");
	    return 42;
	}
	if ((!n1.Delete()) || (n1.IsValid()) || (t2.IsValid()) ||
	    (t3.IsValid()) || (s.MarkerCount() != 1)) {
	    fprintf(stderr, "test node42 failed (step 9)\n");
	    return 42;
	}
	r.Delete();
	s.Delete();
	if ((s.IsValid()) || (t1.IsValid()) || (t2.IsValid()) ||
	    (t3.IsValid()) || (n1.IsValid()) || (n2.IsValid())) {
	    fprintf(stderr, "test node42 failed (step 10)\n");
	    return 42;
	}
    }

    return 0;
}

static int
test_node43()
{
    e4_Marker t;
    e4_Node r, n;
    e4_NodeUniqueID nuid0, nuid1;
    int rank, i;

    /*
     * This test checks that storage space (as denoted by the unique
     * identifier assigned to a node) is properly being recycled. This
     * test is specific to the E4_METAKIT driver because it relies on
     * the unique ID recycling behavior of that driver.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

        if (!s.IsValid()) {
            fprintf(stderr, "test node43 failed (step 1)\n");
            return 43;
        }
        if (!s.GetMarker("tree", t)) {
            fprintf(stderr, "test node43 failed (step 2)\n");
            return 43;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test node43 failed (step 3)\n");
            return 43;
        }
        if (!t.GetMarkedNode(r)) {
            fprintf(stderr, "test node43 failed (step 4)\n");
            return 43;
        }
        if (!r.IsValid()) {
            fprintf(stderr, "test node43 failed (step 5)\n");
            return 43;
        }
	if ((!r.AddNode("n", E4_IOLAST, rank, n)) || (!n.IsValid())) {
	    fprintf(stderr, "test node43 failed (step 6)\n");
	    return 43;
	}
	(void) n.GetUniqueID(nuid0);
	for (i = 0; i < 255; i++) {
	    if ((!n.Delete()) || (n.IsValid())) {
		fprintf(stderr, "test node43 failed (step 7.1.%d)\n", i);
		return 43;
	    }
	    if ((!r.AddNode("n", E4_IOLAST, rank, n)) || (!n.IsValid())) {
		fprintf(stderr, "test node43 failed (step 7.2.%d)\n", i);
		return 43;
	    }
	    if ((!n.GetUniqueID(nuid1)) || (nuid0 != nuid1)) {
		fprintf(stderr, "test node43 failed (step 7.3.%d)\n", i);
		return 43;
	    }
	}
	s.Delete();
	if ((s.IsValid()) || (t.IsValid()) || (r.IsValid()) || (n.IsValid())) {
	    fprintf(stderr, "test node43 failed (step 8)\n");
	    return 43;
	}
    }

    return 0;
}

static int
test_node44()
{
    e4_Node n1, n2, n3;
    int rank;
    
    /*
     * Test SetVertex with a vertex whose value is a node.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test node44 failed (step 1)\n");
	    return 44;
	}

	if (!s.GetMarkedNode("n1", n1) || !n1.IsValid()) {
	    fprintf(stderr, "test node44 failed (step 2)\n");
	    return 44;
	}
	if (!s.GetMarkedNode("n2", n2) || !n2.IsValid()) {
	    fprintf(stderr, "test node44 failed (step 3)\n");
	    return 44;
	}
	if (!n1.AddNode("n3", E4_IOLAST, rank, n3) || !n3.IsValid()) {
	    fprintf(stderr, "test node44 failed (step 4)\n");
	    return 44;
	}
	if (!n2.AddVertex("n3", E4_IOLAST, rank, n3)) {
	    fprintf(stderr, "test node44 failed (step 5)\n");
	    return 44;
	}
	if ((n3.ParentCount() != 2) || (n1.VertexCount() != 1) ||
	    (n2.VertexCount() != 1) || (n3.VertexCount() != 0)) {
	    fprintf(stderr, "test node44 failed (step 6)\n");
	    return 44;
	}
	if (!n1.SetVertex("n3", 1)) {
	    fprintf(stderr, "test node44 failed (step 7)\n");
	    return 44;
	}
	if (n3.ParentCount() != 1) {
	    fprintf(stderr, "test node44 failed (step 8)\n");
	    return 44;
	}
	if (n1.VertexCount() != 1) {
	    fprintf(stderr, "test node44 failed (step 9)\n");
	    return 44;
	}
	if (n2.VertexCount() != 1) {
	    fprintf(stderr, "test node44 failed (step 10)\n");
	    return 44;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node44 failed (step 11)\n");
	    return 44;
	}
    }
    if (n1.IsValid() || n2.IsValid() || n3.IsValid()) {
	fprintf(stderr, "test node44 failed (step 12)\n");
	return 44;
    }

    return 0;
}

static int
test_node45()
{
    e4_Node n1, n2, n3;
    int rank;
    
    /*
     * Test SetVertexByRank with a vertex whose value is a node.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test node45 failed (step 1)\n");
	    return 45;
	}

	if (!s.GetMarkedNode("n1", n1) || !n1.IsValid()) {
	    fprintf(stderr, "test node45 failed (step 2)\n");
	    return 45;
	}
	if (!s.GetMarkedNode("n2", n2) || !n2.IsValid()) {
	    fprintf(stderr, "test node45 failed (step 3)\n");
	    return 45;
	}
	if (!n1.AddNode("n3", E4_IOLAST, rank, n3) || !n3.IsValid()) {
	    fprintf(stderr, "test node45 failed (step 4)\n");
	    return 45;
	}
	if (!n2.AddVertex("n3", E4_IOLAST, rank, n3)) {
	    fprintf(stderr, "test node45 failed (step 5)\n");
	    return 45;
	}
	if ((n3.ParentCount() != 2) || (n1.VertexCount() != 1) ||
	    (n2.VertexCount() != 1) || (n3.VertexCount() != 0)) {
	    fprintf(stderr, "test node45 failed (step 6)\n");
	    return 45;
	}
	if (!n1.SetVertexByRank(1, 1)) {
	    fprintf(stderr, "test node45 failed (step 7)\n");
	    return 45;
	}
	if (n3.ParentCount() != 1) {
	    fprintf(stderr, "test node45 failed (step 8)\n");
	    return 45;
	}
	if (n1.VertexCount() != 1) {
	    fprintf(stderr, "test node45 failed (step 9)\n");
	    return 45;
	}
	if (n2.VertexCount() != 1) {
	    fprintf(stderr, "test node45 failed (step 10)\n");
	    return 45;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test node45 failed (step 11)\n");
	    return 45;
	}
    }
    if (n1.IsValid() || n2.IsValid() || n3.IsValid()) {
	fprintf(stderr, "test node45 failed (step 12)\n");
	return 45;
    }

    return 0;
}

int
test_node()
{
    int result;

    fprintf(stderr, "Running e4_Node tests: ");
    result = test_node1();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node41();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");

    /*
     * test_node2() and test_node3() have been removed.
     */

    result = test_node4();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node5();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node6();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node7();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node8();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node9();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node10();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node11();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node12();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node13();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node14();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node15();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node16();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node17();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node18();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node19();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node20();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node21();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node22();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node23();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node24();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node25();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node26();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node27();
    if (result != 0) {
	return result;
    }

    /*
     * test_node28 was removed.
     */

    fprintf(stderr, ".");
    result = test_node29();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node30();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node31();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node32();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node33();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node34();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node35();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node36();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node37();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node38();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node39();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node40();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node42();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node43();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node44();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".");
    result = test_node45();
    if (result != 0) {
	return result;
    }
    fprintf(stderr, ".\n");
    return 0;
}
