/*
 * teststorage.cpp --
 *
 * Tests of the e4_StorageImpl class.
 *
 * Copyright (c) 2000-2001, JYL Software Inc.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE, EVEN IF
 * JYL SOFTWARE INC. IS MADE AWARE OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#ifndef	_WIN32
#include <unistd.h>
#endif
#include "e4graph.h"
#include "test.h"

/*
 * Size of array of e4_MarkerImpls to create.
 */

#define	TEST_ARRAY_SIZE		256

static int
test_storage1()
{
    /*
     * Test that we are able to open an e4Storage file (and create it
     * if needed).
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test storage1 failed, step 1\n");
            return 1;
        }
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test storage1 failed, step 2\n");
	    return 1;
	}
    }

    return 0;
}

static int
test_storage2()
{
    e4_Marker t;

    /*
     * Test that we are able to get an e4Marker from an e4Storage (and
     * create it if needed).
     */
    
    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test storage2 failed (step 1)\n");
            return 2;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test storage2 failed (step 2)\n");
            return 2;
        }
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test storage2 failed (step 3)\n");
	    return 2;
	}
    }
    if (t.IsValid()) {
        fprintf(stderr, "test storage2 failed (step 3)\n");
        return 2;
    }
    return 0;
}

static int
test_storage3()
{
    e4_Marker t;
    
    /*
     * Test that the MarkerCount operation works.
     */
    
    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test storage3 failed (step 1)\n");
            return 3;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test storage3 failed (step 2)\n");
            return 3;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test storage3 failed (step 3)\n");
            return 3;
        }
        if (s.MarkerCount() != 1) {
            fprintf(stderr, "test storage3 failed (step 4)\n");
            return 3;
        }
    }
    if (t.IsValid()) {
        fprintf(stderr, "test storage3 failed (step 5)\n");
        return 3;
    }
    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test storage3 failed (step 6)\n");
	    return 3;
	}
	if (s.MarkerCount() != 1) {
	    fprintf(stderr, "test storage3 failed (step 7)\n");
	    return 3;
	}
	s.Delete();
    }

    return 0;
}

static int
test_storage4()
{
    e4_Marker t;

    /*
     * Test that Commit works.
     */
    
    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test storage4 failed (step 1)\n");
            return 4;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test storage4 failed (step 2)\n");
            return 4;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test storage4 failed (step 3)\n");
            return 4;
        }
        s.Commit();
    }
    if (t.IsValid()) {
        fprintf(stderr, "test storage4 failed (step 4)\n");
        return 4;
    }
    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test storage4 failed (step 5)\n");
            return 4;
        }
        if (s.MarkerCount() != 1) {
            fprintf(stderr, "test storage4 failed (step 6), %d\n",
                    s.MarkerCount());
            return 4;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test storage4 failed (step 7)\n");
            return 4;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test storage4 faield (step 8)\n");
            return 4;
        }
        if (s.MarkerCount() != 1) {
            fprintf(stderr, "test storage4 failed (step 9)\n");
            return 4;
        }
    }
    if (t.IsValid()) {
        fprintf(stderr, "test storage4 failed (step 10)\n");
        return 4;
    }
    {
	e4_Storage s("foo.db", E4_METAKIT);

	s.Delete();
    }

    return 0;
}

static int
test_storage5()
{
    /*
     * Test that autocommit works.
     */
    
    {
        e4_Storage s1("foo.db", E4_METAKIT);
        e4_Marker t1;

        if (!s1.IsValid()) {
            fprintf(stderr, "test storage5 failed (step 1)\n");
            return 5;
        }
        if (!s1.GetMarker("mymarker", t1)) {
            fprintf(stderr, "test storage5 failed (step 2)\n");
            return 5;
        }
        if (!t1.IsValid()) {
            fprintf(stderr, "test storage5 failed (step 3)\n");
            return 5;
        }
        if (s1.AutoCommit() == false) {
            fprintf(stderr, "test storage5 failed (step 4)\n");
            return 5;
        }
        s1.AutoCommit(true);
        if (s1.AutoCommit() == false) {
            fprintf(stderr, "test storage5 failed (step 5)\n");
            return 5;
        }

	/*
	 * Now rely on the C++ scoping rules to cause a commit.
	 */
    }
    {
        e4_Storage s2("foo.db", E4_METAKIT);
        e4_Marker t2;

        if (!s2.IsValid()) {
            fprintf(stderr, "test storage5 failed (step 6)\n");
            return 5;
        }
        if (s2.MarkerCount() != 1) {
            fprintf(stderr, "test storage5 failed (step 7)\n");
            return 5;
        }
        if (!s2.GetMarker("mymarker", t2)) {
            fprintf(stderr, "test storage5 failed (step 8)\n");
            return 5;
        }
        if (!t2.IsValid()) {
            fprintf(stderr, "test storage5 failed (step 9)\n");
            return 5;
        }
        if (s2.MarkerCount() != 1) {
            fprintf(stderr, "test storage5 failed (step 10, %d)\n",
                    s2.MarkerCount());
            return 5;
        }
    }
    {
	e4_Storage s3("foo.db", E4_METAKIT);

	s3.Delete();
    }

    return 0;
}

static int
test_storage6()
{
    e4_Marker t;
    
    /*
     * Test assignment operator, caching, operator==, operator!=
     */

    {
        e4_Storage s1("foo.db", E4_METAKIT);

        if (!s1.IsValid()) {
            fprintf(stderr, "test storage6 failed (step 1)\n");
            return 6;
        }

        e4_Storage s2 = s1;
        if (!s2.IsValid()) {
            fprintf(stderr, "test storage6 failed (step 2)\n");
            return 6;
        }
        if (s2 == s1) {
        } else {
            fprintf(stderr, "test storage6 failed (step 3)\n");
            return 6;
        }
        if (s1 != s2) {
            fprintf(stderr, "test storage6 failed (step 4)\n");
            return 6;
        }

        /*
         * The next line should return the same storage as s1.
         */
        
        e4_Storage s3("foo.db", E4_METAKIT);

        if (!s3.IsValid()) {
            fprintf(stderr, "test storage6 failed (step 5)\n");
            return 6;
        }
        if (s3 == s1) {
        } else {
            fprintf(stderr, "test storage6 failed (step 6)\n");
            return 6;
        }
        if (s3 != s1) {
            fprintf(stderr, "test storage6 failed (step 7)\n");
            return 6;
        }
        e4_Storage s4 = s3;
        if (!s3.IsValid()) {
            fprintf(stderr, "test storage6 failed (step 8)\n");
            return 6;
        }
        if (s4 == s1) {
        } else {
            fprintf(stderr, "test storage6 failed (step 9)\n");
            return 6;
        }
        if (s4 != s1) {
            fprintf(stderr, "test storage6 failed (step 10)\n");
            return 6;
        }

        if (!s1.GetMarker("mymarker", t)) {
            fprintf(stderr, "test storage6 failed (step 11)\n");
            return 6;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test storage6 failed (step 12)\n");
            return 6;
        }
        if ((s1.MarkerCount() != 1) ||
            (s2.MarkerCount() != 1) ||
            (s3.MarkerCount() != 1) ||
            (s4.MarkerCount() != 1)) {
            fprintf(stderr, "test storage6 failed (step 13)\n");
            return 6;
        }
        if ((s1 == t) || (s2 == t) || (s3 == t) || (s4 == t)) {
            fprintf(stderr, "test storage6 failed (step 14)\n");
            return 6;
        }
	s1.Delete();
	s2.Delete();
	s3.Delete();
	s4.Delete();

	if (s1.IsValid() || s2.IsValid() || s3.IsValid() || s4.IsValid() ||
	    t.IsValid()) {
	    fprintf(stderr, "test storage6 failed (step 15)\n");
	    return 6;
	}
    }

    return 0;
}

static int
test_storage7()
{
    /*
     * Test that an e4Storage file is initially empty.
     */
    
    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test storage7 failed (step 1)\n");
            return 7;
        }
        if (s.MarkerCount() != 0) {
            fprintf(stderr, "test storage7 failed (step 2, %d)\n",
                    s.MarkerCount());
            return 7;
        }
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test storage7 failed (step 3)\n");
	    return 7;
	}
    }

    return 0;
}

static int
test_storage8()
{
    e4_Marker t;
    
    /*
     * Test that autocommit off works.
     */

    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test storage8 failed (step 1)\n");
            return 8;
        }
        if (!s.GetMarker("mymarker", t)) {
            fprintf(stderr, "test storage8 failed (step 2)\n");
            return 8;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test storage8 failed (step 3)\n");
            return 8;
        }
        s.AutoCommit(false);
        if (s.AutoCommit() == true) {
            fprintf(stderr, "test storage8 failed (step 4)\n");
            return 8;
        }
    }
    if (t.IsValid()) {
        fprintf(stderr, "test storage8 failed (step 5)\n");
        return 8;
    }
    {
        e4_Storage s("foo.db", E4_METAKIT);
        if (!s.IsValid()) {
            fprintf(stderr, "test storage8 failed (step 6)\n");
            return 8;
        }
        if (s.MarkerCount() != 0) {
            fprintf(stderr, "test storage8 failed (step 7)\n");
            return 8;
        }
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test storage8 failed (step 8)\n");
	    return 8;
	}
    }

    return 0;
}

static int
test_storage9()
{
    /*
     * Test that we can have several e4Storages open at the same time.
     */

    {
        e4_Storage s1("foo1.db", E4_METAKIT);
        if (!s1.IsValid()) {
            fprintf(stderr, "test storage9 failed (step 1)\n");
            return 9;
        }
        e4_Storage s2("foo2.db", E4_METAKIT);
        if (!s2.IsValid()) {
            fprintf(stderr, "test storage9 failed (step 2)\n");
            return 9;
        }
        e4_Storage s3("foo3.db", E4_METAKIT);
        if (!s3.IsValid()) {
            fprintf(stderr, "test storage9 failed (step 3)\n");
            return 9;
        }
        e4_Storage s4("foo4.db", E4_METAKIT);
        if (!s4.IsValid()) {
            fprintf(stderr, "test storage9 failed (step 4)\n");
            return 9;
        }
        if ((s1 == s2) || (s1 == s3) || (s1 == s4)) {
            fprintf(stderr, "test storage9 failed (step 5)\n");
            return 9;
        }
	s1.Delete();
	s2.Delete();
	s3.Delete();
	s4.Delete();
    }

    return 0;
}

static int
test_storage10()
{
    e4_Marker t;

    /*
     * Test that creating stuff in one storage does not show up in the other.
     */

    {
        e4_Storage s1("foo1.db", E4_METAKIT);
        if (!s1.IsValid()) {
            fprintf(stderr, "test storage10 failed (step 1)\n");
            return 10;
        }
        e4_Storage s2("foo2.db", E4_METAKIT);
        if (!s2.IsValid()) {
            fprintf(stderr, "test storage10 failed (step 2)\n");
            return 10;
        }
        if ((s1.MarkerCount() != 0) || (s2.MarkerCount() != 0)) {
            fprintf(stderr, "test storage10 failed (step 3)\n");
            return 10;
        }
        if (!s1.GetMarker("mymarker", t)) {
            fprintf(stderr, "test storage10 failed (step 4)\n");
            return 10;
        }
        if ((s1.MarkerCount() != 1) || (s2.MarkerCount() != 0)) {
            fprintf(stderr, "test storage10 failed (step 5)\n");
            return 10;
        }
    }
    if (t.IsValid()) {
        fprintf(stderr, "test storage10 failed (step 6)\n");
        return 10;
    }
    {
	e4_Storage sd1("foo1.db", E4_METAKIT);
	e4_Storage sd2("foo2.db", E4_METAKIT);

	sd1.Delete();
	sd2.Delete();
    }
    {
        e4_Storage s1("foo1.db", E4_METAKIT);
        if (!s1.IsValid()) {
            fprintf(stderr, "test storage10 failed (step 7)\n");
            return 10;
        }
        if (s1.MarkerCount() != 0) {
            fprintf(stderr, "test storage10 failed (step 8)\n");
            return 10;
        }
        if (!s1.GetMarker("mymarker", t)) {
            fprintf(stderr, "test storage10 failed (step 9)\n");
            return 10;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test storage10 failed (step 10)\n");
            return 10;
        }
        e4_Storage s2("foo2.db", E4_METAKIT);
        if (!s2.IsValid()) {
            fprintf(stderr, "test storage10 failed (step 11)\n");
            return 10;
        }
        if (s1 == s2) {
            fprintf(stderr, "test storage10 failed (step 12)\n");
            return 10;
        }
        if ((s1.MarkerCount() != 1) || (s2.MarkerCount() != 0)) {
            fprintf(stderr, "test storage10 failed (step 13)\n");
            return 10;
        }
    }
    if (t.IsValid()) {
        fprintf(stderr, "test storage10 failed (step 14)\n");
        return 10;
    }
    {
	e4_Storage sd1("foo1.db", E4_METAKIT);
	e4_Storage sd2("foo2.db", E4_METAKIT);

	sd1.Delete();
	sd2.Delete();
    }

    return 0;
}

static int
test_storage11()
{
    e4_Marker t;

    /*
     * Test that creating stuff in one storage does not show up in the other.
     */

    {
        e4_Storage s1("foo1.db", E4_METAKIT);

        if (!s1.IsValid()) {
            fprintf(stderr, "test storage11 failed (step 1)\n");
            return 11;
        }
        if (!s1.GetMarker("mymarker", t)) {
            fprintf(stderr, "test storage11 failed (step 2)\n");
            return 11;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test storage11 failed (step 3)\n");
            return 11;
        }
        s1.Commit();
        
        e4_Storage s2("foo2.db", E4_METAKIT);
        if (!s2.IsValid()) {
            fprintf(stderr, "test storage11 failed (step 4)\n");
            return 11;
        }
        if (s2.MarkerCount() != 0) {
            fprintf(stderr, "test storage11 failed (step 5)\n");
            return 11;
        }
    }
    if (t.IsValid()) {
        fprintf(stderr, "test storage11 failed (step 6)\n");
        return 11;
    }
    {
	e4_Storage s3("foo1.db", E4_METAKIT);
	e4_Storage s4("foo2.db", E4_METAKIT);

	if (!s3.IsValid() || !s4.IsValid()) {
	    fprintf(stderr, "test storage11 failed (step 7)\n");
	    return 11;
	}
	if (s3.MarkerCount() != 1 || s4.MarkerCount() != 0) {
	    fprintf(stderr, "test storage11 failed (step 8)\n");
	    return 11;
	}
	s3.Delete();
	s4.Delete();
	if (s3.IsValid() || s4.IsValid()) {
	    fprintf(stderr, "test storage11 failed (step 9)\n");
	    return 11;
	}
    }

    return 0;
}

static int
test_storage12()
{
    e4_Marker t;

    /*
     * Test that creating stuff in one storage does not show up in the other.
     */

    {
        e4_Storage s1("foo1.db", E4_METAKIT);
	e4_Storage s2("foo2.db", E4_METAKIT);

        if (!s1.IsValid()) {
            fprintf(stderr, "test storage12 failed (step 1)\n");
            return 12;
        }
        if (!s1.GetMarker("mymarker", t)) {
            fprintf(stderr, "test storage12 failed (step 2)\n");
            return 12;
        }
        if (!t.IsValid()) {
            fprintf(stderr, "test storage12 failed (step 3)\n");
            return 12;
        }
	if (!s2.IsValid()) {
	    fprintf(stderr, "test storage12 failed (step 4)\n");
	    return 12;
	}
    }
    if (t.IsValid()) {
        fprintf(stderr, "test storage12 failed (step 5)\n");
        return 12;
    }
    {
        e4_Storage s3("foo2.db", E4_METAKIT);

        if (!s3.IsValid()) {
            fprintf(stderr, "test storage12 failed (step 6)\n");
            return 12;
        }
        if (s3.MarkerCount() != 0) {
            fprintf(stderr, "test storage12 failed (step 7)\n");
            return 12;
        }
	s3.Delete();
	if (s3.IsValid()) {
	    fprintf(stderr, "test storage12 failed (step 8)\n");
	    return 12;
	}
    }
    {
	e4_Storage s4("foo1.db", E4_METAKIT);

	if (!s4.IsValid()) {
	    fprintf(stderr, "test storage12 failed (step 9)\n");
	    return 12;
	}
	if (s4.MarkerCount() != 1) {
	    fprintf(stderr, "test storage12 failed (step 10)\n");
	    return 12;
	}
	s4.Delete();
	if (s4.IsValid()) {
	    fprintf(stderr, "test storage12 failed (step 11)\n");
	    return 12;
	}
    }

    return 0;
}

static int
test_storage13()
{
    e4_Marker t1, t2;

    /*
     * Test that if we get the same e4Marker, we get the same storage.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test storage13 failed (step 1)\n");
	    return 13;
	}
	if (!s.GetMarker("mymarker", t1)) {
	    fprintf(stderr, "test storage13 failed (step 2)\n");
	    return 13;
	}
	if (!t1.IsValid()) {
	    fprintf(stderr, "test storage13 failed (step 3)\n");
	    return 13;
	}
	if (!s.GetMarker("mymarker", t2)) {
	    fprintf(stderr, "test storage13 failed (step 4)\n");
	    return 13;
	}
	if (!t2.IsValid()) {
	    fprintf(stderr, "test storage13 failed (step 5)\n");
	    return 13;
	}
	if (t1 != t2) {
	    fprintf(stderr, "test storage13 failed (step 6)\n");
	    return 13;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test storage13 failed (step 7)\n");
	    return 13;
	}
    }
    if (t1.IsValid() || t2.IsValid()) {
	fprintf(stderr, "test storage13 failed (step 8)\n");
	return 13;
    }

    return 0;
}

static int
test_storage14()
{
    e4_Marker t1, t2, t3, t4;

    /*
     * Test that we can have several e4Markers in an e4Storage.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test storage14 failed (step 1)\n");
	    return 14;
	}
	if (!s.GetMarker("t1", t1)) {
	    fprintf(stderr, "test storage14 failed (step 2)\n");
	    return 14;
	}
	if (!t1.IsValid()) {
	    fprintf(stderr, "test storage14 failed (step 3)\n");
	    return 14;
	}
	if (!s.GetMarker("t2", t2)) {
	    fprintf(stderr, "test storage14 failed (step 4)\n");
	    return 14;
	}
	if (!t2.IsValid()) {
	    fprintf(stderr, "test storage14 failed (step 5)\n");
	    return 14;
	}
	if (!s.GetMarker("t3", t3)) {
	    fprintf(stderr, "test storage14 failed (step 6)\n");
	    return 14;
	}
	if (!t3.IsValid()) {
	    fprintf(stderr, "test storage14 failed (step 7)\n");
	    return 14;
	}
	if (!s.GetMarker("t4", t4)) {
	    fprintf(stderr, "test storage14 failed (step 8)\n");
	    return 14;
	}
	if (!t4.IsValid()) {
	    fprintf(stderr, "test storage14 failed (step 9)\n");
	    return 14;
	}
	if (s.MarkerCount() != 4) {
	    fprintf(stderr, "test storage14 failed (step 10)\n");
	    return 14;
	}
	if ((t1 == t2) || (t1 == t3) || (t1 == t4)) {
	    fprintf(stderr, "test storage14 failed (step 11)\n");
	    return 14;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test storage14 failed (step 12)\n");
	    return 14;
	}
    }
    if (t1.IsValid() || t2.IsValid() || t3.IsValid() || t3.IsValid()) {
	fprintf(stderr, "test storage14 failed (step 13)\n");
	return 14;
    }

    return 0;
}

static int
test_storage15()
{
    const char *nm;

    /*
     * Test GetName() and GetDriver().
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test storage15 failed (step 1)\n");
	    return 15;
	}
	if (strcmp(s.GetDriver(), E4_METAKIT) != 0) {
	    fprintf(stderr, "test storage15 failed (step 2)\n");
	    return 15;
	}
	nm = s.GetName();
	if (nm == NULL) {
	    fprintf(stderr, "test storage15 failed (step 3)\n");
	    return 15;
	}
	if (strcmp(nm, "foo.db") != 0) {
	    fprintf(stderr, "test storage15 failed (step 3)\n");
	    return 15;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test storage15 failed (step 4)\n");
	    return 15;
	}
    }

    return 0;
}

static int
test_storage16()
{
    e4_Marker markers[TEST_ARRAY_SIZE];
    char nm[32];
    int i;

    /*
     * Test that we can have a large number of markers in a storage.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test storage16 failed (step 1)\n");
	    return 16;
	}
	for (i = 0; i < TEST_ARRAY_SIZE; i++) {
	    sprintf(nm, "t%d", i);
	    if (!s.GetMarker((const char *) nm, markers[i])) {
	        fprintf(stderr, "test storage16 failed (step 2.1, t%d)\n", i);
		return 16;
	    }
	    if (!markers[i].IsValid()) {
		fprintf(stderr, "test storage16 failed (step 2.2, t%d)\n", i);
		return 16;
	    }
	}
	if (s.MarkerCount() != TEST_ARRAY_SIZE) {
	    fprintf(stderr, "test storage16 failed (step 3)\n");
	    return 16;
	}

        /*
         * Auto-commit is on by default so the following step should
         * commit the markers to the stable storage.
         */
    }
    for (i = 0; i < TEST_ARRAY_SIZE; i++) {
	if (markers[i].IsValid()) {
	    fprintf(stderr, "test storage16 failed (step 4, t%d)\n", i);
	    return 16;
	}
    }
    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test storage16 failed (step 4)\n");
	    return 16;
	}
	if (s.MarkerCount() != TEST_ARRAY_SIZE) {
	    fprintf(stderr, "test storage16 failed (step 5)\n");
	    return 16;
	}
	for (i = 0; i < TEST_ARRAY_SIZE; i++) {
	    sprintf(nm, "t%d", i);
	    if (!s.GetMarker((const char *) nm, markers[i])) {
		fprintf(stderr, "test storage16 failed (step 6.1, t%d)\n", i);
		return 16;
	    }
	    if (!markers[i].IsValid()) {
		fprintf(stderr, "test storage16 failed (step 6.2, t%d)\n", i);
		return 16;
	    }
	}
	if (s.MarkerCount() != TEST_ARRAY_SIZE) {
	    fprintf(stderr, "test storage16 failed (step 7)\n");
	    return 16;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test storage16 failed (step 8)\n");
	    return 16;
	}
    }
    for (i = 0; i < TEST_ARRAY_SIZE; i++) {
	if (markers[i].IsValid()) {
	    fprintf(stderr, "test storage16 failed (step 9.%d)\n", i);
	    return 16;
	}
    }

    return 0;
}

static int
test_storage17()
{
    e4_Marker t;

    /*
     * Test that deleting the instance of the e4_StorageImpl invalidates any
     * instances of e4_MarkerImpl for that storage.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test storage17 failed (step 1)\n");
	    return 17;
	}
	if (!s.GetMarker("mymarker", t)) {
	    fprintf(stderr, "test storage17 failed (step 2)\n");
	    return 17;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test storage17 failed (step 3)\n");
	    return 17;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test storage17 failed (step 4)\n");
	    return 17;
	}
	if (t.IsValid()) {
	    fprintf(stderr, "test storage17 failed (step 5)\n");
	    return 17;
	}
    }
    if (t.IsValid()) {
	fprintf(stderr, "test storage17 failed (step 4)\n");
	return 17;
    }

    return 0;
}

static int
test_storage18()
{
    /*
     * Test e4_StorageImpl::IsValid under various conditions.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);
	if (!s.IsValid()) {
	    fprintf(stderr, "test storage18 failed (step 1)\n");
	    return 18;
	}
	{
	    e4_Marker t;

	    if (!s.GetMarker("mymarker", t)) {
		fprintf(stderr, "test storage18 failed (step 2)\n");
		return 18;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test storage18 failed (step 4)\n");
		return 18;
	    }
	}
	if (!s.IsValid()) {
	    fprintf(stderr, "test storage18 failed (step 5)\n");
	    return 18;
	}
	{
	    e4_Marker t;

	    if (!s.GetMarker("mymarker", t)) {
		fprintf(stderr, "test storage18 failed (step 6)\n");
		return 18;
	    }
	    if (!t.IsValid()) {
		fprintf(stderr, "test storage18 failed (step 7)\n");
		return 18;
	    }
	    if (!s.IsValid()) {
		fprintf(stderr, "test storage18 failed (step 8)\n");
		return 18;
	    }
	    t.Delete();
	    if (!s.IsValid()) {
		fprintf(stderr, "test storage18 failed (step 8)\n");
		return 18;
	    }
	    if (t.IsValid()) {
		fprintf(stderr, "test storage18 failed (step 8.a)\n");
		return 18;
	    }
	    if (s.MarkerCount() != 0) {
		fprintf(stderr, "test storage18 failed (step 9), %d\n",
			s.MarkerCount());
		return 18;
	    }

	    e4_Marker t1;

	    if (!s.GetMarker("mymarker", t1)) {
		fprintf(stderr, "test storage18 failed (step 10)\n");
		return 18;
	    }
	    if (!t1.IsValid()) {
		fprintf(stderr, "test storage18 failed (step 11)\n");
		return 18;
	    }
	    s.Delete();
	    if (s.IsValid()) {
		fprintf(stderr, "test storage18 failed (step 12)\n");
		return 18;
	    }
	    if (t1.IsValid()) {
		fprintf(stderr, "test storage18 failed (step 13)\n");
		return 18;
	    }
	}
	if (s.IsValid()) {
	    fprintf(stderr, "test storage18 failed (step 14)\n");
	    return 18;
	}
    }

    return 0;
}

static int
test_storage19()
{
    /*
     * Test that the same underlying storage is used if it is opened
     * more than once for the same file and driver name. Test that it returns
     * different entities for different file names.
     */

    {
	e4_Storage s1("foo1.db", E4_METAKIT);
	if (!s1.IsValid()) {
	    fprintf(stderr, "test storage19 failed (step 1)\n");
	    return 19;
	}
	e4_Storage s2("foo2.db", E4_METAKIT);
	if (!s2.IsValid()) {
	    fprintf(stderr, "test storage19 failed (step 2)\n");
	    return 19;
	}
	if (s1 == s2) {
	    fprintf(stderr, "test storage19 failed (step 3)\n");
	    return 19;
	}

	/*
	 * The s1 and s2 variables go out of scope without Deleteing
	 * the underlying storages.
	 */
    }
    {
	e4_Storage s3("foo1.db", E4_METAKIT);
	if (!s3.IsValid()) {
	    fprintf(stderr, "test storage19 failed (step 4)\n");
	    return 19;
	}
	e4_Storage s4("foo1.db", E4_METAKIT);
	if (!s4.IsValid()) {
	    fprintf(stderr, "test storage19 failed (step 5)\n");
	    return 19;
	}
	if (s3 != s4) {
	    fprintf(stderr, "test storage19 failed (step 6)\n");
	    return 19;
	}
	s3.Delete();
	if (s3.IsValid() || s4.IsValid()) {
	    fprintf(stderr, "test storage19 failed (step 7)\n");
	    return 19;
	}
	e4_Storage s5("foo2.db", E4_METAKIT);
	if (!s5.IsValid()) {
	    fprintf(stderr, "test storage19 failed (step 8)\n");
	    return 19;
	}
	s5.Delete();
	if (s5.IsValid()) {
	    fprintf(stderr, "test storage19 failed (step 9)\n");
	    return 19;
	}
    }

    return 0;
}

static int
test_storage20()
{
    e4_Storage *sp;

    {
	e4_Storage s;

	/*
	 * Try new-style construction.
	 */

	sp = new e4_Storage("foo.db", E4_METAKIT);
	s = *sp;
	delete sp;

	if (!s.IsValid()) {
	    fprintf(stderr, "test storage20 failed (step 1)\n");
	    return 20;
	}
	sp = new e4_Storage("foo.db", E4_METAKIT);
	if (!sp->IsValid()) {
	    fprintf(stderr, "test storage20 failed (step 2)\n");
	    return 20;
	}
	if (*sp != s) {
	    fprintf(stderr, "test storage20 failed (step 3)\n");
	    return 20;
	}
	delete sp;
	if (!s.IsValid()) {
	    fprintf(stderr, "test storage20 failed (step 4)\n");
	    return 20;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test storage20 failed (step 5)\n");
	    return 20;
	}
    }

    return 0;
}

static int
test_storage21()
{
    e4_Storage *sp;

    /*
     * Test operator==, operator!= and IsValid.
     */

    sp = new e4_Storage("foo.db", E4_METAKIT);
    if (!sp->IsValid()) {
	fprintf(stderr, "test storage21 failed (step 1)\n");
	return 21;
    }
    {
	e4_Storage s1("foo.db", E4_METAKIT);
	if (!s1.IsValid()) {
	    fprintf(stderr, "test storage21 failed (step 2)\n");
	    return 21;
	}
	if (s1 != *sp) {
	    fprintf(stderr, "test storage21 failed (step 3)\n");
	    return 21;
	}
	if (s1 == *sp) {
	} else {
	    fprintf(stderr, "test storage21 failed (step 4)\n");
	    return 21;
	}
	e4_Storage s2;
	if (s2.IsValid()) {
	    fprintf(stderr, "test storage21 failed (step 5)\n");
	    return 21;
	}
	e4_Storage s3;
	if (s3.IsValid()) {
	    fprintf(stderr, "test storage21 failed (step 6)\n");
	    return 21;
	}
	if (s3 == s2) {
	} else {
	    fprintf(stderr, "test storage21 failed (step 7)\n");
	    return 21;
	}
	if ((s3 != s2) || (s2 != s3)) {
	    fprintf(stderr, "test storage21 failed (step 8)\n");
	    return 21;
	}
    }
    if (!sp->IsValid()) {
	fprintf(stderr, "test storage21 failed (step 9)\n");
	return 21;
    }
    {
	e4_Storage s5("foo.db", E4_METAKIT);
	if ((!s5.IsValid()) || (!sp->IsValid())) {
	    fprintf(stderr, "test storage21 failed (step 10)\n");
	    return 21;
	}
	delete sp;
	if (!s5.IsValid()) {
	    fprintf(stderr, "test storage21 failed (step 11)\n");
	    return 21;
	}
	s5.Delete();
	if (s5.IsValid()) {
	    fprintf(stderr, "test storage21 failed (step 12)\n");
	    return 21;
	}
    }

    return 0;
}

static int
test_storage22()
{
    /*
     * Test invalidStorage.
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test storage22 failed (step 1)\n");
	    return 22;
	}
	s.Delete();
	if (s.IsValid()) {
	    fprintf(stderr, "test storage22 failed (step 2)\n");
	    return 22;
	}
	s = invalidStorage;
	if (s.IsValid()) {
	    fprintf(stderr, "test storage22 failed (step 3)\n");
	    return 22;
	}
    }

    return 0;
}

static int
test_storage23()
{
    int i, rank;
    char buf[32];
    e4_Marker t;
    e4_Node n;
    e4_Vertex f;

    /*
     * Test e4_Storage::MakeEmpty() and e4_Storage::IsEmpty().
     */

    {
	e4_Storage s("foo.db", E4_METAKIT);

	if (!s.IsValid()) {
	    fprintf(stderr, "test storage23 failed (step 1)\n");
	    return 23;
	}
	if (!s.IsEmpty()) {
	    fprintf(stderr, "test storage23 failed (step 2)\n");
	    return 23;
	}
	for (i = 0; i < TEST_ARRAY_SIZE; i++) {
	    sprintf(buf, "marker%d", i);
	    if (!s.GetMarker(buf, t)) {
		fprintf(stderr, "test storage23 failed (step 3.%d)\n", i);
		return 23;
	    }
	}
	if (s.IsEmpty()) {
	    fprintf(stderr, "test storage23 failed (step 4)\n");
	    return 23;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test storage23 failed (step 5)\n");
	    return 23;
	}
	if (!s.MakeEmpty()) {
	    fprintf(stderr, "test storage23 failed (step 6)\n");
	    return 23;
	}
	if (!s.IsEmpty() || !s.IsValid() || t.IsValid()) {
	    fprintf(stderr, "test storage23 failed (step 7)\n");
	    return 23;
	}
	for (i = 0; i < TEST_ARRAY_SIZE; i++) {
	    sprintf(buf, "marker%d", i);
	    if (!s.GetMarker(buf, t)) {
		fprintf(stderr, "test storage23 failed (step 8.%d)\n", i);
		return 23;
	    }
	}
	if (s.IsEmpty()) {
	    fprintf(stderr, "test storage23 failed (step 9)\n");
	    return 23;
	}
	if (!t.IsValid()) {
	    fprintf(stderr, "test storage23 failed (step 10)\n");
	    return 23;
	}
	if (!t.GetMarkedNode(n)) {
	    fprintf(stderr, "test storage23 failed (step 11)\n");
	    return 23;
	}
	if (!n.AddVertexRef("f", E4_IOFIRST, rank, 23, f)) {
	    fprintf(stderr, "test storage23 failed (step 12)\n");
	    return 23;
	}
	if (!t.IsValid() || !n.IsValid() || !f.IsValid()) {
	    fprintf(stderr, "test storage23 failed (step 13)\n");
	    return 23;
	}
	if (!s.MakeEmpty()) {
	    fprintf(stderr, "test storage23 failed (step 14)\n");
	    return 23;
	}
	if (!s.IsEmpty() || !s.IsValid() || t.IsValid() || n.IsValid() ||
	    f.IsValid()) {
	    fprintf(stderr, "test storage23 failed (step 15)\n");
	    return 23;
	}
	s.Delete();
	if (s.IsValid() || t.IsValid()) {
	    fprintf(stderr, "test storage23 failed (step 16)\n");
	    return 23;
	}
    }

    return 0;
}

int
test_storage()
{
    int result;
    
    fprintf(stderr, "Running e4_Storage tests: ");

    result = test_storage1();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage2();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage3();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage4();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage5();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage6();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage7();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage8();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage9();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage10();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage11();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage12();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage13();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage14();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage15();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage16();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage17();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage18();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage19();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage20();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage21();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage22();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".");
    result = test_storage23();
    if (result != 0) {
        return result;
    }
    fprintf(stderr, ".\n");
    
    return 0;
}
